SorhaQuestLog = LibStub("AceAddon-3.0"):NewAddon("SorhaQuestLog", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0", "LibSink-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SorhaQuestLog")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local LSM = LibStub("LibSharedMedia-3.0")

local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
local SQLBroker = nil

-- local LBF = LibStub("LibButtonFacade", true)
local LBF = nil
local LBFGroup = nil

-- Used to stop the need to use self.db all the time
local db
local dbChar

-- Just bringing some frequently used global functions into local scope
local pairs = pairs
local tinsert = tinsert
local tContains = tContains
local tremove = tremove
local select = select
local format = format
local SecondsToTime = SecondsToTime
local GetTime = GetTime


-- Register framd font
LSM:Register("font", "framd", [[Interface\AddOns\SorhaQuestLog\Fonts\framd.ttf]])
LSM:Register("sound", "Peon Ready", [[Sound\Creature\Peon\PeonReady1.wav]])
LSM:Register("sound", "More work?", [[Sound\Creature\Peasant\PeasantWhat3.wav]])
LSM:Register("sound", "Work complete!", [[Sound\Creature\Peon\PeonBuildingComplete1.wav]])
LSM:Register("sound", "Quest Added", [[Sound\Interface\iQuestActivate.wav]])
LSM:Register("sound", "Quest Completed", [[Sound\Interface\iQuestComplete.wav]])
LSM:Register("sound", "Quest Failed", [[Sound\Interface\igQuestFailed.wav]])

-- Binding globals for bindings.xml
BINDING_HEADER_SORHAQUESTLOG = "Sorha Quest Log";
BINDING_NAME_TOGGLE_SORHAQUESTLOG = L["Toggle Minion"]

-- Frames
local fraQuestMinionAnchor = nil
local fraQuestMinionBorder = nil
local fraQuestTimerAnchor = nil
local fraRemoteQuestsAnchor = nil
local fraAchievementMinionAnchor = nil

-- Frame Name Prefixes
local strButtonPrefix = "SorhaQuestLogButton"
local strItemButtonPrefix = "SorhaQuestLogItemButton"
local strRemoteButtonPrefix = "SorhaQuestLogRemoteButton"
local strStatusBarPrefix = "SorhaQuestLogStatusBar"

-- Frame Counts
local intNumberOfLogButtons = 0
local intNumberOfItemButtons = 0
local intNumberOfRemoteButtons = 0
local intNumberOfStatusBars = 0

-- Number Stores
local intItemButtonSize = 26
local intRemoteQuestButtonWidth = 224
local intRemoteQuestButtonHeight = 72
local intLastCountRemoteQuests = 0
local intNumAchievementTimers = 0

-- Tables
local tblAchievementTimers = {}
local tblBagsToCheck = {}
local tblHaveQuestItems = {}

-- Frame Caches
local tblItemButtonCache =  {}
local tblLogButtonCache = {}
local tblRemoteButtonCache = {}
local tblStatusBarsCache = {}

-- Frame Stores
local tblQuestMinionLogButtons = {}
local tblQuestTimerButtons = {}
local tblRemoteQuestsButtons = {}
local tblAchievementMinionButtons = {}


-- Colour strings
local strInfoColour = "|cffffffff"
local strQuestMinionTitleColour = "|cffffffff"
local strAchievementMinionTitleColour = "|cffffffff"
local strHeaderColour = "|cffffffff"
local strQuestStatusFailed = "|cffffffff"
local strQuestStatusDone = "|cffffffff"
local strQuestStatusGoto = "|cffffffff"
local strQuestLevelColour = "|cffffffff"
local strQuestTitleColour = "|cffffffff"
local strObjectiveTitleColour = "|cffffffff"
local strObjectiveStatusColour = "|cffffffff"
local strObjective00to24Colour = "|cffffffff"
local strObjective25to49Colour = "|cffffffff"
local strObjective50to74Colour = "|cffffffff"
local strObjective75to99Colour = "|cffffffff"
local strObjective100Colour = "|cffffffff"
local strObjectiveDescriptionColour = "|cffffffff"
local strAchievementTitleColour = "|cffffffff"
local strAchievementObjectiveColour = "|cffffffff"

-- Booleans
local blnQuestMinionInitialized = false -- Quest Minion CreateLayout completed
local blnTimerInitialized = false -- Quest Timers CreateLayout completed
local blnRemoteQuestsMinionInitialized = false -- Remote Quests CreateLayout completed
local blnAchievementMinionInitialized = false -- Achievement Minion CreateLayout completed

local blnIsUpdating = false -- Minion is updating
local blnRemoteQuestsMinionIsUpdating = false
local blnAchievementMinionIsUpdating = false
local blnBagCheckUpdating = false

local blnWasAClick = false -- Was QUEST_LOG_UPDATE called by a click on a header etc
local blnIgnoreUpdateEvents = false -- Ignores QLU events, used when making large scale collapse/expands to log
local blnFirstUpdate = true -- Was first update of quest log, no old data will be on store yet
local blnHaveRegisteredBagUpdate = false
local blnFirstBagCheck = true
local blnHaveRefreshedHeaders = false

-- Strings used to store current location for auto collapse/expand
local strZone = ""
local strSubZone = ""

-- Questlog data stores
local curQuestInfo = nil
local oldQuestInfo = nil

-- Time stores
local intLastOnUpdateTime = 0 -- Time OnUpdate event was last used
local intLastAchievementTimerUpdateTime = 0
local intTimeOfLastSound = 0 -- Time last sound played


-- Item button parts to use with LBF
local tblItemButtonLBFData = {
	Cooldown = false,
}

-- Tables for select menus in options
local dicRepLevels = {
	["Hated"] = {["MTitle"] = FACTION_STANDING_LABEL1, ["FTitle"] = FACTION_STANDING_LABEL1_FEMALE, ["Value"] = 1.00},
	["Hostile"] = {["MTitle"] = FACTION_STANDING_LABEL2, ["FTitle"] = FACTION_STANDING_LABEL2_FEMALE, ["Value"] = 1.30},
	["Unfriendly"] = {["MTitle"] = FACTION_STANDING_LABEL3, ["FTitle"] = FACTION_STANDING_LABEL3_FEMALE, ["Value"] = 1.70},
	["Neutral"] = {["MTitle"] = FACTION_STANDING_LABEL4, ["FTitle"] = FACTION_STANDING_LABEL4_FEMALE, ["Value"] = 2.20},
	["Friendly"] = {["MTitle"] = FACTION_STANDING_LABEL5, ["FTitle"] = FACTION_STANDING_LABEL5_FEMALE, ["Value"] = 2.85},
	["Honored"] = {["MTitle"] = FACTION_STANDING_LABEL6, ["FTitle"] = FACTION_STANDING_LABEL6_FEMALE, ["Value"] = 3.71},
	["Revered"] = {["MTitle"] = FACTION_STANDING_LABEL7, ["FTitle"] = FACTION_STANDING_LABEL7_FEMALE, ["Value"] = 4.82},
	["Exalted"] = {["MTitle"] = FACTION_STANDING_LABEL8, ["FTitle"] = FACTION_STANDING_LABEL8_FEMALE, ["Value"] = 6.27},
}

local dicOutlines = {
	[""] = L["None"],
	["OUTLINE"] = L["Outline"],
	["THICKOUTLINE"] = L["Thick Outline"],
}

local dicQuestTitleColourOptions = {
	["Custom"] = L["Custom"],
	["Level"] = L["Level"],
	["Completion"] = L["Completion"],
	["Done/Undone"] = L["Done/Undone"],
}

local dicObjectiveColourOptions = {
	["Custom"] = L["Custom"],
	["Done/Undone"] = L["Done/Undone"],
	["Completion"] = L["Completion"],
}

local dicNotificationColourOptions = {
	["Custom"] = L["Custom"],
	["Completion"] = L["Completion"],
}

local dicAutoHideOptions = {
	["Do Nothing"] = L["Do Nothing"],
	["Hide"] = L["Hide"],
	["Show"] = L["Show"],
}

-- Tables used for quest tags (Group, Elite etc)
local dicQuestTags = {
	[ELITE] = "+",
	[GROUP] = "g",
	[PVP] = "p",
	[RAID] = "r",
	[LFG_TYPE_DUNGEON] = "d",
	[PLAYER_DIFFICULTY2] = "d+",
	["Daily"] = "*",
}

local dicLongQuestTags = {
	[ELITE] = ELITE,
	[GROUP] = GROUP,
	[PVP] = PVP,
	[RAID] = RAID,
	[LFG_TYPE_DUNGEON] = LFG_TYPE_DUNGEON,
	[PLAYER_DIFFICULTY2] = PLAYER_DIFFICULTY2,
	["Daily"] = L["Daily"],
}

local dicQuestTagsLength = {
	["None"] = L["None"],
	["Short"] = L["Short"],
	["Full"] = L["Full"],
}

-- Table used for matching quest completion outputs
local function getPattern(strPattern)
	strPattern = string.gsub(strPattern, "%(", "%%%1")
	strPattern = string.gsub(strPattern, "%)", "%%%1")
	strPattern = string.gsub(strPattern, "%%%d?$?.", "(.+)")
	return format("^%s$", strPattern)
end

local tblQuestMatchs = {
	["Found"] = getPattern(ERR_QUEST_ADD_FOUND_SII),
	["Item"] = getPattern(ERR_QUEST_ADD_ITEM_SII),
	["Kill"] = getPattern(ERR_QUEST_ADD_KILL_SII),
	["PKill"] = getPattern(ERR_QUEST_ADD_PLAYER_KILL_SII),
	["ObjectiveComplete"] = getPattern(ERR_QUEST_OBJECTIVE_COMPLETE_S),
	["QuestComplete"] = getPattern(ERR_QUEST_COMPLETE_S),
	["QuestFailed"] = getPattern(ERR_QUEST_FAILED_S),
}

-- Addon Setup
local defaults = {
	profile = {
		Main = {
			HideAll = false,
			HideSorhaQuestLog = false,
			HideBlizzardTracker = true,
			ShowQuestTimerFrame = true,
			ShowRemoteQuestsMinionFrame = true,
			ShowAchievementMinionFrame = true,
			ShowHelpTooltips = true,
			ShowAnchors = false,
		},
		Minion = {
			MinionWindowLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			MinionWindowScale = 1,
			MinionWindowLocked = false,			
			MinionWidth = 220,
			ShowItemButtons = true,
			ShowNumberOfQuests = true,
			ShowNumberOfDailyQuests = false,
			MoveItemsAndTooltipsRight = false,
			IndentItemButtons = false,
			IndentItemButtonQuestsOnly = false,
			CollapseToLeft = false,
			ItemButtonScale = 0.8,
			HideItemButtonsForCompletedQuests = false,
			HideMinionIfNoTrackedQuests = false,
			ConfirmQuestAbandons = true,
			GrowUpwards = false,
		},
		Fonts = {
			-- Quest minion title text font
			QuestMinionTitleFontSize = 14,
			QuestMinionTitleFont = "framd",
			QuestMinionTitleFontOutline = "",
			QuestMinionTitleFontShadowed = true,	
			
			-- Zone header font
			HeaderFontSize = 11,
			HeaderFont = "framd",
			HeaderFontOutline = "",
			HeaderFontShadowed = true,
						
			-- Quest title font
			QuestFontSize = 11,
			QuestFont = "framd",
			QuestFontOutline = "",
			QuestFontShadowed = true,
			
			-- Objective text font
			ObjectiveFontSize = 11,
			ObjectiveFont = "framd",
			ObjectiveFontOutline = "",
			ObjectiveFontShadowed = true,

			-- Achievement minion title text font
			AchievementMinionTitleFontSize = 14,
			AchievementMinionTitleFont = "framd",
			AchievementMinionTitleFontOutline = "",
			AchievementMinionTitleFontShadowed = true,			
			
			-- Achievement title text font
			AchievementTitleFontSize = 11,
			AchievementTitleFont = "framd",
			AchievementTitleFontOutline = "",
			AchievementTitleFontShadowed = true,
			
			-- Achievement task text font			
			AchievementObjectiveFontSize = 11,
			AchievementObjectiveFont = "framd",
			AchievementObjectiveFontOutline = "",
			AchievementObjectiveFontShadowed = true,
		},
		ZonesAndQuests = {
			QuestLevelColouringSetting = "Level",
			QuestTitleColouringSetting = "Level",
			ObjectiveTitleColouringSetting = "Custom",
			ObjectiveStatusColouringSetting = "Completion",
			QuestTagsLength = "Short",
			ShowQuestLevels = true,
			HideCompletedObjectives = false,
			HideCompletedQuests = false,
			ShowDescWhenNoObjectives = false,
			AllowHiddenQuests = false,
			CollapseOnLeave = false,
			ExpandOnEnter = false,
			HideZoneHeaders = false,
			QuestHeadersHideWhenEmpty = false,
			ShowHiddenCountOnZones = false,
			QuestTitleIndent = 5,
			ObjectivesIndent = 0,
			QuestClicksOpenFullQuestLog = true,
		},
		Sounds = {
			UseQuestDoneSound = false,
			UseObjectiveDoneSound = false,
		},
		Colours = {
			InfoColour = {r = 0, g = 1, b = 0.5, a = 1},
			QuestMinionTitleColour = {r = 0, g = 1, b = 0.5, a = 1},
			AchievementMinionTitleColour = {r = 0, g = 1, b = 0.5, a = 1},
			HeaderColour = {r = 0, g = 0.6, b = 1, a = 1},
			QuestStatusFailedColour = {r = 1, g = 1, b = 1, a = 1},
			QuestStatusDoneColour = {r = 1, g = 1, b = 1, a = 1},
			QuestStatusGotoColour = {r = 1, g = 1, b = 1, a = 1},
			QuestLevelColour = {r = 1, g = 1, b = 1, a = 1},
			QuestTitleColour = {r = 1, g = 1, b = 1, a = 1},
			ObjectiveTitleColour = {r = 1, g = 1, b = 1, a = 1},
			ObjectiveStatusColour = {r = 1, g = 1, b = 1, a = 1},
			Objective00PlusColour = {r = 1, g = 0, b = 0, a = 1},
			Objective25PlusColour = {r = 1, g = 0.3, b = 0, a = 1},
			Objective50PlusColour  = {r = 1, g = 0.6, b = 0, a = 1},
			Objective75PlusColour = {r = 1, g = 0.95, b = 0, a = 1},
			ObjectiveDoneColour = {r = 0, g = 1, b = 0, a = 1},
			ObjectiveDescColour = {r = 0.5, g = 0.5, b = 0.5, a = 0.5},
			MinionBackGroundColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
			MinionBorderColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
			AchievementMinionBackGroundColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
			AchievementMinionBorderColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
			NotificationsColour = {r = 0, g = 1, b = 0, a = 1},
			AchievementTitleColour = {r = 0, g = 1, b = 0, a = 1},
			AchievementObjectiveColour = {r = 1, g = 1, b = 1, a = 1},
			AchievementStatusBarFillColour = {r = 0, g = 1, b = 0, a = 1},
			AchievementStatusBarBackColour = {r = 0, g = 0, b = 0, a = 1},
		},		
		Timer = {
			TimerWindowLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			TimerWindowScale = 1,
			TimerWindowLocked = false,
			ShowTitle = true,
			HideTitleWhenNoTimers = false,
			ShowBorder = false,
		},
		RemoteQuests = {
			MinionLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			MinionScale = 1,
			MinionLocked = false,
			ShowTitle = true,
			HideTitleWhenNoRemoteQuests = false,
			ShowBorder = false,
		},
		AchievementMinion = {
			MinionLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			MinionScale = 1,
			MinionLocked = false,
			MinionWidth = 220,
			MinionCollapseToLeft = false,
			MoveTooltipsRight = false,
			ShowTitle = true,
			HideTitleWhenNoTracking = false,
			UseStatusBars = false,
			StatusBarTexture = LSM.DefaultMedia.statusbar,
			MaxTasksEachAchievement = 0,
			GrowUpwards = false,
		},
		Notifications = {
			LibSinkColourSetting = "Custom",
			SuppressBlizzardNotifications = false,
			LibSinkObjectiveNotifications = false,
			DisplayQuestOnObjectiveNotifications = true,
			ShowQuestCompletesAndFails = false,
			QuestDoneSound = "None",
			ObjectiveDoneSound = "None",
			QuestItemFoundSound = "None",
			ShowMessageOnPickingUpQuestItem = false,
		},
		AutoHide = {
			OnInstance = "Do Nothing",
			OnRaid = "Do Nothing",
			OnArena = "Do Nothing",
			OnBattleground = "Do Nothing",
			OnNormal = "Do Nothing",
			OnEnterCombat = "Do Nothing",
			OnLeaveCombat = "Do Nothing",
		},
		ButtonSkins = {
			SkinID = "DreamLayout",
			Gloss = 0,
			Backdrop = false,
			Colors = {},
			SQLItemButtons = {
				SkinID = "DreamLayout",
				Gloss = 0,
				Backdrop = false,
				Colors = {},
			},
		},
	},
	char = {
		ZoneIsAllHiddenQuests = {},
		ZoneIsCollapsed = {},
		ZonesAndQuests = {
			ShowAllQuests = false,
		}
	},
}

local function getOptions()
	if not SorhaQuestLog.options then
		SorhaQuestLog.options = {
			type = "group",
			name = "SorhaQuestLog ".. "v1.2",
			handler = SorhaQuestLog,
			args = {
				General = {
					name = L["General Options"],
					type = "group",
					order = 1,
					args = {
						HideAll = {
							name = L["Hide All Sorha Quest Log"],
							desc = L["Hides all of Sorha Quest Log"],
							type = "toggle",
							width = "full",
							get = function() return db.Main.HideAll end,
							set = function()
								db.Main.HideAll = not db.Main.HideAll
								SorhaQuestLog:doAnchorUpdates(false)
							end,
							order = 1,
						},
						ShowAnchorsToggle = {
							name = L["Show minion anchors"],
							desc = L["Shows the anchors for minions to make them easier to move"],
							type = "toggle",
							width = "full",
							get = function() return db.Main.ShowAnchors end,
							set = function()
								db.Main.ShowAnchors = not db.Main.ShowAnchors
								SorhaQuestLog:doAnchorUpdates(false)
							end,
							order = 2,
						},
						ShowHelpTooltipsToggle = {
							name = L["Show helpful tooltips"],
							desc = L["Shows helpful tooltips for people learning the addon"],
							type = "toggle",
							width = "full",
							get = function() return db.Main.ShowHelpTooltips end,
							set = function()
								db.Main.ShowHelpTooltips = not db.Main.ShowHelpTooltips
							end,
							order = 3,
						},	
						HeaderBlizzardFrameSettingsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 50,
						},
						HeaderBlizzardFrameSettings = {
							name = L["Blizzard Frame Settings"],
							type = "header",
							order = 51,
						},
						HideBlizzardTrackerToggle = {
							name = L["Hide the default quest tracker"],
							desc = L["Hides blizzards quest tracker.. which is also used for Achievement tracking"],
							type = "toggle",
							width = "full",
							get = function() return db.Main.HideBlizzardTracker end,
							set = function()
								db.Main.HideBlizzardTracker = not db.Main.HideBlizzardTracker
								SorhaQuestLog:HandleBlizzardTracker()
							end,
							order = 52,
						},	
						HeaderAutoHideSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 100,
						},
						HeaderAutoHide = {
							name = L["Auto Hide/Showing"],
							type = "header",
							order = 101,
						},
						OnInstanceAutoHideSelect = {
							name = L["When entering a Dungeon"],
							desc = L["What to do when entering a Dungeon"],
							type = "select",
							order = 102,
							values = dicAutoHideOptions,
							get = function() return db.AutoHide.OnInstance end,
							set = function(info, value)
								db.AutoHide.OnInstance = value
							end,
						},
						OnRaidAutoHideSelect = {
							name = L["When entering a Raid"],
							desc = L["What to do when entering a Raid"],
							type = "select",
							order = 103,
							values = dicAutoHideOptions,
							get = function() return db.AutoHide.OnRaid end,
							set = function(info, value)
								db.AutoHide.OnRaid = value
							end,
						},
						OnArenaAutoHideSelect = {
							name = L["When entering an Arena"],
							desc = L["What to do when entering an Arena"],
							type = "select",
							order = 104,
							values = dicAutoHideOptions,
							get = function() return db.AutoHide.OnArena end,
							set = function(info, value)
								db.AutoHide.OnArena = value
							end,
						},
						OnBattlegroundAutoHideSelect = {
							name = L["When entering a Battleground"],
							desc = L["What to do when entering a Battleground"],
							type = "select",
							order = 105,
							values = dicAutoHideOptions,
							get = function() return db.AutoHide.OnBattleground end,
							set = function(info, value)
								db.AutoHide.OnBattleground = value
							end,
						},
						OnNormalAutoHideSelect = {
							name = L["When entering normal world"],
							desc = L["What to do when entering an area that is not an Arena, Battleground, Dungeon or Raid"],
							type = "select",
							order = 106,
							values = dicAutoHideOptions,
							get = function() return db.AutoHide.OnNormal end,
							set = function(info, value)
								db.AutoHide.OnNormal = value
							end,
						},
						OnEnterCombatAutoHideSelect = {
							name = L["When entering combat"],
							desc = L["What to do when entering combat"],
							type = "select",
							order = 107,
							values = dicAutoHideOptions,
							get = function() return db.AutoHide.OnEnterCombat end,
							set = function(info, value)
								db.AutoHide.OnEnterCombat = value
							end,
						},
						OnLeaveCombatAutoHideSelect = {
							name = L["When leaving combat"],
							desc = L["What to do when leaving combat"],
							type = "select",
							order = 108,
							values = dicAutoHideOptions,
							get = function() return db.AutoHide.OnLeaveCombat end,
							set = function(info, value)
								db.AutoHide.OnLeaveCombat = value
							end,
						},
					},
				},
				QuestMinion = {
					name = L["Quest Minion"],
					type = "group",
					order = 2,
					args = {
						MainHide = {
							name = L["Hide Minion"],
							desc = L["Hides/Shows the Main Frame"],
							type = "toggle",
							get = function() return db.Main.HideSorhaQuestLog end,
							set = function()
										db.Main.HideSorhaQuestLog = not db.Main.HideSorhaQuestLog
										SorhaQuestLog:doQuestMinionAnchorUpdate(false)
									end,
							order = 1,
						},
						MainLock = {
							name = L["Lock Frame"],
							desc = L["Unlocks/Locks the Main Frame"],
							type = "toggle",
							get = function() return db.Minion.MinionWindowLocked end,
							set = function()
										db.Minion.MinionWindowLocked = not db.Minion.MinionWindowLocked
										SorhaQuestLog:doQuestMinionAnchorUpdate(false)
									end,
							order = 2,
						},
						HeaderMainFrame = {
							name = L["Main Frame Settings"],
							type = "header",
							order = 10,
						},
						MainWidth = {
							order = 11,
							name = L["Width"],
							desc = L["Controls the width of the main window."],
							type = "range",
							min = 150, max = 600, step = 1,
							isPercent = false,
							get = function() return db.Minion.MinionWidth end,
							set = function(info, value)
								db.Minion.MinionWidth = value
								SorhaQuestLog:UpdateQuestMinion()
							end,
						},	
						Reset = {
							order = 12,
							type = "execute",
							name = L["Reset Main Frame"],
							desc = L["Resets Main Frame position"],
							func = function()
								db.Minion.MinionWindowLocation.Point = "CENTER"
								db.Minion.MinionWindowLocation.RelativePoint =  "CENTER"
								db.Minion.MinionWindowLocation.X = 0
								db.Minion.MinionWindowLocation.Y = 0
								SorhaQuestLog:doQuestMinionAnchorUpdate(true)
							end,
						},
						HeaderMiscSettingsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 30,
						},
						HeaderMiscSettings = {
							name = L["Misc. Settings"],
							type = "header",
							order = 31,
						},
						ShowNumberQuestsToggle = {
							name = L["Show number of quests"],
							desc = L["Shows/Hides the number of quests"],
							type = "toggle",
							get = function() return db.Minion.ShowNumberOfQuests end,
							set = function()
								db.Minion.ShowNumberOfQuests = not db.Minion.ShowNumberOfQuests
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 32,
						},
						ShowNumberOfDailyQuestsToggle = {
							name = L["Show # of Dailys"],
							desc = L["Shows/Hides the number of daily quests completed"],
							type = "toggle",
							get = function() return db.Minion.ShowNumberOfDailyQuests end,
							set = function()
								db.Minion.ShowNumberOfDailyQuests = not db.Minion.ShowNumberOfDailyQuests
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 33,
						},
						CollapseToLeftToggle = {
							name = L["Autoshrink to left"],
							desc = L["Shrinks the width down when the length of current quests is less then the max width\nNote: Doesn't work well with quests that wordwrap"],
							type = "toggle",
							get = function() return db.Minion.CollapseToLeft end,
							set = function()
								db.Minion.CollapseToLeft = not db.Minion.CollapseToLeft
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 34,
						},
						GrowUpwardsToggle = {
							name = L["Grow Upwards"],
							desc = L["Minions grows upwards from the anchor"],
							type = "toggle",
							get = function() return db.Minion.GrowUpwards end,
							set = function()
								db.Minion.GrowUpwards = not db.Minion.GrowUpwards
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 35,
						},						
						ConfirmQuestAbandonsToggle = {
							name = L["Require confirmation when abandoning a Quest"],
							desc = L["Shows the confirm box when you try to abandon a quest"],
							type = "toggle",
							width = "full",
							get = function() return db.Minion.ConfirmQuestAbandons end,
							set = function()
								db.Minion.ConfirmQuestAbandons = not db.Minion.ConfirmQuestAbandons
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 36,
						},		
						AutoTrackQuests = {
							name = L["Automatically track quests"],
							desc = L["Same as blizzard setting. Tracked quests are shown quests when the ability to hide quests is on."],
							width = "full",
							type = "toggle",
							get = function()
								if (GetCVar("autoQuestWatch") ==  "1") then
									return true
								else
									return false
								end
							end,
							set = function()
								if (GetCVar("autoQuestWatch") ==  "1") then
									SetCVar("autoQuestWatch", "0", AUTO_QUEST_WATCH_TEXT)
								else
									SetCVar("autoQuestWatch", "1", AUTO_QUEST_WATCH_TEXT)
								end
							end,
							order = 37,
						},
						AutoTrackQuestsWhenObjectiveupdate = {
							name = L["Automatically track quests when objectives update"],
							desc = L["Same as blizzard setting. Tracked quests are shown quests when the ability to hide quests is on."],
							width = "full",
							type = "toggle",
							get = function()
								if (GetCVar("autoQuestProgress") ==  "1") then
									return true
								else
									return false
								end
							end,
							set = function()
								if (GetCVar("autoQuestProgress") ==  "1") then
									SetCVar("autoQuestProgress", "0", AUTO_QUEST_PROGRESS_TEXT)
								else
									SetCVar("autoQuestProgress", "1", AUTO_QUEST_PROGRESS_TEXT)
								end
							end,
							order = 38,
						},
						HideMinionIfNoTrackedQuestsToggle = {
							name = L["Hide minion when not tracking any quests"],
							desc = L["Doesn't display the minion when quests are not being tracked. Does not HIDE the minion"],
							type = "toggle",
							width = "full",
							get = function() return db.Minion.HideMinionIfNoTrackedQuests end,
							set = function()
								db.Minion.HideMinionIfNoTrackedQuests = not db.Minion.HideMinionIfNoTrackedQuests
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 39,
						},
						HeaderTitleFontSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 60,
						},
						HeaderTitleFont = {
							name = L["Title Font Settings"],
							type = "header",
							order = 61,
						},
						QuestMinionTitleFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 62,
							name = L["Font"],
							desc = L["The font that the minion title will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.QuestMinionTitleFont end,
							set = function(info, value)
								db.Fonts.QuestMinionTitleFont = value
								SorhaQuestLog:UpdateQuestMinion()
								SorhaQuestLog:UpdateRemoteQuestsMinion()
							end,
						},
						QuestMinionTitleFontOutlineSelect = {
							name = L["Font Outline"],
							desc = L["The outline that the minion title will use"],
							type = "select",
							order = 63,
							values = dicOutlines,
							get = function() return db.Fonts.QuestMinionTitleFontOutline end,
							set = function(info, value)
								db.Fonts.QuestMinionTitleFontOutline = value
								SorhaQuestLog:UpdateQuestMinion()
								SorhaQuestLog:UpdateRemoteQuestsMinion()
							end,
						},
						QuestMinionTitleFontSizeSelect = {
							order = 64,
							name = L["Font Size"],
							desc = L["Controls the font size of the minion title."],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.QuestMinionTitleFontSize end,
							set = function(info, value)
								db.Fonts.QuestMinionTitleFontSize = value
								SorhaQuestLog:UpdateQuestMinion()
								SorhaQuestLog:UpdateRemoteQuestsMinion()
							end,
						},
						QuestMinionTitleFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.QuestMinionTitleFontShadowed end,
							set = function()
								db.Fonts.QuestMinionTitleFontShadowed = not db.Fonts.QuestMinionTitleFontShadowed
								SorhaQuestLog:UpdateQuestMinion()
								SorhaQuestLog:UpdateRemoteQuestsMinion()
							end,
							order = 65,
						},
						HeaderItemButtonsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 70,
						},
						HeaderItemButtons = {
							name = L["Item Button Settings"],
							type = "header",
							order = 71,
						},
						ShowItemButtonsToggle = {
							name = L["Show quest item buttons"],
							desc = L["Shows/Hides the quest item buttons"],
							type = "toggle",
							get = function() return db.Minion.ShowItemButtons end,
							set = function()
								db.Minion.ShowItemButtons = not db.Minion.ShowItemButtons
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 72,
						},
						ItemsAndTooltipsRightToggle = {
							name = L["Display items and tooltips on right"],
							desc = L["Moves items and tooltips to the right"],
							type = "toggle",
							get = function() return db.Minion.MoveItemsAndTooltipsRight end,
							set = function()
								db.Minion.MoveItemsAndTooltipsRight = not db.Minion.MoveItemsAndTooltipsRight
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 73,
						},
						IndentItemButtonsToggle = {
							name = L["Indent item buttons inside tracker"],
							desc = L["Indents the item buttons into the quest tracker so they are flush with zone headers"],
							type = "toggle",
							width = "full",
							disabled = function() return (db.Minion.MoveItemsAndTooltipsRight == true or db.Minion.ShowItemButtons == false) end,
							get = function() return db.Minion.IndentItemButtons end,
							set = function()
								db.Minion.IndentItemButtons = not db.Minion.IndentItemButtons
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 74,
						},
						IndentItemButtonQuestsOnlyToggle = {
							name = L["Indent only quests with item buttons"],
							desc = L["Only indents a quest if the quest has an item button"],
							type = "toggle",
							width = "full",
							disabled = function() return (db.Minion.MoveItemsAndTooltipsRight == true or db.Minion.IndentItemButtons == false or db.Minion.ShowItemButtons == false) end,
							get = function() return db.Minion.IndentItemButtonQuestsOnly end,
							set = function()
								db.Minion.IndentItemButtonQuestsOnly = not db.Minion.IndentItemButtonQuestsOnly
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 75,
						},
						HideItemButtonsForCompletedQuestsToggle = {
							name = L["Hide Item Buttons for completed quests"],
							desc = L["Hides the quests item button once the quest is complete"],
							type = "toggle",
							width = "full",
							disabled = function() return not(db.Minion.ShowItemButtons) end,
							get = function() return db.Minion.HideItemButtonsForCompletedQuests end,
							set = function()
								db.Minion.HideItemButtonsForCompletedQuests = not db.Minion.HideItemButtonsForCompletedQuests
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 76,
						},						
						ItemButtonsSizeSlider = {
							order = 77,
							name = L["Item Button Size"],
							desc = L["Controls the size of the Item Buttons."],
							type = "range",
							disabled = function() return not(db.Minion.ShowItemButtons) end,
							min = 0.5, max = 2, step = 0.05,
							isPercent = false,
							get = function() return db.Minion.ItemButtonScale end,
							set = function(info, value)
								db.Minion.ItemButtonScale = value
								SorhaQuestLog:UpdateQuestMinion()
							end,
						},
						QuestOptions = {
							name = L["Quest Options"],
							type = "group",
							order = 100,
							args = {
								DisplaySettingsHeader = {
									name = L["Display Settings"],
									type = "header",
									order = 1,
								},
								AllowHiddenQuestsToggle = {
									name = L["Allow quests to be hidden"],
									desc = L["Allows quests to be hidden and enables the show/hide button"],
									type = "toggle",
									width = "full",
									get = function() return db.ZonesAndQuests.AllowHiddenQuests end,
									set = function()
										db.ZonesAndQuests.AllowHiddenQuests = not db.ZonesAndQuests.AllowHiddenQuests
										SorhaQuestLog:doHiddenQuestsUpdate()
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 3,
								},
								AllowHiddenHeadersToggle = {
									name = L["Zone headers hide when all contained quests are hidden"],
									desc = L["Makes zone headers hide when all contained quests are hidden"],
									type = "toggle",
									disabled = function() return not db.ZonesAndQuests.AllowHiddenQuests end,
									width = "full",
									get = function() return db.ZonesAndQuests.QuestHeadersHideWhenEmpty end,
									set = function()
										db.ZonesAndQuests.QuestHeadersHideWhenEmpty = not db.ZonesAndQuests.QuestHeadersHideWhenEmpty
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 4,
								},		
								AllowHiddenCountOnZonesToggle = {
									name = L["Display count of hidden quest in each zone"],
									desc = L["Displays a count of the hidden quests in each zone on the zone header"],
									type = "toggle",
									disabled = function() return not db.ZonesAndQuests.AllowHiddenQuests end,
									width = "full",
									get = function() return db.ZonesAndQuests.ShowHiddenCountOnZones end,
									set = function()
										db.ZonesAndQuests.ShowHiddenCountOnZones = not db.ZonesAndQuests.ShowHiddenCountOnZones
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 5,
								},
								ShowQuestLevelsToggle = {
									name = L["Display level in Quest Title"],
									desc = L["Displays the level of the quest in the title"],
									type = "toggle",
									width = "full",
									get = function() return db.ZonesAndQuests.ShowQuestLevels end,
									set = function()
										db.ZonesAndQuests.ShowQuestLevels = not db.ZonesAndQuests.ShowQuestLevels
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 6,
								},
								QuestTagsLengthSelect = {
									name = L["Quest Tag Length:"],
									desc = L["The length of the quest tags (d, p, g5, ELITE etc)"],
									type = "select",
									order = 7,
									values = dicQuestTagsLength,
									get = function() return db.ZonesAndQuests.QuestTagsLength end,
									set = function(info, value)
										db.ZonesAndQuests.QuestTagsLength = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderZoneHeadersSpacer = {
									name = "   ",
									width = "full",
									type = "description",
									order = 20,
								},
								HeaderZoneHeaders = {
									name = L["Zone Header Settings"],
									type = "header",
									order = 21,
								},
								ExpandOnEnterToggle = {
									name = L["Auto expand zones on enter"],
									desc = L["Automatically expands zone headers when you enter the zone"],
									type = "toggle",
									get = function() return db.ZonesAndQuests.ExpandOnEnter end,
									set = function()
										db.ZonesAndQuests.ExpandOnEnter = not db.ZonesAndQuests.ExpandOnEnter
										SorhaQuestLog:doHandleZoneChange()
									end,
									order = 22,
								},	
								CollapseOnLeaveToggle = {
									name = L["Auto collapse zones on exit"],
									desc = L["Automatically collapses zone headers when you exit the zone"],
									type = "toggle",
									get = function() return db.ZonesAndQuests.CollapseOnLeave end,
									set = function()
										db.ZonesAndQuests.CollapseOnLeave = not db.ZonesAndQuests.CollapseOnLeave
										SorhaQuestLog:doHandleZoneChange()
									end,
									order = 23,
								},	
								HeaderFontSelect = {
									type = "select", dialogControl = 'LSM30_Font',
									order = 24,
									name = L["Header Font"],
									desc = L["The font that the zone headers will use"],
									values = AceGUIWidgetLSMlists.font,
									get = function() return db.Fonts.HeaderFont end,
									set = function(info, value)
										db.Fonts.HeaderFont = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderFontOutlineSelect = {
									name = L["Header Font Outline"],
									desc = L["The outline that the zone headers will use"],
									type = "select",
									order = 25,
									values = dicOutlines,
									get = function() return db.Fonts.HeaderFontOutline end,
									set = function(info, value)
										db.Fonts.HeaderFontOutline = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderFontSize = {
									order = 26,
									name = L["Header Font Size"],
									desc = L["Controls the font size of the header label."],
									type = "range",
									min = 8, max = 20, step = 1,
									isPercent = false,
									get = function() return db.Fonts.HeaderFontSize end,
									set = function(info, value)
										db.Fonts.HeaderFontSize = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderFontShadowedToggle = {
									name = L["Shadow Text"],
									desc = L["Shows/Hides text shadowing"],
									type = "toggle",
									get = function() return db.Fonts.HeaderFontShadowed end,
									set = function()
										db.Fonts.HeaderFontShadowed = not db.Fonts.HeaderFontShadowed
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 27,
								},
								HideZoneHeadersToggle = {
									name = L["Hide Zone Headers"],
									desc = L["Hides all zone headers and just displays quests. Note: Does not expand zone headers for you"],
									type = "toggle",
									get = function() return db.ZonesAndQuests.HideZoneHeaders end,
									set = function()
										db.ZonesAndQuests.HideZoneHeaders = not db.ZonesAndQuests.HideZoneHeaders
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 28,
								},	
								HeaderQuestsSpacer = {
									name = "   ",
									width = "full",
									type = "description",
									order = 40,
								},
								HeaderQuests = {
									name = L["Quest Settings"],
									type = "header",
									order = 41,
								},
								OpenFullQuestLogToggle = {
									name = L["Left-click opens full Quest Log panel"],
									desc = L["Open the full quest log when clicking a quest.\nWhen disabled opens only the quest details panel for quests which are not completed remote quests.\n|cffffff78Alt-click always opens full log panel. |r"],
									width = "full",
									type = "toggle",
									get = function() return db.ZonesAndQuests.QuestClicksOpenFullQuestLog end,
									set = function()
										db.ZonesAndQuests.QuestClicksOpenFullQuestLog = not db.ZonesAndQuests.QuestClicksOpenFullQuestLog
									end,
									order = 42,
								},
								HideCompletedQuestsToggle = {
									name = L["Hide Completed quests/goto Quests"],
									desc = L["Automatically hides completed quests on completion. Also hides goto quests"],
									width = "full",
									type = "toggle",
									get = function() return db.ZonesAndQuests.HideCompletedQuests end,
									set = function()
										db.ZonesAndQuests.HideCompletedQuests = not db.ZonesAndQuests.HideCompletedQuests
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 43,
								},
								QuestFontSelect = {
									type = "select", dialogControl = 'LSM30_Font',
									order = 44,
									name = L["Quest Font"],
									desc = L["The font that the quest titles will use"],
									values = AceGUIWidgetLSMlists.font,
									get = function() return db.Fonts.QuestFont end,
									set = function(info, value)
										db.Fonts.QuestFont = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								QuestFontOutlineSelect = {
									name = L["Quest Font Outline"],
									desc = L["The outline that the quest titles will use"],
									type = "select",
									order = 45,
									values = dicOutlines,
									get = function() return db.Fonts.QuestFontOutline end,
									set = function(info, value)
										db.Fonts.QuestFontOutline = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								QuestFontSize = {
									order = 46,
									name = L["Quest Font Size"],
									desc = L["Controls the font size of the quest info."],
									type = "range",
									min = 8, max = 20, step = 1,
									isPercent = false,
									get = function() return db.Fonts.QuestFontSize end,
									set = function(info, value)
										db.Fonts.QuestFontSize = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								QuestFontShadowedToggle = {
									name = L["Shadow Text"],
									desc = L["Shows/Hides text shadowing"],
									type = "toggle",
									get = function() return db.Fonts.QuestFontShadowed end,
									set = function()
										db.Fonts.QuestFontShadowed = not db.Fonts.QuestFontShadowed
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 47,
								},
								QuestTitleIndent = {
									order = 48,
									name = L["Quest Text Indent"],
									desc = L["Controls the level of indentation for the quest text"],
									type = "range",
									min = 0, max = 20, step = 1,
									isPercent = false,
									get = function() return db.ZonesAndQuests.QuestTitleIndent end,
									set = function(info, value)
										db.ZonesAndQuests.QuestTitleIndent = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderObjectivesSpacer = {
									name = "   ",
									width = "full",
									type = "description",
									order = 80,
								},
								HeaderObjectives = {
									name = L["Objective Settings"],
									type = "header",
									order = 81,
								},
								HideCompletedObjectivesToggle = {
									name = L["Hide completed objectives"],
									desc = L["Shows/Hides completed objectives"],
									type = "toggle",
									width = "full",
									get = function() return db.ZonesAndQuests.HideCompletedObjectives end,
									set = function()
										db.ZonesAndQuests.HideCompletedObjectives = not db.ZonesAndQuests.HideCompletedObjectives
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 82,
								},
								ShowDescWhenNoObjectivesToggle = {
									name = L["Display quest description if not objectives"],
									desc = L["Displays a quests description if there are no objectives available"],
									type = "toggle",
									width = "full",
									get = function() return db.ZonesAndQuests.ShowDescWhenNoObjectives end,
									set = function()
										db.ZonesAndQuests.ShowDescWhenNoObjectives = not db.ZonesAndQuests.ShowDescWhenNoObjectives
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 83,
								},	
								ObjectiveFontSelect = {
									type = "select", dialogControl = 'LSM30_Font',
									order = 84,
									name = L["Objective Font"],
									desc = L["The font that the quest objectives will use"],
									values = AceGUIWidgetLSMlists.font,
									get = function() return db.Fonts.ObjectiveFont end,
									set = function(info, value)
										db.Fonts.ObjectiveFont = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveFontOutlineSelect = {
									name = L["Objective Font Outline"],
									desc = L["The outline that the quest objectives will use"],
									type = "select",
									order = 85,
									values = dicOutlines,
									get = function() return db.Fonts.ObjectiveFontOutline end,
									set = function(info, value)
										db.Fonts.ObjectiveFontOutline = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveFontSize = {
									order = 86,
									name = L["Objective Font Size"],
									desc = L["Controls the font size of the quest objectives"],
									type = "range",
									min = 8, max = 20, step = 1,
									isPercent = false,
									get = function() return db.Fonts.ObjectiveFontSize end,
									set = function(info, value)
										db.Fonts.ObjectiveFontSize = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveFontShadowedToggle = {
									name = L["Shadow Text"],
									desc = L["Shows/Hides text shadowing"],
									type = "toggle",
									get = function() return db.Fonts.ObjectiveFontShadowed end,
									set = function()
										db.Fonts.ObjectiveFontShadowed = not db.Fonts.ObjectiveFontShadowed
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 87,
								},
								ObjectivesIndent = {
									order = 88,
									name = L["Objective Text Indent"],
									desc = L["Controls the level of indentation for the Objective text"],
									type = "range",
									min = 0, max = 20, step = 1,
									isPercent = false,
									get = function() return db.ZonesAndQuests.ObjectivesIndent end,
									set = function(info, value)
										db.ZonesAndQuests.ObjectivesIndent = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
							},
						},
						ColourOptions = {
							name = L["Colour Options"],
							type = "group",
							order = 150,
							args = {
								HeaderColourSettings = {
									name = L["Colour Settings"],
									type = "header",
									order = 1,
								},
								QuestLevelColouringSelect = {
									name = L["Colour quest levels by:"],
									desc = L["The setting by which the colour of quest levels are determined"],
									type = "select",
									order = 2,
									values = dicQuestTitleColourOptions,
									get = function() return db.ZonesAndQuests.QuestLevelColouringSetting end,
									set = function(info, value)
										db.ZonesAndQuests.QuestLevelColouringSetting = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								QuestTitleColouringSelect = {
									name = L["Colour quest titles by:"],
									desc = L["The setting by which the colour of quest titles is determined"],
									type = "select",
									order = 3,
									values = dicQuestTitleColourOptions,
									get = function() return db.ZonesAndQuests.QuestTitleColouringSetting end,
									set = function(info, value)
										db.ZonesAndQuests.QuestTitleColouringSetting = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveTitleColouringSelect = {
									name = L["Colour objective title text by:"],
									desc = L["The setting by which the colour of objective title is determined"],
									type = "select",
									order = 4,
									values = dicObjectiveColourOptions,
									get = function() return db.ZonesAndQuests.ObjectiveTitleColouringSetting end,
									set = function(info, value)
										db.ZonesAndQuests.ObjectiveTitleColouringSetting = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveStatusColouringSelect = {
									name = L["Colour objective status text by:"],
									desc = L["The setting by which the colour of objective statuses is determined"],
									type = "select",
									order = 5,
									values = dicObjectiveColourOptions,
									get = function() return db.ZonesAndQuests.ObjectiveStatusColouringSetting end,
									set = function(info, value)
										db.ZonesAndQuests.ObjectiveStatusColouringSetting = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderMainColoursSpacer = {
									name = "   ",
									width = "full",
									type = "description",
									order = 20,
								},
								HeaderMainColours = {
									name = L["Main Colours"],
									type = "header",
									order = 21,
								},
								MinionBackGroundColour = {
									name = L["Background Colour"],
									desc = L["Sets the color of the minions background"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.MinionBackGroundColour.r = r
											db.Colours.MinionBackGroundColour.g = g
											db.Colours.MinionBackGroundColour.b = b
											db.Colours.MinionBackGroundColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 22,
								},
								MinionBorderColour = {
									name = L["Border Colour"],
									desc = L["Sets the color of the minions border"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.MinionBorderColour.r = r
											db.Colours.MinionBorderColour.g = g
											db.Colours.MinionBorderColour.b = b
											db.Colours.MinionBorderColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 23,
								},
								HeaderColour = {
									name = L["Zone Header Colour"],
									desc = L["Sets the color for the header of each zone"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.HeaderColour.r, db.Colours.HeaderColour.g, db.Colours.HeaderColour.b, db.Colours.HeaderColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.HeaderColour.r = r
											db.Colours.HeaderColour.g = g
											db.Colours.HeaderColour.b = b
											db.Colours.HeaderColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 24,
								},
								NoObjectivesColour = {
									name = L["No objectives description colour"],
									desc = L["Sets the color for the description displayed when there is no quest objectives"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.ObjectiveDescColour.r, db.Colours.ObjectiveDescColour.g, db.Colours.ObjectiveDescColour.b, db.Colours.ObjectiveDescColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.ObjectiveDescColour.r = r
											db.Colours.ObjectiveDescColour.g = g
											db.Colours.ObjectiveDescColour.b = b
											db.Colours.ObjectiveDescColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 25,
								},
								QuestLevelColour = {
									name = L["Quest levels"],
									desc = L["Sets the color for the quest levels if custom colouring is on"],
									type = "color",
									disabled = function() return not(db.ZonesAndQuests.QuestLevelColouringSetting == "Custom") end,
									hasAlpha = true,
									get = function() return db.Colours.QuestLevelColour.r, db.Colours.QuestLevelColour.g, db.Colours.QuestLevelColour.b, db.Colours.QuestLevelColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestLevelColour.r = r
											db.Colours.QuestLevelColour.g = g
											db.Colours.QuestLevelColour.b = b
											db.Colours.QuestLevelColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 26,
								},
								QuestTitleColour = {
									name = L["Quest titles"],
									desc = L["Sets the color for the quest titles if colouring by level is off"],
									type = "color",
									disabled = function() return not(db.ZonesAndQuests.QuestTitleColouringSetting == "Custom") end,
									hasAlpha = true,
									get = function() return db.Colours.QuestTitleColour.r, db.Colours.QuestTitleColour.g, db.Colours.QuestTitleColour.b, db.Colours.QuestTitleColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestTitleColour.r = r
											db.Colours.QuestTitleColour.g = g
											db.Colours.QuestTitleColour.b = b
											db.Colours.QuestTitleColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 27,
								},
								ObjectiveTitleColourPicker = {
									name = L["Objective title colour"],
									desc = L["Sets the custom color for objectives titles"],
									type = "color",
									disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Custom") end,
									hasAlpha = true,
									get = function() return db.Colours.ObjectiveTitleColour.r, db.Colours.ObjectiveTitleColour.g, db.Colours.ObjectiveTitleColour.b, db.Colours.ObjectiveTitleColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.ObjectiveTitleColour.r = r
											db.Colours.ObjectiveTitleColour.g = g
											db.Colours.ObjectiveTitleColour.b = b
											db.Colours.ObjectiveTitleColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 28,
								},
								ObjectiveStatusColourPicker = {
									name = L["Objective status colour"],
									desc = L["Sets the custom color for objectives statuses"],
									type = "color",
									disabled = function() return not(db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Custom") end,
									hasAlpha = true,
									get = function() return db.Colours.ObjectiveStatusColour.r, db.Colours.ObjectiveStatusColour.g, db.Colours.ObjectiveStatusColour.b, db.Colours.ObjectiveStatusColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.ObjectiveStatusColour.r = r
											db.Colours.ObjectiveStatusColour.g = g
											db.Colours.ObjectiveStatusColour.b = b
											db.Colours.ObjectiveStatusColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 29,
								},
								QuestStatusFailedColourPicker = {
									name = L["Quest failed tag"],
									desc = L["Sets the color for the quest failed tag"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.QuestStatusFailedColour.r, db.Colours.QuestStatusFailedColour.g, db.Colours.QuestStatusFailedColour.b, db.Colours.QuestStatusFailedColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestStatusFailedColour.r = r
											db.Colours.QuestStatusFailedColour.g = g
											db.Colours.QuestStatusFailedColour.b = b
											db.Colours.QuestStatusFailedColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 30,
								},								
								QuestStatusDoneColourPicker = {
									name = L["Quest done tag"],
									desc = L["Sets the color for the quest done tag"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.QuestStatusDoneColour.r, db.Colours.QuestStatusDoneColour.g, db.Colours.QuestStatusDoneColour.b, db.Colours.QuestStatusDoneColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestStatusDoneColour.r = r
											db.Colours.QuestStatusDoneColour.g = g
											db.Colours.QuestStatusDoneColour.b = b
											db.Colours.QuestStatusDoneColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 31,
								},									
								QuestStatusGotoColourPicker = {
									name = L["Quest goto Tag"],
									desc = L["Sets the color for the quest goto tag"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.QuestStatusGotoColour.r, db.Colours.QuestStatusGotoColour.g, db.Colours.QuestStatusGotoColour.b, db.Colours.QuestStatusGotoColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestStatusGotoColour.r = r
											db.Colours.QuestStatusGotoColour.g = g
											db.Colours.QuestStatusGotoColour.b = b
											db.Colours.QuestStatusGotoColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 32,
								},	
								QuestMinionTitleColourPicker = {
									name = L["Minion Title Text"],
									desc = L["Sets the color for the minion title text"],
									width = "full",
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.QuestMinionTitleColour.r, db.Colours.QuestMinionTitleColour.g, db.Colours.QuestMinionTitleColour.b, db.Colours.QuestMinionTitleColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestMinionTitleColour.r = r
											db.Colours.QuestMinionTitleColour.g = g
											db.Colours.QuestMinionTitleColour.b = b
											db.Colours.QuestMinionTitleColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 33,
								},									
							},
						},
						QuestTimer = {
							name = L["Quest Timer"],
							type = "group",
							order = 175,
							args = {
								ShowQuestTimerFrameToggle = {
									name = L["Show Quest Timer Frame"],
									desc = L["Shows a quest timer frame since the blizzard one hides with the default watcher"],
									type = "toggle",
									get = function() return db.Main.ShowQuestTimerFrame end,
									set = function()
										db.Main.ShowQuestTimerFrame = not db.Main.ShowQuestTimerFrame
										SorhaQuestLog:doQuestTimersAnchorUpdate(false)
									end,
									order = 1,
								},
								TimerWindowLockedToggle = {
									name = L["Lock Quest Timer Frame"],
									desc = L["Locks the quest timer frame"],
									type = "toggle",
									disabled = function() return not db.Main.ShowQuestTimerFrame end,
									get = function() return db.Timer.TimerWindowLocked end,
									set = function()
										db.Timer.TimerWindowLocked = not db.Timer.TimerWindowLocked
										SorhaQuestLog:doQuestTimersAnchorUpdate(false)
									end,
									order = 2,
								},
								ShowTitleToggle = {
									name = L["Show Quest Timer Frame Title"],
									desc = L["Shows the Quest Timer Frame title"],
									type = "toggle",
									width = "full",
									get = function() return db.Timer.ShowTitle end,
									set = function()
										db.Timer.ShowTitle = not db.Timer.ShowTitle
									end,
									order = 3,
								},
								HideTitleWhenNoTrackingToggle = {
									name = L["Hide title when you have no timers"],
									desc = L["Hide the Quest Timer frames title when you have no timers running"],
									type = "toggle",
									width = "full",
									disabled = function() return not(db.Timer.ShowTitle) end,
									get = function() return db.Timer.HideTitleWhenNoTimers end,
									set = function()
										db.Timer.HideTitleWhenNoTimers = not db.Timer.HideTitleWhenNoTimers
									end,
									order = 4,
								},
								ShowBorderToggle = {
									name = L["Enable border"],
									desc = L["Shows a border when the minion is visible, uses the Quest Minion colouring"],
									type = "toggle",
									width = "full",
									get = function() return db.Timer.ShowBorder end,
									set = function()
										db.Timer.ShowBorder = not db.Timer.ShowBorder
									end,
									order = 5,
								},
							},
						},
						RemoteQuests = {
							name = L["Remote Quests"],
							type = "group",
							order = 200,
							args = {
								ShowRemoteQuestMinionToggle = {
									name = L["Show Remote Quest Minion"],
									desc = L["Shows a minion for displaying remote quests since hiding the blizzard watcher hides these"],
									type = "toggle",
									get = function() return db.Main.ShowRemoteQuestsMinionFrame end,
									set = function()
										db.Main.ShowRemoteQuestsMinionFrame = not db.Main.ShowRemoteQuestsMinionFrame
										SorhaQuestLog:doRemoteQuestsMinionAnchorUpdate(false)
									end,
									order = 1,
								},								
								MinionLockedToggle = {
									name = L["Lock Remote Quests Minion"],
									desc = L["Locks the remote quests minion"],
									type = "toggle",
									disabled = function() return not db.Main.ShowQuestTimerFrame end,
									get = function() return db.RemoteQuests.MinionLocked end,
									set = function()
										db.RemoteQuests.MinionLocked = not db.RemoteQuests.MinionLocked
										SorhaQuestLog:doRemoteQuestsMinionAnchorUpdate(false)
									end,
									order = 2,
								},
								ShowTitleToggle = {
									name = L["Show Remote Quests Minion title"],
									desc = L["Shows the title for the Remote Quests Minion"],
									type = "toggle",
									width = "full",
									get = function() return db.RemoteQuests.ShowTitle end,
									set = function()
										db.RemoteQuests.ShowTitle = not db.RemoteQuests.ShowTitle
										SorhaQuestLog:UpdateRemoteQuestsMinion()
									end,
									order = 3,
								},
								HideTitleWhenNoRemoteQuestsToggle = {
									name = L["Hide title when you have no remote quests"],
									desc = L["Hide the title when you have no current remote quests"],
									type = "toggle",
									width = "full",
									disabled = function() return not(db.RemoteQuests.ShowTitle) end,
									get = function() return db.RemoteQuests.HideTitleWhenNoRemoteQuests end,
									set = function()
										db.RemoteQuests.HideTitleWhenNoRemoteQuests = not db.RemoteQuests.HideTitleWhenNoRemoteQuests
										SorhaQuestLog:UpdateRemoteQuestsMinion(false)
									end,
									order = 4,
								},
								ShowBorderToggle = {
									name = L["Enable border"],
									desc = L["Shows a border when the minion is visible, uses the Quest Minion colouring"],
									type = "toggle",
									width = "full",
									get = function() return db.RemoteQuests.ShowBorder end,
									set = function()
										db.RemoteQuests.ShowBorder = not db.RemoteQuests.ShowBorder
										SorhaQuestLog:UpdateRemoteQuestsMinion()
									end,
									order = 5,
								},
								RemoteMinionButtonSizeSlider = {
									order = 6,
									name = L["Minion Scale"],
									desc = L["Controls the size of the Minion"],
									type = "range",
									min = 0.5, max = 2, step = 0.05,
									isPercent = false,
									get = function() return db.RemoteQuests.MinionScale end,
									set = function(info, value)
										db.RemoteQuests.MinionScale = value
										SorhaQuestLog:UpdateRemoteQuestsMinion()
									end,
								},
							},
						},
					},
				},
				AchievementMinionOptions = {
					name = L["Achievement Options"],
					type = "group",
					order = 3,
					args = {
						ShowAchievementMinionFrameToggle = {
							name = L["Show Minion"],
							desc = L["Shows the Achievement Minion"],
							type = "toggle",
							get = function() return db.Main.ShowAchievementMinionFrame end,
							set = function()
								db.Main.ShowAchievementMinionFrame = not db.Main.ShowAchievementMinionFrame
								SorhaQuestLog:doAchievementMinionAnchorUpdate(false)
							end,
							order = 2,
						},					
						AchievementMinionLockedToggle = {
							name = L["Lock Minion"],
							desc = L["Locks the Achievement Minion"],
							type = "toggle",
							get = function() return db.AchievementMinion.MinionLocked end,
							set = function()
								db.AchievementMinion.MinionLocked = not db.AchievementMinion.MinionLocked
								SorhaQuestLog:doAchievementMinionAnchorUpdate(false)
							end,
							order = 3,
						},
						DisplaySettingsHeader = {
							name = L["Achievement Minion Settings"],
							type = "header",
							order = 10,
						},
						AchievementMinionWidth = {
							order = 11,
							name = L["Width"],
							desc = L["Controls the width of the minion"],
							type = "range",
							min = 150, max = 600, step = 1,
							isPercent = false,
							get = function() return db.AchievementMinion.MinionWidth end,
							set = function(info, value)
								db.AchievementMinion.MinionWidth = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},	
						AchievementMinionReset = {
							order = 12,
							type = "execute",
							name = L["Reset Minion Position"],
							desc = L["Resets Achievement Minions position"],
							func = function()
								db.AchievementMinion.MinionLocation.Point = "CENTER"
								db.AchievementMinion.MinionLocation.RelativePoint =  "CENTER"
								db.AchievementMinion.MinionLocation.X = 0
								db.AchievementMinion.MinionLocation.Y = 0
								SorhaQuestLog:doAchievementMinionAnchorUpdate(true)
							end,
						},
						HeaderMiscSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 20,
						},
						HeaderMisc = {
							name = L["Misc Settings"],
							type = "header",
							order = 21,
						},
						ShowTitleToggle = {
							name = L["Show Minion Title"],
							desc = L["Shows the Achievement Minions title"],
							type = "toggle",
							get = function() return db.AchievementMinion.ShowTitle end,
							set = function()
								db.AchievementMinion.ShowTitle = not db.AchievementMinion.ShowTitle
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 22,
						},
						GrowUpwardsToggle = {
							name = L["Grow Upwards"],
							desc = L["Minions grows upwards from the anchor"],
							type = "toggle",
							get = function() return db.AchievementMinion.GrowUpwards end,
							set = function()
								db.AchievementMinion.GrowUpwards = not db.AchievementMinion.GrowUpwards
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 23,
						},						
						HideTitleWhenNoTrackingToggle = {
							name = L["Hide title when not tracking"],
							desc = L["Hide the Achievement Minions title when not tracking"],
							type = "toggle",
							width = "full",
							disabled = function() return not(db.AchievementMinion.ShowTitle) end,
							get = function() return db.AchievementMinion.HideTitleWhenNoTracking end,
							set = function()
								db.AchievementMinion.HideTitleWhenNoTracking = not db.AchievementMinion.HideTitleWhenNoTracking
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 24,
						},
						CollapseToLeftToggle = {
							name = L["Autoshrink to left"],
							desc = L["Shrinks the width down when the length of current achivements is less then the max width\nNote: Doesn't work well with achivements that wordwrap"],
							type = "toggle",
							get = function() return db.AchievementMinion.MinionCollapseToLeft end,
							set = function()
								db.AchievementMinion.MinionCollapseToLeft = not db.AchievementMinion.MinionCollapseToLeft
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 25,
						},
						MoveTooltipsRightToggle = {
							name = L["Tooltips on right"],
							desc = L["Moves the tooltips to the right"],
							type = "toggle",
							get = function() return db.AchievementMinion.MoveTooltipsRight end,
							set = function()
								db.AchievementMinion.MoveTooltipsRight = not db.AchievementMinion.MoveTooltipsRight
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 26,
						},
						UseStatusBarsToggle = {
							name = L["Use Bars"],
							desc = L["Uses status bars for achivements with a quanity"],
							type = "toggle",
							get = function() return db.AchievementMinion.UseStatusBars end,
							set = function()
								db.AchievementMinion.UseStatusBars = not db.AchievementMinion.UseStatusBars
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 27,
						},		
						StatusBarTextureSelect = {
							name = L["Bar Texture"],
							desc = L["The texture the status bars for achievement progress use"],
							type = "select", dialogControl = "LSM30_Statusbar", 
							values = AceGUIWidgetLSMlists.statusbar, 
							disabled = function() return not(db.AchievementMinion.UseStatusBars) end,
							get = function() return db.AchievementMinion.StatusBarTexture end,
							set = function(info, value)
								db.AchievementMinion.StatusBarTexture = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 28,
						},
						MaxNumTasks = {
							order = 29,
							name = L["Tasks # Cap (0 = All)"],
							desc = L["# Tasks shown per Achievement. Set to 0 to display all tasks"],
							type = "range",
							min = 0, max = 20, step = 1,
							isPercent = false,
							get = function() return db.AchievementMinion.MaxTasksEachAchievement end,
							set = function(info, value)
								db.AchievementMinion.MaxTasksEachAchievement = value
								if (blnAchievementMinionIsUpdating == false) then
									blnAchievementMinionIsUpdating = true
									SorhaQuestLog:ScheduleTimer("UpdateAchievementMinion", 0.25)
								end
							end,
						},
						HeaderTitleFontSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 40,
						},
						HeaderTitleFont = {
							name = L["Minion Title Font Settings"],
							type = "header",
							order = 41,
						},
						AchievementMinionTitleFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 42,
							name = L["Font"],
							desc = L["The font that the minion title will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.AchievementMinionTitleFont end,
							set = function(info, value)
								db.Fonts.AchievementMinionTitleFont = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementMinionTitleFontOutlineSelect = {
							name = L["Font Outline"],
							desc = L["The outline that the minion title will use"],
							type = "select",
							order = 43,
							values = dicOutlines,
							get = function() return db.Fonts.AchievementMinionTitleFontOutline end,
							set = function(info, value)
								db.Fonts.AchievementMinionTitleFontOutline = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementMinionTitleFontSizeSelect = {
							order = 44,
							name = L["Font Size"],
							desc = L["Controls the font size of the minion title."],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.AchievementMinionTitleFontSize end,
							set = function(info, value)
								db.Fonts.AchievementMinionTitleFontSize = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementMinionTitleFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.AchievementMinionTitleFontShadowed end,
							set = function()
								db.Fonts.AchievementMinionTitleFontShadowed = not db.Fonts.AchievementMinionTitleFontShadowed
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 45,
						},
						HeaderFontsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 50,
						},
						HeaderFonts = {
							name = L["Font Settings"],
							type = "header",
							order = 51,
						},
						AchievementTitleFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 52,
							name = L["Title Font"],
							desc = L["The font that the achievement titles will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.AchievementTitleFont end,
							set = function(info, value)
								db.Fonts.AchievementTitleFont = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementTitleFontOutlineSelect = {
							name = L["Title Font Outline"],
							desc = L["The outline that the achievement titles will use"],
							type = "select",
							order = 53,
							values = dicOutlines,
							get = function() return db.Fonts.AchievementTitleFontOutline end,
							set = function(info, value)
								db.Fonts.AchievementTitleFontOutline = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementTitleFontSizeSize = {
							order = 54,
							name = L["Title Font Size"],
							desc = L["Controls the font size of the achievement title"],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.AchievementTitleFontSize end,
							set = function(info, value)
								db.Fonts.AchievementTitleFontSize = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementTitleFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.AchievementTitleFontShadowed end,
							set = function()
								db.Fonts.AchievementTitleFontShadowed = not db.Fonts.AchievementTitleFontShadowed
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 55,
						},
						AchievementObjectiveFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 56,
							name = L["Task Font"],
							desc = L["The font that the achievement tasks will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.AchievementObjectiveFont end,
							set = function(info, value)
								db.Fonts.AchievementObjectiveFont = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementObjectiveFontOutlineSelect = {
							name = L["Task Font Outline"],
							desc = L["The outline that the achievement tasks will use"],
							type = "select",
							order = 57,
							values = dicOutlines,
							get = function() return db.Fonts.AchievementObjectiveFontOutline end,
							set = function(info, value)
								db.Fonts.AchievementObjectiveFontOutline = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementObjectiveFontSizeSize = {
							order = 58,
							name = L["Task Font Size"],
							desc = L["Controls the font size of the achievement task"],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.AchievementObjectiveFontSize end,
							set = function(info, value)
								db.Fonts.AchievementObjectiveFontSize = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementObjectiveFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.AchievementObjectiveFontShadowed end,
							set = function()
								db.Fonts.AchievementObjectiveFontShadowed = not db.Fonts.AchievementObjectiveFontShadowed
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 59,
						},
						ColourOptions = {
							name = L["Colour Options"],
							type = "group",
							order = 50,
							args = {
								HeaderColourSettings = {
									name = L["Colour Settings"],
									type = "header",
									order = 1,
								},
								AchievementMinionTitleColour = {
									name = L["Achievement Minion Title"],
									desc = L["Sets the color for Achievement Minion Titles"],
									type = "color",
									width = "full",
									hasAlpha = true,
									get = function() return db.Colours.AchievementMinionTitleColour.r, db.Colours.AchievementMinionTitleColour.g, db.Colours.AchievementMinionTitleColour.b, db.Colours.AchievementMinionTitleColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.AchievementMinionTitleColour.r = r
											db.Colours.AchievementMinionTitleColour.g = g
											db.Colours.AchievementMinionTitleColour.b = b
											db.Colours.AchievementMinionTitleColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 2,
								},
								AchievementTitleColour = {
									name = L["Achievement Titles"],
									desc = L["Sets the color for Achievement Titles"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.AchievementTitleColour.r, db.Colours.AchievementTitleColour.g, db.Colours.AchievementTitleColour.b, db.Colours.AchievementTitleColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.AchievementTitleColour.r = r
											db.Colours.AchievementTitleColour.g = g
											db.Colours.AchievementTitleColour.b = b
											db.Colours.AchievementTitleColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 3,
								},
								AchievementObjectiveColour = {
									name = L["Achievement Task"],
									desc = L["Sets the color for Achievement Objectives"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.AchievementObjectiveColour.r, db.Colours.AchievementObjectiveColour.g, db.Colours.AchievementObjectiveColour.b, db.Colours.AchievementObjectiveColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.AchievementObjectiveColour.r = r
											db.Colours.AchievementObjectiveColour.g = g
											db.Colours.AchievementObjectiveColour.b = b
											db.Colours.AchievementObjectiveColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 4,
								},
								AchievementStatusBarFillColour = {
									name = L["Bar Fill Colour"],
									desc = L["Sets the color for the completed part of the achievement status bars"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.AchievementStatusBarFillColour.r, db.Colours.AchievementStatusBarFillColour.g, db.Colours.AchievementStatusBarFillColour.b, db.Colours.AchievementStatusBarFillColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.AchievementStatusBarFillColour.r = r
											db.Colours.AchievementStatusBarFillColour.g = g
											db.Colours.AchievementStatusBarFillColour.b = b
											db.Colours.AchievementStatusBarFillColour.a = a
											SorhaQuestLog:UpdateAchievementMinion()
										end,
									order = 5,
								},
								AchievementStatusBarBackColour = {
									name = L["Bar Back Colour"],
									desc = L["Sets the color for the un-completed part of the achievement status bars"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.AchievementStatusBarBackColour.r, db.Colours.AchievementStatusBarBackColour.g, db.Colours.AchievementStatusBarBackColour.b, db.Colours.AchievementStatusBarBackColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.AchievementStatusBarBackColour.r = r
											db.Colours.AchievementStatusBarBackColour.g = g
											db.Colours.AchievementStatusBarBackColour.b = b
											db.Colours.AchievementStatusBarBackColour.a = a
											SorhaQuestLog:UpdateAchievementMinion()
										end,
									order = 6,
								},
								AchievementMinionBackGroundColour = {
									name = L["Background Colour"],
									desc = L["Sets the color of the minions background"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.AchievementMinionBackGroundColour.r, db.Colours.AchievementMinionBackGroundColour.g, db.Colours.AchievementMinionBackGroundColour.b, db.Colours.AchievementMinionBackGroundColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.AchievementMinionBackGroundColour.r = r
											db.Colours.AchievementMinionBackGroundColour.g = g
											db.Colours.AchievementMinionBackGroundColour.b = b
											db.Colours.AchievementMinionBackGroundColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 7,
								},
								AchievementMinionBorderColour = {
									name = L["Border Colour"],
									desc = L["Sets the color of the minions border"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.AchievementMinionBorderColour.r, db.Colours.AchievementMinionBorderColour.g, db.Colours.AchievementMinionBorderColour.b, db.Colours.AchievementMinionBorderColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.AchievementMinionBorderColour.r = r
											db.Colours.AchievementMinionBorderColour.g = g
											db.Colours.AchievementMinionBorderColour.b = b
											db.Colours.AchievementMinionBorderColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 8,
								},
							},
						},
					},
				},
				NotificationOptions = {
					name = L["Notification Settings"],
					type = "group",
					order = 5,
					args = {
						NotificationSettingsHeader = {
							name = L["Text Notification Settings"],
							type = "header",
							order = 1,
						},
						SuppressBlizzardNotificationsToggle = {
							name = L["Suppress blizzard notification messages"],
							desc = L["Suppresses the notification messages sent by blizzard to the UIErrors Frame for progress updates"],
							type = "toggle",
							width = "full",
							get = function() return db.Notifications.SuppressBlizzardNotifications end,
							set = function()
								db.Notifications.SuppressBlizzardNotifications = not db.Notifications.SuppressBlizzardNotifications
							end,
							order = 2,
						},
						LibSinkHeaderSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 20,
						},
						LibSinkHeader = {
							name = L["LibSink Options"],
							type = "header",
							order = 21,
						},
						LibSinkObjectivesSmallHeader = {
							name = "|cff00ff00" .. L["Objective Notifications"] .. "|r",
							width = "full",
							type = "description",
							order = 22,
						},
						LibSinkObjectiveNotificationsToggle = {
							name = L["Use for Objective notification messages"],
							desc = L["Displays objective notification messages using LibSink"],
							type = "toggle",
							width = "full",
							get = function() return db.Notifications.LibSinkObjectiveNotifications end,
							set = function()
								db.Notifications.LibSinkObjectiveNotifications = not db.Notifications.LibSinkObjectiveNotifications
							end,
							order = 23,
						},
						DisplayQuestOnObjectiveNotificationsToggle = {
							name = L["Display Quest Name in objective notifications"],
							desc = L["Adds the quest name to objective notification messages"],
							type = "toggle",
							width = "full",
							disabled = function() return not(db.Notifications.LibSinkObjectiveNotifications) end,
							get = function() return db.Notifications.DisplayQuestOnObjectiveNotifications end,
							set = function()
								db.Notifications.DisplayQuestOnObjectiveNotifications = not db.Notifications.DisplayQuestOnObjectiveNotifications
							end,
							order = 24,
						},
						LibSinkQuestsSmallHeaderSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 25,
						},
						LibSinkQuestsSmallHeader = {
							name = "|cff00ff00" .. L["Quest Notifications"] .. "|r",
							width = "full",
							type = "description",
							order = 26,
						},
						ShowQuestCompletesAndFailsToggle = {
							name = L["Output Complete and Failed messages for quests"],
							desc = L["Displays '<Quest Title> (Complete)' etc messages once you finish all objectives"],
							type = "toggle",
							width = "full",
							get = function() return db.Notifications.ShowQuestCompletesAndFails end,
							set = function()
								db.Notifications.ShowQuestCompletesAndFails = not db.Notifications.ShowQuestCompletesAndFails
							end,
							order = 27,
						},
						ShowMessageOnPickingUpQuestItemToggle = {
							name = L["Show message when picking up an item that starts a quest"],
							desc = L["Displays a message through LibSink when you pick up an item that starts a quest"],
							type = "toggle",
							width = "full",
							get = function() return db.Notifications.ShowMessageOnPickingUpQuestItem end,
							set = function()
								db.Notifications.ShowMessageOnPickingUpQuestItem = not db.Notifications.ShowMessageOnPickingUpQuestItem
							end,
							order = 28,
						},
						LibSinkColourSmallHeaderSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 29,
						},
						LibSinkColourSmallHeader = {
							name = "|cff00ff00" .. L["Colour Settings"] .. "|r",
							width = "full",
							type = "description",
							order = 30,
						},
						NotificationsColourSelect = {
							name = L["Lib Sink Colour by:"],
							desc = L["The setting by which the colour of notification messages are determined"],
							type = "select",
							order = 31,
							values = dicNotificationColourOptions,
							get = function() return db.Notifications.LibSinkColourSetting end,
							set = function(info, value)
								db.Notifications.LibSinkColourSetting = value
							end,
						},
						NotificationsColour = {
							name = L["Notifications"],
							desc = L["Sets the color for notifications"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b, db.Colours.NotificationsColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.NotificationsColour.r = r
									db.Colours.NotificationsColour.g = g
									db.Colours.NotificationsColour.b = b
									db.Colours.NotificationsColour.a = a
								end,
							order = 32,
						},
						SoundSettingsHeaderSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 80,
						},
						SoundSettingsHeader = {
							name = L["Sound Settings"],
							type = "header",
							order = 81,
						},
						ObjectiveDoneSoundSelect = {
							name = L["Objective Completion Sound"], 
							desc = L["The sound played when you complete a quests objective"],
							type = "select", 
							dialogControl = "LSM30_Sound", 
							values = AceGUIWidgetLSMlists.sound, 
							get = function() return db.Notifications.ObjectiveDoneSound end,
							set = function(info, value)
								db.Notifications.ObjectiveDoneSound = value
							end,
							order = 82
						},
						QuestDoneSoundSelect = {
							name = L["Quest Completion Sound"], 
							desc = L["The sound played when you complete a quest (Finish all objectives)"],
							type = "select", 
							dialogControl = "LSM30_Sound", 
							values = AceGUIWidgetLSMlists.sound, 
							get = function() return db.Notifications.QuestDoneSound end,
							set = function(info, value)
								db.Notifications.QuestDoneSound = value
							end,
							order = 83
						},
						QuestItemFoundSoundSelect = {
							name = L["Quest Starting Item Picked Up"], 
							desc = L["The sound played when you pickup an item that starts a quest"],
							type = "select", 
							dialogControl = "LSM30_Sound", 
							values = AceGUIWidgetLSMlists.sound, 
							get = function() return db.Notifications.QuestItemFoundSound end,
							set = function(info, value)
								db.Notifications.QuestItemFoundSound = value
							end,
							order = 84
						},
						NotificationsOptions = SorhaQuestLog:GetSinkAce3OptionsDataTable(),
					},
				},
				ColourOptions = {
					name = L["Colour Settings"],
					type = "group",
					order = 6,
					args = {
						InfoTextColour = {
							name = L["Info Text"],
							desc = L["Sets the color of the info text (Title bar, # of quests hidden etc)"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.InfoColour.r, db.Colours.InfoColour.g, db.Colours.InfoColour.b, db.Colours.InfoColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.InfoColour.r = r
									db.Colours.InfoColour.g = g
									db.Colours.InfoColour.b = b
									db.Colours.InfoColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 1,
						},
						HeaderGradualColoursSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 30,
						},
						HeaderGradualColours = {
							name = L["Gradual objective Colours"],
							type = "header",
							order = 31,
						},
						Objective00PlusColour = {
							name = L["0% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 0% complete"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.Objective00PlusColour.r, db.Colours.Objective00PlusColour.g, db.Colours.Objective00PlusColour.b, db.Colours.Objective00PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective00PlusColour.r = r
									db.Colours.Objective00PlusColour.g = g
									db.Colours.Objective00PlusColour.b = b
									db.Colours.Objective00PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 32,
						},
						Objective25PlusColour = {
							name = L["25% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 25% complete"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.QuestLevelColouringSetting == "Completion" or db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") end,
							hasAlpha = true,
							get = function() return db.Colours.Objective25PlusColour.r, db.Colours.Objective25PlusColour.g, db.Colours.Objective25PlusColour.b, db.Colours.Objective25PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective25PlusColour.r = r
									db.Colours.Objective25PlusColour.g = g
									db.Colours.Objective25PlusColour.b = b
									db.Colours.Objective25PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 33,
						},
						Objective50PlusColour = {
							name = L["50% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 50% complete"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.QuestLevelColouringSetting == "Completion" or db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") end,
							hasAlpha = false,
							get = function() return db.Colours.Objective50PlusColour.r, db.Colours.Objective50PlusColour.g, db.Colours.Objective50PlusColour.b, db.Colours.Objective50PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective50PlusColour.r = r
									db.Colours.Objective50PlusColour.g = g
									db.Colours.Objective50PlusColour.b = b
									db.Colours.Objective50PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 34,
						},
						Objective75PlusColour = {
							name = L["75% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 75% complete"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.QuestLevelColouringSetting == "Completion" or db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") end,
							hasAlpha = true,
							get = function() return db.Colours.Objective75PlusColour.r, db.Colours.Objective75PlusColour.g, db.Colours.Objective75PlusColour.b, db.Colours.Objective75PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective75PlusColour.r = r
									db.Colours.Objective75PlusColour.g = g
									db.Colours.Objective75PlusColour.b = b
									db.Colours.Objective75PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 35,
						},
						DoneObjectiveColour = {
							name = L["Complete objective colour"],
							desc = L["Sets the color for the complete objectives"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.ObjectiveDoneColour.r, db.Colours.ObjectiveDoneColour.g, db.Colours.ObjectiveDoneColour.b, db.Colours.ObjectiveDoneColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ObjectiveDoneColour.r = r
									db.Colours.ObjectiveDoneColour.g = g
									db.Colours.ObjectiveDoneColour.b = b
									db.Colours.ObjectiveDoneColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 36,
						},
					},
				},	
			},
		}
		SorhaQuestLog.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(SorhaQuestLog.db)
		SorhaQuestLog.options.args.profiles.order = 9
	end
	return SorhaQuestLog.options
	
end

function SorhaQuestLog:OnInitialize() -- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("SorhaQuestLogDB", defaults, true)
	db = self.db.profile
	dbChar = self.db.char
	self:SetSinkStorage(db)

	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	
	self:SetupOptions()
	self:SetupColours()
end

function SorhaQuestLog:OnEnable() -- Called when the addon is enabled
	strZone = GetRealZoneText()
	strSubZone = GetSubZoneText()

	self:RegisterEvent("QUEST_LOG_UPDATE")	
	self:RegisterEvent('PLAYER_LEVEL_UP');
	self:RegisterEvent("ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_INDOORS")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("TRACKED_ACHIEVEMENT_UPDATE")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")

	
	-- Remove achievements from blizzard questminion to skip thier error
	SetCVar("trackerFilter", "6");
		
	-- Hook for hiding blizzard quest tracker
	self:SecureHook("WatchFrame_Update",function()
		self:HandleBlizzardTracker()
	end);
	
	-- Hook for moving quest progress messages
	self:RawHookScript(UIErrorsFrame, "OnEvent", function(self, event, msg, ...) 
		SorhaQuestLog:HandleUIErrorsFrame(self, event, msg, ...) 
	end)
	
	-- Hook for firing an update when user tracks/untracks a quest in blizzard QL
	self:SecureHook("QuestLogTitleButton_OnClick", function(self, button)
		if (IsModifiedClick("QUESTWATCHTOGGLE")) then
			if (db.Main.HideSorhaQuestLog == false and db.Main.HideAll == false and blnIsUpdating == false) then
				SorhaQuestLog:UpdateQuestMinion()
			end
		end
	end);
	
	-- Hook for remote quest entry being added
	self:SecureHook("WatchFrameAutoQuest_AddPopUp")

	-- Hook for remote quest entry being removed
	self:SecureHook("WatchFrameAutoQuest_ClearPopUp")
	
	-- LibDataBroker setup
	if LDB then
		SQLBroker = LDB:NewDataObject("SorhaQuestLog", {
			type = "data source",
			label = L["Quests"],
			icon = "Interface\\GossipFrame\\AvailableQuestIcon",	
			text = "0/25",
			OnClick = function(clickedframe, button)
				if button == "RightButton" then 
					if (IsAltKeyDown()) then
						db.Main.ShowAnchors = not db.Main.ShowAnchors
						SorhaQuestLog:doAnchorUpdates(false)
						
					elseif (IsControlKeyDown()) then
						LibStub("AceConfigDialog-3.0"):Open("SorhaQuestLog") 
						
					elseif (IsShiftKeyDown()) then
						db.Minion.MinionWindowLocked = not db.Minion.MinionWindowLocked
						db.AchievementMinion.MinionLocked = db.Minion.MinionWindowLocked
						db.Timer.TimerWindowLocked = db.Minion.MinionWindowLocked
						db.RemoteQuests.MinionLocked = db.Minion.MinionWindowLocked
						SorhaQuestLog:doAnchorUpdates(false)
						
					else
						LibStub("AceConfigDialog-3.0"):Open("SorhaQuestLog") 
					end
				else 
					if (IsAltKeyDown()) then
						db.Main.ShowAchievementMinionFrame = not db.Main.ShowAchievementMinionFrame
						SorhaQuestLog:doAchievementMinionAnchorUpdate(false)
						
					elseif (IsControlKeyDown()) then
						SorhaQuestLog:ToggleSorhaQuestLog() 
						
					elseif (IsShiftKeyDown()) then
						db.Main.HideSorhaQuestLog = not db.Main.HideSorhaQuestLog
						SorhaQuestLog:doQuestMinionAnchorUpdate(false)
						
					else
						SorhaQuestLog:ToggleSorhaQuestLog() 
					end
				end
			end,
			OnTooltipShow = function(tooltip)
				if not tooltip or not tooltip.AddLine then return end
				tooltip:AddLine(L["Sorha Quest Log"])
				tooltip:AddDoubleLine("|cffffffff" .. L["Left-click"] .. "|r", L["Show/hide All enabled minions"])
				tooltip:AddDoubleLine("|cffffffff" .. L["Shift Left-click"] .. "|r", L["Show/hide Quest minion"])
				tooltip:AddDoubleLine("|cffffffff" .. L["Alt Left-click"] .. "|r", L["Show/hide Achievement minion"])
				-- tooltip:AddDoubleLine("|cffffffff" .. L["Control Left-click"] .. "|r", L["Show/hide all enabled minions"])
				
				tooltip:AddDoubleLine("   ", "   ")
				
				tooltip:AddDoubleLine("|cffffffff" .. L["Right-click"] .. "|r", L["Show options"])
				tooltip:AddDoubleLine("|cffffffff" .. L["Shift Right-click"] .. "|r", L["Lock/unlock all minions"])
				tooltip:AddDoubleLine("|cffffffff" .. L["Alt Right-click"] .. "|r", L["Show/hide minion anchors"])
				-- tooltip:AddDoubleLine("|cffffffff" .. L["Control Right-click"] .. "|r", L["Show/hide all enabled minions"])
			end,
		})
	end

	-- LibButtonFacade
	if (LBF) then
		LBF:RegisterSkinCallback("SorhaQuestLog", self.SkinChanged)

		local g = LBF:Group("SorhaQuestLog")
		g:Skin(db.ButtonSkins.SkinID, db.ButtonSkins.Gloss, db.ButtonSkins.Backdrop, db.ButtonSkins.Colors)
	end

	intTimeSinceLastSound = GetTime()
	LSM.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateMedia")
	
	if (db.Main.HideSorhaQuestLog == false and db.Main.HideAll == false) then
		self:UpdateQuestMinion()
	end
	if (db.Main.ShowQuestTimerFrame == true and db.Main.HideAll == false) then
		self:UpdateQuestTimers()	
	end
	if (db.Main.ShowRemoteQuestsMinionFrame == true and db.Main.HideAll == false) then
		self:UpdateRemoteQuestsMinion()	
	end
	if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
		self:UpdateAchievementMinion()	
	end
end

function SorhaQuestLog:SetupOptions() --Creates options. Registers chat commands
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SorhaQuestLog", getOptions())
	
	AceConfigDialog:SetDefaultSize("SorhaQuestLog", 680,525)
	self:RegisterChatCommand("sql", "ChatCommand")
	self:RegisterChatCommand("SorhaQuestLog", "ChatCommand")
end

function SorhaQuestLog:ChatCommand(input) --Handles chat commands
	if not input or input:trim() == "" then
		LibStub("AceConfigDialog-3.0"):Open("SorhaQuestLog")
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SorhaQuestLog, "SorhaQuestLog", "SorhaQuestLog", input)
	end
end


-- Event handlers
function SorhaQuestLog:QUEST_LOG_UPDATE(...)
	if (blnHaveRefreshedHeaders == false) then
		blnHaveRefreshedHeaders = true
		self:RefreshZoneHeadersState()
	end
	--QuestInfoDescriptionText:SetAlphaGradient(0, 10000);    
	
	if (db.Main.HideSorhaQuestLog == false) then
		if (blnWasAClick == true) then -- Was the event called from a click (Skips the delay on header collapses etc)
			blnIsUpdating = true
			self:UpdateQuestMinion()
		elseif (blnIsUpdating == false) then --If not updating then update, forces a 0.5 second delay between system called updates
			if (blnIgnoreUpdateEvents == false) then
				blnIsUpdating = true
				self:ScheduleTimer("UpdateQuestMinion", 0.3)
			end
		end
	end
end

function SorhaQuestLog:PLAYER_LEVEL_UP(...)
	if (db.Main.HideSorhaQuestLog == false) then
		self:UpdateQuestMinion()
	end

end

function SorhaQuestLog:ZONE_CHANGED(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:ZONE_CHANGED_INDOORS(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:ZONE_CHANGED_NEW_AREA(...)
	if (strZone == nil) then 
		strZone = GetRealZoneText()
		strSubZone = GetSubZoneText()
	end
	self:doHandleZoneChange()
end

function SorhaQuestLog:TRACKED_ACHIEVEMENT_UPDATE(...)
	local achievementID = select(2, ...);
	local elapsed = select(4, ...);
	local duration = select(5, ...);
	if (duration and duration > 0 and (elapsed < duration)) then
		tblAchievementTimers[achievementID] = {["StartTime"] = GetTime(), ["Duration"] = duration, ["TimeLeft"] = duration}
	end
	
	if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
		self:UpdateAchievementMinion()
	end
end

function SorhaQuestLog:PLAYER_ENTERING_WORLD(...)
	if (blnHaveRegisteredBagUpdate == false) then
		blnHaveRegisteredBagUpdate = true
		self:RegisterEvent("BAG_UPDATE")
	end
	isInstance, instanceType = IsInInstance()

	if (isInstance == 1) then
		if (instanceType == "arena") then
			if (db.AutoHide.OnArena == "Hide") then
				SorhaQuestLog:HideSorhaQuestLog()
			elseif (db.AutoHide.OnArena == "Show") then
				SorhaQuestLog:ShowSorhaQuestLog()
			end
		elseif (instanceType == "party") then -- 5 Man
			if (db.AutoHide.OnInstance == "Hide") then
				SorhaQuestLog:HideSorhaQuestLog()
			elseif (db.AutoHide.OnInstance == "Show") then
				SorhaQuestLog:ShowSorhaQuestLog()
			end		
		elseif (instanceType == "raid") then
			if (db.AutoHide.OnRaid == "Hide") then
				SorhaQuestLog:HideSorhaQuestLog()
			elseif (db.AutoHide.OnRaid == "Show") then
				SorhaQuestLog:ShowSorhaQuestLog()
			end		
		elseif (instanceType == "pvp") then -- Battleground
			if (db.AutoHide.OnBattleground == "Hide") then
				SorhaQuestLog:HideSorhaQuestLog()
			elseif (db.AutoHide.OnBattleground == "Show") then
				SorhaQuestLog:ShowSorhaQuestLog()
			end
		end
	else
		if (db.AutoHide.OnNormal == "Hide") then
			SorhaQuestLog:HideSorhaQuestLog()
		elseif (db.AutoHide.OnNormal == "Show") then
			SorhaQuestLog:ShowSorhaQuestLog()
		end
	end
end

function SorhaQuestLog:BAG_UPDATE(...)
	local intBag = select(2,...)
	
	if (db.Notifications.ShowMessageOnPickingUpQuestItem == true) then
		if (intBag < 5) then
			if (tContains(tblBagsToCheck, intBag) == nil) then
				tinsert(tblBagsToCheck, intBag)
			end
			if (blnBagCheckUpdating == false) then
				blnBagCheckUpdating = true
				self:ScheduleTimer("CheckBags", 1)
			end
		end
	end
end
	
function SorhaQuestLog:PLAYER_REGEN_ENABLED(...)
	if (db.AutoHide.OnLeaveCombat == "Hide") then
		SorhaQuestLog:HideSorhaQuestLog()
	elseif (db.AutoHide.OnLeaveCombat == "Show") then
		SorhaQuestLog:ShowSorhaQuestLog()
	end
end

function SorhaQuestLog:PLAYER_REGEN_DISABLED(...)
	if (db.AutoHide.OnEnterCombat == "Hide") then
		SorhaQuestLog:HideSorhaQuestLog()
	elseif (db.AutoHide.OnEnterCombat == "Show") then
		SorhaQuestLog:ShowSorhaQuestLog()
	end
end

function SorhaQuestLog:HandleUIErrorsFrame(frame, event, msg, ...)
	if (event == "UI_INFO_MESSAGE") then
		for k, strPattern in pairs(tblQuestMatchs) do
			if (msg:match(strPattern)) then
				if (db.Notifications.SuppressBlizzardNotifications == true) then
					return
				end
				break
			end
		end
	end
	SorhaQuestLog.hooks[frame].OnEvent(frame, event, msg, ...)
end

function SorhaQuestLog:WatchFrameAutoQuest_AddPopUp(...)
	if (blnRemoteQuestsMinionIsUpdating == false) then
		blnRemoteQuestsMinionIsUpdating = true
		self:ScheduleTimer("UpdateRemoteQuestsMinion", 1)
	end
end

function SorhaQuestLog:WatchFrameAutoQuest_ClearPopUp(...)
	if (blnRemoteQuestsMinionIsUpdating == false) then
		SorhaQuestLog:UpdateRemoteQuestsMinion()
	end
	
end

function SorhaQuestLog:UpdateQuestTimers()
	-- If half a second is past and timer shown, (then if timer not started create), then refresh
	if ((GetTime() - intLastOnUpdateTime) > 0.5) then
		intLastOnUpdateTime = GetTime()
		SorhaQuestLog:UpdateQuestTimerMinion(GetQuestTimers())
	end
end

function SorhaQuestLog:UpdateAchievementTimers()
	if ((GetTime() - intLastAchievementTimerUpdateTime) > 0.5) then
		intLastAchievementTimerUpdateTime = GetTime()
		if (intNumAchievementTimers > 0 and blnAchievementMinionIsUpdating == false) then
			if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
				SorhaQuestLog:UpdateAchievementMinion()
			end
		end
	end
end

function SorhaQuestLog:RefreshConfig()
	db = self.db.profile
	self:SetSinkStorage(db)
	self:doAnchorUpdates(true)
	self:doHiddenQuestsUpdate()
	self:doColourChange()
	self:HandleBlizzardTracker()
end

function SorhaQuestLog:UpdateMedia(event, mediatype, key) -- LSM getting new media
	if mediatype == "font" then
        if key == db.Fonts.QuestFont or key == db.Fonts.HeaderFont then 
			if (blnIsUpdating == false) then
				self:UpdateQuestMinion()
			end
		end
    end
end

function SorhaQuestLog:SkinChanged(SkinID, Gloss, Backdrop, Group, Button, Colors)
	if not(Group) then -- Addon level
		db.ButtonSkins.SkinID = SkinID
		db.ButtonSkins.Gloss = Gloss
		db.ButtonSkins.Backdrop = Backdrop
		db.ButtonSkins.Colors = Colors
	else			  -- Subgroup level
		db.ButtonSkins.SQLItemButtons.SkinID = SkinID
		db.ButtonSkins.SQLItemButtons.Gloss = Gloss
		db.ButtonSkins.SQLItemButtons.Backdrop = Backdrop
		db.ButtonSkins.SQLItemButtons.Colors = Colors	
	end
end

function SorhaQuestLog:HandleBlizzardTracker()
	if (db.Main.HideBlizzardTracker == true) then
		WatchFrame:Hide()
    else
		WatchFrame:Show()
    end
end


-- Minion Creation
function SorhaQuestLog:CreateQuestMinionLayout() -- Create main minion and its buttons etc
	-- Main Anchor
	fraQuestMinionAnchor = self:doCreateFrame("FRAME","SQLQuestMinionAnchor",UIParent,db.Minion.MinionWidth,20,1,"BACKGROUND",1, db.Minion.MinionWindowLocation.Point, UIParent, db.Minion.MinionWindowLocation.RelativePoint, db.Minion.MinionWindowLocation.X, db.Minion.MinionWindowLocation.Y, 1)
	fraQuestMinionAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraQuestMinionAnchor:SetBackdropColor(0, 1, 0, 0)
	fraQuestMinionAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
	
	fraQuestMinionAnchor:SetMovable(true)
	fraQuestMinionAnchor:SetClampedToScreen(true)
	fraQuestMinionAnchor:RegisterForDrag("LeftButton")
	fraQuestMinionAnchor:SetScript("OnDragStart", fraQuestMinionAnchor.StartMoving)
	fraQuestMinionAnchor:SetScript("OnDragStop",  function(self)
		fraQuestMinionAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraQuestMinionAnchor:GetPoint()
		db.Minion.MinionWindowLocation.Point = strPoint
		db.Minion.MinionWindowLocation.RelativePoint = strRelativePoint
		db.Minion.MinionWindowLocation.X = intPosX
		db.Minion.MinionWindowLocation.Y = intPosY
	end)
	fraQuestMinionAnchor:SetScript("OnEnter", function(self) 
		if (db.Main.ShowHelpTooltips == true) then
			if (db.Minion.MoveItemsAndTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, 0);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, 0);
			end
			
			GameTooltip:SetText(L["Quest Minion Anchor"], 0, 1, 0, 1);
			GameTooltip:AddLine(L["Drag this to move the Quest minion when it is unlocked.\n"], 1, 1, 1, 1);
			local strOutput = ""
			if (db.Minion.ShowNumberOfQuests == true) then
				strOutput = strOutput .. L["Displays # of quests you have in your log and the max limit\n"]
			end
			if (db.Minion.ShowNumberOfDailyQuests == true) then
				strOutput = strOutput .. L["Displays # of daily quests you have done today of the max limit\n"]
			end			
			GameTooltip:AddLine(strOutput, 1, 1, 1, 1);
			GameTooltip:AddLine(L["You can disable help tooltips in general settings"], 0.5, 0.5, 0.5, 1);
			
			GameTooltip:Show();
		end
	end)
	fraQuestMinionAnchor:SetScript("OnLeave", function(self) 
		GameTooltip:Hide()
	end)
	
	-- Quests Anchor
	fraQuestMinionAnchor.fraQuestsAnchor = self:doCreateLooseFrame("FRAME","SQLQuestsAnchor",fraQuestMinionAnchor, fraQuestMinionAnchor:GetWidth(),1,1,"LOW",1,1)
	fraQuestMinionAnchor.fraQuestsAnchor:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, 0);
	fraQuestMinionAnchor.fraQuestsAnchor:SetBackdropColor(0, 0, 0, 0)
	fraQuestMinionAnchor.fraQuestsAnchor:SetBackdropBorderColor(0,0,0,0)
	fraQuestMinionAnchor.fraQuestsAnchor:SetAlpha(0)
	
	-- Number of quests fontstring
	fraQuestMinionAnchor.objFontString = fraQuestMinionAnchor:CreateFontString(nil, "OVERLAY");
	fraQuestMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.QuestMinionTitleFont), db.Fonts.QuestMinionTitleFontSize, db.Fonts.QuestMinionTitleFontOutline)
	fraQuestMinionAnchor.objFontString:SetJustifyH("LEFT")
	fraQuestMinionAnchor.objFontString:SetJustifyV("TOP")
	fraQuestMinionAnchor.objFontString:SetText("");
	if (db.Fonts.QuestMinionTitleFontShadowed == true) then
		fraQuestMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		fraQuestMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	fraQuestMinionAnchor.objFontString:SetShadowOffset(1, -1)

	
	-- Show/Hide hidden quest button
	fraQuestMinionAnchor.buttonShowHidden = self:doCreateLooseFrame("BUTTON","SQLShowHiddenButton",fraQuestMinionAnchor,16,16,1,"LOW",1,1)
	fraQuestMinionAnchor.buttonShowHidden:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, 2);
	fraQuestMinionAnchor.buttonShowHidden:SetBackdrop({bgFile="Interface\\AddOns\\SorhaQuestLog\\Textures\\button1.tga", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=0, tileSize=0, edgeSize=1, insets={left=-0, right=0, top=0, bottom=0}})
	fraQuestMinionAnchor.buttonShowHidden:SetBackdropBorderColor(0, 0, 0, 0)
	
	-- Show/Hide hidden quest button hover
	fraQuestMinionAnchor.buttonShowHiddenHover = self:doCreateLooseFrame("FRAME","SQLShowHiddenButtonHover",fraQuestMinionAnchor.buttonShowHidden,fraQuestMinionAnchor.buttonShowHidden:GetWidth(),fraQuestMinionAnchor.buttonShowHidden:GetHeight(),1,"LOW",1,1)
	fraQuestMinionAnchor.buttonShowHiddenHover:SetPoint("TOPLEFT", fraQuestMinionAnchor.buttonShowHidden, "TOPLEFT", 0, 0);
	fraQuestMinionAnchor.buttonShowHiddenHover:SetBackdrop({bgFile="Interface\\AddOns\\SorhaQuestLog\\Textures\\button1_hover.tga", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=0, tileSize=0, edgeSize=1, insets={left=-0, right=0, top=0, bottom=0}})
	fraQuestMinionAnchor.buttonShowHiddenHover:SetBackdropColor(0, 0, 1, 1)
	fraQuestMinionAnchor.buttonShowHiddenHover:SetBackdropBorderColor(0,0,0,0)
	fraQuestMinionAnchor.buttonShowHiddenHover:SetAlpha(0)
	
	-- Show/Hide hidden quest button events
	fraQuestMinionAnchor.buttonShowHidden:RegisterForClicks("AnyUp")
	fraQuestMinionAnchor.buttonShowHidden:SetScript("OnEnter", function(self) 
		fraQuestMinionAnchor.buttonShowHiddenHover:SetAlpha(1) 
		if (db.Main.ShowHelpTooltips == true) then
			if (db.Minion.MoveItemsAndTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, 0);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, 0);
			end
			
			GameTooltip:SetText(L["Show all quests button"], 0, 1, 0, 1);
			GameTooltip:AddLine(L["Enable to show all hidden quests"], 1, 1, 1, 1);
			GameTooltip:AddLine(L["You can disable help tooltips in general settings"], 0.5, 0.5, 0.5, 1);
			
			GameTooltip:Show();
		end
	end)
	fraQuestMinionAnchor.buttonShowHidden:SetScript("OnLeave", function(self) 
		fraQuestMinionAnchor.buttonShowHiddenHover:SetAlpha(0) 
		GameTooltip:Hide()
	end)
	
	fraQuestMinionAnchor.buttonShowHidden:SetScript("OnClick", function() 
		dbChar.ZonesAndQuests.ShowAllQuests = not dbChar.ZonesAndQuests.ShowAllQuests
		if (dbChar.ZonesAndQuests.ShowAllQuests == true) then
			fraQuestMinionAnchor.buttonShowHidden:SetBackdropColor(0, 0.3, 1, 1)
		else
			fraQuestMinionAnchor.buttonShowHidden:SetBackdropColor(0.5, 0.5, 0.5, 1)
		end
		self:UpdateQuestMinion()
	end)
	
	fraQuestMinionBorder = self:doCreateFrame("FRAME","SQLQuestMinionBorder", fraQuestMinionAnchor, db.Minion.MinionWidth,40,1,"BACKGROUND",1, "TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, 0, 1)
	fraQuestMinionBorder:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraQuestMinionBorder:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
	fraQuestMinionBorder:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)
	
	blnQuestMinionInitialized = true
	self:doQuestMinionAnchorUpdate(false) -- Update backcolour and visibility
	self:doHiddenQuestsUpdate()	-- Update show/hide hidden quests button position/visibility etc
end

function SorhaQuestLog:CreateQuestTimerLayout() -- Creates quest timer frame
	-- Timer anchor
	fraQuestTimerAnchor = self:doCreateFrame("FRAME","SQLQuestTimerAnchor",UIParent,100,20,1,"BACKGROUND",1, db.Timer.TimerWindowLocation.Point, UIParent, db.Timer.TimerWindowLocation.RelativePoint, db.Timer.TimerWindowLocation.X, db.Timer.TimerWindowLocation.Y, 1)
	fraQuestTimerAnchor:SetMovable(true)
	fraQuestTimerAnchor:SetClampedToScreen(true)
	fraQuestTimerAnchor:RegisterForDrag("LeftButton")
	fraQuestTimerAnchor:SetScript("OnDragStart", fraQuestTimerAnchor.StartMoving)
	fraQuestTimerAnchor:SetScript("OnUpdate", self.UpdateQuestTimers)
	fraQuestTimerAnchor:SetScript("OnDragStop",  function(self)
		fraQuestTimerAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraQuestTimerAnchor:GetPoint()
		db.Timer.TimerWindowLocation.Point = strPoint
		db.Timer.TimerWindowLocation.RelativePoint = strRelativePoint
		db.Timer.TimerWindowLocation.X = intPosX
		db.Timer.TimerWindowLocation.Y = intPosY
	end)
	fraQuestTimerAnchor:SetScript("OnEnter", function(self) 
		if (db.Main.ShowHelpTooltips == true) then
			if (db.Minion.MoveItemsAndTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, 0);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, 0);
			end
			
			GameTooltip:SetText(L["Quest Timer Minion Anchor"], 0, 1, 0, 1);
			GameTooltip:AddLine(L["Drag this to move the Quest Timer minion when it is unlocked.\n"], 1, 1, 1, 1);
			GameTooltip:AddLine(L["You can disable help tooltips in general settings"], 0.5, 0.5, 0.5, 1);
			
			GameTooltip:Show();
		end
	end)
	fraQuestTimerAnchor:SetScript("OnLeave", function(self) 
		GameTooltip:Hide()
	end)
	
	fraQuestTimerAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraQuestTimerAnchor:SetBackdropColor(0, 0, 0, 0)
	fraQuestTimerAnchor:SetBackdropBorderColor(0, 0, 0, 0)

	-- Timer fontstring for title "Quest Timer"
	fraQuestTimerAnchor.objFontString = fraQuestTimerAnchor:CreateFontString(nil, "OVERLAY");
	fraQuestTimerAnchor.objFontString:SetPoint("TOPLEFT", fraQuestTimerAnchor, "TOPLEFT",0, 0);
	fraQuestTimerAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.QuestMinionTitleFont), db.Fonts.QuestMinionTitleFontSize, db.Fonts.QuestMinionTitleFontOutline)
	if (db.Fonts.QuestMinionTitleFontShadowed == true) then
		fraQuestTimerAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		fraQuestTimerAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	fraQuestTimerAnchor.objFontString:SetJustifyH("LEFT")
	fraQuestTimerAnchor.objFontString:SetJustifyV("TOP")
	fraQuestTimerAnchor.objFontString:SetText("");
	fraQuestTimerAnchor.objFontString:SetShadowOffset(1, -1)
	
	fraQuestTimerAnchor.BorderFrame = self:doCreateFrame("FRAME","SQLQuestTimerMinionBorder", fraQuestTimerAnchor, 100,20,1,"BACKGROUND",1, "TOPLEFT", fraQuestTimerAnchor, "TOPLEFT", -6, 6, 1)
	fraQuestTimerAnchor.BorderFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraQuestTimerAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
	fraQuestTimerAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)
	
	blnTimerInitialized = true
	self:doQuestTimersAnchorUpdate(false)
end

function SorhaQuestLog:CreateRemoteQuestsLayout() -- Creates remote quests frame
	-- Remote quests anchor
	fraRemoteQuestsAnchor = self:doCreateFrame("FRAME","SQLRemoteQuestsAnchor",UIParent,200,20,1,"BACKGROUND",1, db.RemoteQuests.MinionLocation.Point, UIParent, db.RemoteQuests.MinionLocation.RelativePoint, db.RemoteQuests.MinionLocation.X, db.RemoteQuests.MinionLocation.Y, 1)
	fraRemoteQuestsAnchor:SetMovable(true)
	fraRemoteQuestsAnchor:SetClampedToScreen(true)
	fraRemoteQuestsAnchor:RegisterForDrag("LeftButton")
	fraRemoteQuestsAnchor:SetScript("OnDragStart", fraRemoteQuestsAnchor.StartMoving)
	fraRemoteQuestsAnchor:SetScript("OnDragStop",  function(self)
		fraRemoteQuestsAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraRemoteQuestsAnchor:GetPoint()
		db.RemoteQuests.MinionLocation.Point = strPoint
		db.RemoteQuests.MinionLocation.RelativePoint = strRelativePoint
		db.RemoteQuests.MinionLocation.X = intPosX
		db.RemoteQuests.MinionLocation.Y = intPosY
	end)
	fraRemoteQuestsAnchor:SetScript("OnEnter", function(self) 
		if (db.Main.ShowHelpTooltips == true) then
			if (db.Minion.MoveItemsAndTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, 0);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, 0);
			end
			
			GameTooltip:SetText(L["Remote Quests Minion Anchor"], 0, 1, 0, 1);
			GameTooltip:AddLine(L["Drag this to move the Remote Quests minion when it is unlocked.\n"], 1, 1, 1, 1);
			GameTooltip:AddLine(L["You can disable help tooltips in general settings"], 0.5, 0.5, 0.5, 1);
			
			GameTooltip:Show();
		end
	end)
	fraRemoteQuestsAnchor:SetScript("OnLeave", function(self) 
		GameTooltip:Hide()
	end)
	
	fraRemoteQuestsAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraRemoteQuestsAnchor:SetBackdropColor(0, 0, 0, 0)
	fraRemoteQuestsAnchor:SetBackdropBorderColor(0, 0, 0, 0)

	-- Fontstring for title "Remote Quests"
	fraRemoteQuestsAnchor.objFontString = fraRemoteQuestsAnchor:CreateFontString(nil, "OVERLAY");
	fraRemoteQuestsAnchor.objFontString:SetPoint("TOPLEFT", fraRemoteQuestsAnchor, "TOPLEFT",0, 0);
	fraRemoteQuestsAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.QuestMinionTitleFont), db.Fonts.QuestMinionTitleFontSize, db.Fonts.QuestMinionTitleFontOutline)
	if (db.Fonts.QuestMinionTitleFontShadowed == true) then
		fraRemoteQuestsAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		fraRemoteQuestsAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	fraRemoteQuestsAnchor.objFontString:SetJustifyH("LEFT")
	fraRemoteQuestsAnchor.objFontString:SetJustifyV("TOP")
	fraRemoteQuestsAnchor.objFontString:SetText("");
	fraRemoteQuestsAnchor.objFontString:SetShadowOffset(1, -1)
	
	fraRemoteQuestsAnchor.BorderFrame = self:doCreateFrame("FRAME","SQLRemoteQuestsMinionBorder", fraRemoteQuestsAnchor, 100,20,1,"BACKGROUND",1, "TOPLEFT", fraRemoteQuestsAnchor, "TOPLEFT", -6, 6, 1)
	fraRemoteQuestsAnchor.BorderFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraRemoteQuestsAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
	fraRemoteQuestsAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)
	
	
	blnRemoteQuestsMinionInitialized = true
	self:doRemoteQuestsMinionAnchorUpdate(false)
end

function SorhaQuestLog:CreateAchievementMinionLayout() -- Creates quest timer frame
	fraAchievementMinionAnchor = self:doCreateFrame("FRAME","SQLAchievementMinionAnchor",UIParent,100,20,1,"BACKGROUND",1, db.AchievementMinion.MinionLocation.Point, UIParent, db.AchievementMinion.MinionLocation.RelativePoint, db.AchievementMinion.MinionLocation.X, db.AchievementMinion.MinionLocation.Y, 1)
	fraAchievementMinionAnchor:SetMovable(true)
	fraAchievementMinionAnchor:SetClampedToScreen(true)
	fraAchievementMinionAnchor:RegisterForDrag("LeftButton")
	fraAchievementMinionAnchor:SetScript("OnDragStart", fraAchievementMinionAnchor.StartMoving)
	fraAchievementMinionAnchor:SetScript("OnUpdate", self.UpdateAchievementTimers)
	fraAchievementMinionAnchor:SetScript("OnDragStop",  function(self)
		fraAchievementMinionAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraAchievementMinionAnchor:GetPoint()
		db.AchievementMinion.MinionLocation.Point = strPoint
		db.AchievementMinion.MinionLocation.RelativePoint = strRelativePoint
		db.AchievementMinion.MinionLocation.X = intPosX
		db.AchievementMinion.MinionLocation.Y = intPosY
	end)
	fraAchievementMinionAnchor:SetScript("OnEnter", function(self) 
		if (db.Main.ShowHelpTooltips == true) then
			if (db.Minion.MoveItemsAndTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, 0);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, 0);
			end
			
			GameTooltip:SetText(L["Achievement Minion Anchor"], 0, 1, 0, 1);
			GameTooltip:AddLine(L["Drag this to move the Achievement minion when it is unlocked.\n"], 1, 1, 1, 1);
			GameTooltip:AddLine(L["You can disable help tooltips in general settings"], 0.5, 0.5, 0.5, 1);
			
			GameTooltip:Show();
		end
	end)
	fraAchievementMinionAnchor:SetScript("OnLeave", function(self) 
		GameTooltip:Hide()
	end)
	
	
	fraAchievementMinionAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraAchievementMinionAnchor:SetBackdropColor(0, 0, 1, 0)
	fraAchievementMinionAnchor:SetBackdropBorderColor(0, 0, 0, 0)

	
	-- Achievements Anchor
	fraAchievementMinionAnchor.fraAchievementAnchor = self:doCreateLooseFrame("FRAME","SQLQuestsAnchor",fraAchievementMinionAnchor, fraAchievementMinionAnchor:GetWidth(),1,1,"LOW",1,1)
	fraAchievementMinionAnchor.fraAchievementAnchor:SetPoint("TOPLEFT", fraAchievementMinionAnchor, "TOPLEFT", 0, 0);
	fraAchievementMinionAnchor.fraAchievementAnchor:SetBackdropColor(0, 0, 0, 0)
	fraAchievementMinionAnchor.fraAchievementAnchor:SetBackdropBorderColor(0,0,0,0)
	fraAchievementMinionAnchor.fraAchievementAnchor:SetAlpha(0)
	
	
	-- Title Fontstring
	fraAchievementMinionAnchor.objFontString = fraAchievementMinionAnchor:CreateFontString(nil, "OVERLAY");
	fraAchievementMinionAnchor.objFontString:SetPoint("TOPLEFT", fraAchievementMinionAnchor, "TOPLEFT",0, 0);
	fraAchievementMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.AchievementMinionTitleFont), db.Fonts.AchievementMinionTitleFontSize, db.Fonts.AchievementMinionTitleFontOutline)
	if (db.Fonts.AchievementMinionTitleFontShadowed == true) then
		fraAchievementMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		fraAchievementMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	fraAchievementMinionAnchor.objFontString:SetJustifyH("LEFT")
	fraAchievementMinionAnchor.objFontString:SetJustifyV("TOP")
	fraAchievementMinionAnchor.objFontString:SetText("");
	fraAchievementMinionAnchor.objFontString:SetShadowOffset(1, -1)
	
	fraAchievementMinionAnchor.BorderFrame = self:doCreateFrame("FRAME","SQLRemoteQuestsMinionBorder", fraAchievementMinionAnchor, 100,20,1,"BACKGROUND",1, "TOPLEFT", fraAchievementMinionAnchor, "TOPLEFT", -8, 6, 1)
	fraAchievementMinionAnchor.BorderFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraAchievementMinionAnchor.BorderFrame:SetBackdropColor(db.Colours.AchievementMinionBackGroundColour.r, db.Colours.AchievementMinionBackGroundColour.g, db.Colours.AchievementMinionBackGroundColour.b, db.Colours.AchievementMinionBackGroundColour.a)
	fraAchievementMinionAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.AchievementMinionBorderColour.r, db.Colours.AchievementMinionBorderColour.g, db.Colours.AchievementMinionBorderColour.b, db.Colours.AchievementMinionBorderColour.a)	
	
	blnAchievementMinionInitialized = true
	self:doAchievementMinionAnchorUpdate(false)
end


-- Minion Update functions
function SorhaQuestLog:UpdateQuestMinion()
	if (blnQuestMinionInitialized == false) then
		self:CreateQuestMinionLayout()
	end
	
	-- Setup variables
	local intSpacingIncrease = 0
	local intLargestWidth = 20
	local blnLargestWidthIsHeader = false
	local intHeaderOutlineOffset = 0
	local intQuestOutlineOffset = 0
	local intObjectiveOutlineOffset = 0
	local intYPosition = 0
	local intInitialYOffset = 0
	local intButtonSize = intItemButtonSize * db.Minion.ItemButtonScale
	local intQuestOffset =  db.ZonesAndQuests.QuestTitleIndent
	local intObjectiveOffset =  db.ZonesAndQuests.ObjectivesIndent
	local intQuestWithItemButtonOffset = intQuestOffset + intButtonSize
	
	-- Reset was click to update boolean		
	blnWasAClick = false
		
	-- Release all buttons in use
	for k, objButton in pairs(tblQuestMinionLogButtons) do
		self:RecycleLogButton(objButton)
	end
	wipe(tblQuestMinionLogButtons)

	-- If this isn't the first get of quest info store the current info for later use
	if not(curQuestInfo == nil) then
		oldQuestInfo = curQuestInfo
		blnFirstUpdate = false
	end
	curQuestInfo = self:GetQuestLogInformation()

	-- Quest/Objective complete Sounds
	if (blnFirstUpdate == false) then
		self:CheckQuestStateChange()
	end	
	
	--Add in slight offsets for outlined text to try stop overlap
	if (db.Fonts.HeaderFontOutline == "THICKOUTLINE") then
		intHeaderOutlineOffset = 2
	elseif (db.Fonts.HeaderFontOutline == "OUTLINE") then
		intHeaderOutlineOffset = 1
	end
	if (db.Fonts.QuestFontOutline == "THICKOUTLINE") then
		intQuestOutlineOffset = 1.5
	elseif (db.Fonts.QuestFontOutline == "OUTLINE") then
		intQuestOutlineOffset = 0.5
	end
	if (db.Fonts.ObjectiveFontOutline == "THICKOUTLINE") then
		intObjectiveOutlineOffset = 1.5
	elseif (db.Fonts.ObjectiveFontOutline == "OUTLINE") then
		intObjectiveOutlineOffset = 0.5
	end	
		
	-- Number of quests title display
	if (curQuestInfo["IsZonesWithTrackedQuests"] == false and db.Minion.HideMinionIfNoTrackedQuests == true) then
		fraQuestMinionAnchor.objFontString:SetText("");
		fraQuestMinionAnchor.buttonShowHidden:Hide()	
	else
		if (db.ZonesAndQuests.AllowHiddenQuests == true) then
			fraQuestMinionAnchor.buttonShowHidden:Show()
		end
		
		if (db.Minion.ShowNumberOfQuests == true or db.Minion.ShowNumberOfDailyQuests == true or db.ZonesAndQuests.AllowHiddenQuests == true) then
			intYPosition = -db.Fonts.QuestMinionTitleFontSize
		else
			intYPosition = -2
		end
		
		if (db.Minion.ShowNumberOfQuests == true or db.Minion.ShowNumberOfDailyQuests == true) then
			fraQuestMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.QuestMinionTitleFont), db.Fonts.QuestMinionTitleFontSize, db.Fonts.QuestMinionTitleFontOutline)
			if (db.Fonts.QuestMinionTitleFontShadowed == true) then
				fraQuestMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
			else
				fraQuestMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
			end
			
			local strText = ""
			if (db.Minion.ShowNumberOfQuests == true) then
				strText = strText .. strQuestMinionTitleColour .. curQuestInfo["NoQuests"] .. "/25 |r"
			end
			if (db.Minion.ShowNumberOfDailyQuests == true) then
				strText = strText ..  strQuestMinionTitleColour .. "(" .. GetDailyQuestsCompleted() .. ")|r"
			end
			
			
			fraQuestMinionAnchor.objFontString:SetText(strText);
			if (fraQuestMinionAnchor.objFontString:GetWidth() > intLargestWidth) then
				if (db.ZonesAndQuests.AllowHiddenQuests == true) then
					intLargestWidth = fraQuestMinionAnchor.objFontString:GetWidth() + 20 -- Offset for the show/hide button pushing the fontstring accross
				else
					intLargestWidth = fraQuestMinionAnchor.objFontString:GetWidth()
				end
			end
		else
			fraQuestMinionAnchor.objFontString:SetText("");
		end
	end
	fraQuestMinionAnchor:SetWidth(db.Minion.MinionWidth)
	
	-- Update LDB 
	if LDB then
		SQLBroker.text = strInfoColour .. curQuestInfo["NoQuests"] .. "/25|r"
	end
	
	intInitialYOffset = intYPosition
	
	-- Zone/Quest buttons
	local objButton = nil
	for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do
		if (curQuestInfo["IsZonesWithTrackedQuests"] == false and db.Minion.HideMinionIfNoTrackedQuests == true) then
			break
		end
		
		if (not(db.ZonesAndQuests.AllowHiddenQuests == true and db.ZonesAndQuests.QuestHeadersHideWhenEmpty == true and dbChar.ZoneIsAllHiddenQuests[ZoneInstance["Title"]] == true) or (dbChar.ZonesAndQuests.ShowAllQuests == true)) then
			if (db.ZonesAndQuests.HideZoneHeaders == false) then
				objButton = self:GetQuestMinionButton()
				objButton.LogPosition = ZoneInstance["LogPosition"]
				objButton.isHeader = 1
				objButton.isCollapsed = ZoneInstance["Status"]
				objButton.intOffset = 0
		
				local strPrefix = strHeaderColour .. "- "
				if (ZoneInstance["Status"] == 1) then
					strPrefix = strHeaderColour .. "+ "
				end

				-- objButton:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, intYPosition - intHeaderOutlineOffset)
				objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);
				objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), db.Fonts.HeaderFontSize, db.Fonts.HeaderFontOutline)
				if (db.Fonts.HeaderFontShadowed == true) then
					objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
				else
					objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 0.0)
				end
				
				if (db.ZonesAndQuests.ShowHiddenCountOnZones == true and db.ZonesAndQuests.AllowHiddenQuests == true) then
					if (ZoneInstance["NumHidden"] > 0) then
						objButton.objFontString1:SetText(strPrefix .. ZoneInstance["Title"] .. "|r (" .. strInfoColour .. ZoneInstance["NumHidden"] .. "/" .. ZoneInstance["NumQuests"] .." Hidden)|r");	
					else
						objButton.objFontString1:SetText(strPrefix .. ZoneInstance["Title"] .. "|r");
					end
				else
					objButton.objFontString1:SetText(strPrefix .. ZoneInstance["Title"] .. "|r");
				end
				
				objButton:SetWidth(db.Minion.MinionWidth)
				--
				if (objButton.objFontString1:GetWidth() > intLargestWidth) then
					intLargestWidth = objButton.objFontString1:GetWidth()
					blnLargestWidthIsHeader = true
				end
				--
				objButton.objFontString1:SetWidth(db.Minion.MinionWidth)
				
				intSpacingIncrease = objButton.objFontString1:GetHeight() + intHeaderOutlineOffset
				objButton:SetHeight(intSpacingIncrease)
				tinsert(tblQuestMinionLogButtons, objButton)

				objButton:SetPoint("TOPLEFT", fraQuestMinionAnchor.fraQuestsAnchor, "TOPLEFT", 0, intYPosition - intHeaderOutlineOffset)
				intYPosition = intYPosition - intSpacingIncrease
			end
			-- Create each quest in zone
			for k2, QuestInstance in pairs(ZoneInstance["Quests"]) do
				if (QuestInstance["IsHidden"] == false or db.ZonesAndQuests.AllowHiddenQuests == false or dbChar.ZonesAndQuests.ShowAllQuests == true) then

					local blnHasShownButton = false
					local intThisQuestsOffset = intQuestOffset
					if (db.Minion.ShowItemButtons == true and QuestInstance["QuestItem"]) then -- Item Buttons on and has a button
						if (db.Minion.HideItemButtonsForCompletedQuests == false or (db.Minion.HideItemButtonsForCompletedQuests == true and not(QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0))) then -- Button not hidden because of completion
							blnHasShownButton = true
							if (db.Minion.IndentItemButtons == true and db.Minion.MoveItemsAndTooltipsRight == false) then
								intThisQuestsOffset = intQuestWithItemButtonOffset
							end
						end
					end
					if (db.Minion.IndentItemButtonQuestsOnly == false and db.Minion.IndentItemButtons == true and db.Minion.MoveItemsAndTooltipsRight == false) then
						intThisQuestsOffset = intQuestWithItemButtonOffset				
					end

					objButton = self:GetQuestMinionButton()
					objButton.LogPosition = QuestInstance["LogPosition"]
					-- objButton:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", intThisQuestsOffset, intYPosition)
					objButton.intOffset = intThisQuestsOffset
					
					-- Get quest text
					strQuestTitle, strObjectiveText = self:GetQuestText(QuestInstance)
					
					-- Setup quest title string
					objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);
					objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.QuestFont), db.Fonts.QuestFontSize, db.Fonts.QuestFontOutline)
					if (db.Fonts.QuestFontShadowed == true) then
						objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
					else
						objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 0.0)
					end
					
					objButton.objFontString1:SetText(strQuestTitle);

					-- Setup quest objectives string
					objButton.objFontString2:SetFont(LSM:Fetch("font", db.Fonts.ObjectiveFont), db.Fonts.ObjectiveFontSize, db.Fonts.ObjectiveFontOutline)
					if (db.Fonts.ObjectiveFontShadowed == true) then
						objButton.objFontString2:SetShadowColor(0.0, 0.0, 0.0, 1.0)
					else
						objButton.objFontString2:SetShadowColor(0.0, 0.0, 0.0, 0.0)
					end
					
					objButton.objFontString2:SetText(strObjectiveText);			
			
					
					objButton:SetWidth(db.Minion.MinionWidth - intThisQuestsOffset)

					-- Find out if either string is larger then the current largest string
					if (objButton.objFontString1:GetWidth() + intThisQuestsOffset > intLargestWidth) then
						intLargestWidth = objButton.objFontString1:GetWidth() + intThisQuestsOffset*2
						blnLargestWidthIsHeader = false
					end
					if (objButton.objFontString2:GetWidth() + (intThisQuestsOffset + intObjectiveOffset) > intLargestWidth) then
						intLargestWidth = objButton.objFontString2:GetWidth() + intThisQuestsOffset + intObjectiveOffset
						blnLargestWidthIsHeader = false
					end

					objButton.objFontString1:SetWidth(db.Minion.MinionWidth - intThisQuestsOffset)
					
					-- Set second fontstring of the buttons position
					local intSecondFSOffset = objButton.objFontString1:GetHeight() + intQuestOutlineOffset
					objButton.objFontString2:SetPoint("TOPLEFT", objButton, "TOPLEFT", intObjectiveOffset, -intSecondFSOffset);
					objButton.objFontString2:SetWidth(db.Minion.MinionWidth - intThisQuestsOffset - intObjectiveOffset)
					
					-- Find spacing needed for next button
					intSpacingIncrease = objButton.objFontString2:GetHeight() + intObjectiveOutlineOffset + intSecondFSOffset				
				
					-- If theres an item button to be shown add it
					if (blnHasShownButton == true) then
						local objItem = QuestInstance["QuestItem"]
						local objItemButton = self:GetItemButton(objItem, QuestInstance["LogPosition"])
						objItemButton:SetParent(objButton)

						if (db.Minion.MoveItemsAndTooltipsRight == true) then
							objItemButton:SetPoint("TOPLEFT", objButton, "TOPRIGHT", (8 * (1 / objItemButton:GetScale())), 0)
						else
							if (db.Minion.IndentItemButtons == true) then
								objItemButton:SetPoint("TOPRIGHT", objButton, "TOPLEFT", 0, 0)
							else
								objItemButton:SetPoint("TOPRIGHT", objButton, "TOPLEFT", -(16 * (1 / objItemButton:GetScale())), 0)
							end
						end
						
						-- If a button is heigher then its quest then expand the quest frame to stop overlapping buttons
						if (intButtonSize > intSpacingIncrease) then
							intSpacingIncrease = intButtonSize
							objButton.objFontString1:SetHeight(intButtonSize)
						end
						objButton.ItemButton = objItemButton
					end
					objButton:SetHeight(intSpacingIncrease)
					tinsert(tblQuestMinionLogButtons, objButton)

					objButton:SetPoint("TOPLEFT", fraQuestMinionAnchor.fraQuestsAnchor, "TOPLEFT", intThisQuestsOffset, intYPosition)
					intYPosition = intYPosition - intSpacingIncrease
				end
			end
		end
	end

	-- Auto collapse
	if (db.Minion.CollapseToLeft == true) then
		if (intLargestWidth < db.Minion.MinionWidth) then
			fraQuestMinionAnchor:SetWidth(intLargestWidth)
			
			for k, objButton in pairs(tblQuestMinionLogButtons) do
				objButton.objFontString1:SetWidth(intLargestWidth - objButton.intOffset)
				objButton:SetWidth(intLargestWidth - objButton.intOffset)
			end
		end
	end
	
	
	-- Border/Background
	if (curQuestInfo["IsZonesWithTrackedQuests"] == false and db.Minion.HideMinionIfNoTrackedQuests == true) then
		fraQuestMinionBorder:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, 0)
		fraQuestMinionBorder:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, 0)		
	else
		fraQuestMinionBorder:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
		fraQuestMinionBorder:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)	
	end

	fraQuestMinionBorder:ClearAllPoints()
	fraQuestMinionBorder:SetWidth(fraQuestMinionAnchor:GetWidth() + 12)
	

	-- Reposition/Resize the border and the Quests Anchor based on grow upwards option
	if (db.Minion.GrowUpwards == false) then
		fraQuestMinionBorder:SetPoint("TOPLEFT", fraQuestMinionAnchor.fraQuestsAnchor, "TOPLEFT", -6, 6)
		fraQuestMinionBorder:SetHeight((-intYPosition) + 6 + fraQuestMinionAnchor:GetHeight()/2)
		fraQuestMinionAnchor.fraQuestsAnchor:ClearAllPoints()
		fraQuestMinionAnchor.fraQuestsAnchor:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, 0);
	else
		fraQuestMinionBorder:SetPoint("TOPLEFT", fraQuestMinionAnchor.fraQuestsAnchor, "TOPLEFT", -6, 6 + intInitialYOffset)
		fraQuestMinionBorder:SetHeight((-intYPosition) + fraQuestMinionAnchor:GetHeight() -2)
		fraQuestMinionAnchor.fraQuestsAnchor:ClearAllPoints()
		fraQuestMinionAnchor.fraQuestsAnchor:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, -intYPosition + 5);
	end
	blnIsUpdating = false
end

function SorhaQuestLog:UpdateQuestTimerMinion(...) -- Refresh timer minion
	local numTimers = select("#", ...);
	local dblYPosition = db.Fonts.AchievementMinionTitleFontSize
	local dblOffset = 0
	local intLargestWidth = 0
	
	-- If QuestTimer minion not initalized then create it
	if (blnTimerInitialized == false) then
		self:CreateQuestTimerLayout()
	end	
	
	-- Release all used buttons
	for k, objButton in pairs(tblQuestTimerButtons) do
		self:RecycleLogButton(objButton)
	end
	wipe(tblQuestTimerButtons)
	
	-- Create timers
	for i = 1, numTimers do
		local objButton = self:GetLogButton()
		objButton:SetPoint("TOPLEFT", fraQuestTimerAnchor, "TOPLEFT", 0, -dblYPosition);
		
		-- Set buttons text
		objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.QuestFont), db.Fonts.QuestFontSize, db.Fonts.QuestFontOutline)
		if (db.Fonts.QuestFontShadowed == true) then
			objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		else
			objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 0.0)
		end
		
		objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);
		objButton.objFontString1:SetText(" - " .. SecondsToTime(select(i, ...)))
		
		-- Get offset for next button
		dblOffset = objButton.objFontString1:GetStringHeight()	
		
		if (intLargestWidth < objButton.objFontString1:GetStringWidth()) then
			intLargestWidth = objButton.objFontString1:GetStringWidth()
		end
		
		-- Set Height and Width
		objButton:SetHeight(dblOffset)
		objButton:SetWidth(objButton.objFontString1:GetStringWidth())
		
		-- Setup scripts
		objButton:SetScript("OnEnter", function (self) GameTooltip:SetOwner(self); GameTooltip:SetHyperlink(GetQuestLink(GetQuestIndexForTimer(i))); GameTooltip:Show(); end);
		objButton:SetScript("OnLeave", GameTooltip_Hide);
		
		dblYPosition = dblYPosition + dblOffset
		tinsert(tblQuestTimerButtons, objButton)
	end
	
	local blnNothingShown = false
	-- Show title if enabled
	if (db.Timer.ShowTitle == true and (db.Timer.HideTitleWhenNoTimers == false or (db.Timer.HideTitleWhenNoTimers == true and (numTimers > 0)))) then
		fraQuestTimerAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.QuestMinionTitleFont), db.Fonts.QuestMinionTitleFontSize, db.Fonts.QuestMinionTitleFontOutline)
		if (db.Fonts.QuestMinionTitleFontShadowed == true) then
			fraQuestTimerAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		else
			fraQuestTimerAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
		end
		
		fraQuestTimerAnchor.objFontString:SetText(strQuestMinionTitleColour .. L["Quest Timer Frame Title"])
	
		if (intLargestWidth < fraQuestTimerAnchor.objFontString:GetStringWidth()) then
			intLargestWidth = fraQuestTimerAnchor.objFontString:GetStringWidth()
		end
	else
		blnNothingShown = true
		fraQuestTimerAnchor.objFontString:SetText("")
	end	
	
	-- If border enabled and at least the title is shown
	if (blnNothingShown == false and db.Timer.ShowBorder == true) then
		fraQuestTimerAnchor.BorderFrame:Show()
		fraQuestTimerAnchor.BorderFrame:SetWidth(intLargestWidth + 12)
		fraQuestTimerAnchor.BorderFrame:SetHeight(dblYPosition + 2 + fraQuestTimerAnchor:GetHeight()/2)	
		
		fraQuestTimerAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
		fraQuestTimerAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)
	else
		fraQuestTimerAnchor.BorderFrame:Hide()
	end
end

function SorhaQuestLog:UpdateRemoteQuestsMinion()
	blnRemoteQuestsMinionIsUpdating = true

	-- If Achievement Minion is not Initialized then do so
	if (blnRemoteQuestsMinionInitialized == false) then
		self:CreateRemoteQuestsLayout()
	end
	
	-- Release all used buttons
	for k, objButton in pairs(tblRemoteQuestsButtons) do
		self:RecycleRemoteQuestButton(objButton)
	end
	wipe(tblRemoteQuestsButtons)
	
	-- Get Number of Auto Quests
	local numAutoQuestPopUps = GetNumAutoQuestPopUps();
	

	local blnNothingShown = false
	local intLargestWidth = 0

	-- Show title if enabled	
	if (db.RemoteQuests.ShowTitle == true and (db.RemoteQuests.HideTitleWhenNoRemoteQuests == false or (db.RemoteQuests.HideTitleWhenNoRemoteQuests == true and (numAutoQuestPopUps > 0)))) then
		fraRemoteQuestsAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.QuestMinionTitleFont), db.Fonts.QuestMinionTitleFontSize, db.Fonts.QuestMinionTitleFontOutline)
		if (db.Fonts.QuestMinionTitleFontShadowed == true) then
			fraRemoteQuestsAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		else
			fraRemoteQuestsAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
		end
		
		fraRemoteQuestsAnchor.objFontString:SetText(strQuestMinionTitleColour .. L["Remote Quests Minion Title"])
		if (intLargestWidth < fraRemoteQuestsAnchor.objFontString:GetStringWidth()) then
			intLargestWidth = fraRemoteQuestsAnchor.objFontString:GetStringWidth()
		end
		
	else
		blnNothingShown = true
		fraRemoteQuestsAnchor.objFontString:SetText("")
	end	

	if (numAutoQuestPopUps > 0) then
		local intButtonWidth = intRemoteQuestButtonWidth * db.RemoteQuests.MinionScale
		if (intLargestWidth < intButtonWidth) then
			intLargestWidth = intButtonWidth
		end
	end
	
	local intYPosition = (db.Fonts.QuestMinionTitleFontSize / db.RemoteQuests.MinionScale)
	for i=1, numAutoQuestPopUps do
		local questID, popUpType = GetAutoQuestPopUp(i);
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, _ = GetQuestLogTitle(GetQuestLogIndexByID(questID));
		
		if ( isComplete and isComplete > 0 ) then
			isComplete = true;
		else
			isComplete = false;
		end	

		if (questTitle and questTitle ~= "") then
			local objButton = self:GetRemoteQuestButton();
			objButton:SetScale(db.RemoteQuests.MinionScale)
			objButton:SetParent(fraRemoteQuestsAnchor);
			objButton.index = i;

			if (isComplete and popUpType == "COMPLETE") then
				objButton.ScrollChild.QuestionMark:Show();
				objButton.ScrollChild.Exclamation:Hide();
				objButton.ScrollChild.TopText:SetText(QUEST_WATCH_POPUP_CLICK_TO_COMPLETE);
				objButton.ScrollChild.BottomText:Hide();
				objButton.ScrollChild.TopText:SetPoint("TOP", 0, -12);
				objButton.ScrollChild.QuestName:SetPoint("TOP", 0, -32);
				if (objButton.questId and objButton.type=="OFFER") then
					objButton.ScrollChild.Flash:Show();
				end
				objButton.type="COMPLETED";
			elseif (popUpType == "OFFER") then
				objButton.ScrollChild.QuestionMark:Hide();
				objButton.ScrollChild.Exclamation:Show();
				objButton.ScrollChild.TopText:SetText(QUEST_WATCH_POPUP_QUEST_DISCOVERED);
				objButton.ScrollChild.BottomText:Show();
				objButton.ScrollChild.BottomText:SetText(QUEST_WATCH_POPUP_CLICK_TO_VIEW);
				objButton.ScrollChild.TopText:SetPoint("TOP", 0, -4);
				objButton.ScrollChild.QuestName:SetPoint("TOP", 0, -24);
				objButton.ScrollChild.Flash:Hide();
				objButton.type="OFFER";
			end
			objButton:SetPoint("TOPLEFT", fraRemoteQuestsAnchor, "TOPLEFT", 0, -intYPosition);

			objButton.ScrollChild.QuestName:SetText(questTitle);
			objButton.questId = questID;
			
			if (i > intLastCountRemoteQuests) then
				objButton.ScrollChild.Flash:Hide();
				objButton.totalTime = 0;
				objButton.slideInTime = 0.5;
				objButton:SetHeight(1);
				objButton:SetScript("OnUpdate", RemoteQuestButton_OnUpdate);
			end
			
			intYPosition = intYPosition + intRemoteQuestButtonHeight
			
			tinsert(tblRemoteQuestsButtons,objButton)			
		end
	end
	
	-- If border enabled and at least the title is shown
	if (blnNothingShown == false and db.RemoteQuests.ShowBorder == true) then
		fraRemoteQuestsAnchor.BorderFrame:Show()
		fraRemoteQuestsAnchor.BorderFrame:SetWidth(intLargestWidth + 12)
		fraRemoteQuestsAnchor.BorderFrame:SetHeight((intYPosition * db.RemoteQuests.MinionScale) + 2 + fraRemoteQuestsAnchor:GetHeight()/2)

		fraRemoteQuestsAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
		fraRemoteQuestsAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)
	else
		fraRemoteQuestsAnchor.BorderFrame:Hide()
	end
	
	intLastCountRemoteQuests = numAutoQuestPopUps
	blnRemoteQuestsMinionIsUpdating = false

	if (numAutoQuestPopUps > #tblRemoteQuestsButtons) then
		if (blnRemoteQuestsMinionIsUpdating == false) then
			blnRemoteQuestsMinionIsUpdating = true
			self:ScheduleTimer("UpdateRemoteQuestsMinion", 1)
		end
	end
end

function SorhaQuestLog:UpdateAchievementMinion()
	blnAchievementMinionIsUpdating = true
	
	-- If Achievement Minion is not Initialized then do so
	if (blnAchievementMinionInitialized == false) then
		self:CreateAchievementMinionLayout()
	end

	-- Release all used buttons
	for k, objButton in pairs(tblAchievementMinionButtons) do
		self:RecycleLogButton(objButton)
	end
	wipe(tblAchievementMinionButtons)
		
	-- Kill dead timers dead
	local intAlive = 0
	for k, tblTimer in pairs(tblAchievementTimers) do
		if (tblTimer["TimeLeft"] <= 0) then
			tblAchievementTimers[k] = nil
		else
			intAlive = intAlive + 1
		end
	end
	intNumAchievementTimers = intAlive
	
	--Add in slight offsets for outlined text to try stop overlap
	local intTitleOutlineOffset = 0
	local intObjectiveOutlineOffset = 0
	if (db.Fonts.AchievementTitleFontOutline == "THICKOUTLINE") then
		intTitleOutlineOffset = 2
	elseif (db.Fonts.AchievementTitleFontOutline == "OUTLINE") then
		intTitleOutlineOffset = 1
	end
	if (db.Fonts.AchievementObjectiveFontOutline == "THICKOUTLINE") then
		intObjectiveOutlineOffset = 1.5
	elseif (db.Fonts.AchievementObjectiveFontOutline == "OUTLINE") then
		intObjectiveOutlineOffset = 0.5
	end

	local intYPosition = 4
	
	
	local intLargestWidth = 0
	local blnNothingShown = false
	
	-- Show title if enabled
	if (db.AchievementMinion.ShowTitle == true and (db.AchievementMinion.HideTitleWhenNoTracking == false or (db.AchievementMinion.HideTitleWhenNoTracking == true and (GetNumTrackedAchievements() > 0)))) then
		fraAchievementMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.AchievementMinionTitleFont), db.Fonts.AchievementMinionTitleFontSize, db.Fonts.AchievementMinionTitleFontOutline)
		if (db.Fonts.AchievementMinionTitleFontShadowed == true) then
			fraAchievementMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		else
			fraAchievementMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
		end
		
		fraAchievementMinionAnchor.objFontString:SetText(strAchievementMinionTitleColour .. L["Achivement Tracker Title"])
		intLargestWidth = fraAchievementMinionAnchor.objFontString:GetWidth()
		
		intYPosition = db.Fonts.AchievementMinionTitleFontSize
	else
		blnNothingShown = true
		fraAchievementMinionAnchor.objFontString:SetText("")
		intLargestWidth = 100
	end			
	fraAchievementMinionAnchor:SetWidth(db.AchievementMinion.MinionWidth)

	-- Check if there is achievements shown even if titles hidden
	if (GetNumTrackedAchievements() > 0) then
		blnNothingShown = false
	end
	
	
	-- Create main minions buttons and set text etc	
	local intOffset = 0
	local intInitialYOffset = intYPosition
	local intStatusBarOffset = 2
	local blnFirstAchivement = true
	for _, intID in ipairs({GetTrackedAchievements()}) do
		local strTitle, strObjectives, blnIsQuantity, intHave, intNeed, intValue, intMax = self:GetAchievementText(intID, false)

		-- Get a button for achievement
		local objButton = self:GetAchievementMinionButton()
		objButton.intID = intID
		
		-- Set buttons title text
		objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);
		objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.AchievementTitleFont), db.Fonts.AchievementTitleFontSize, db.Fonts.AchievementTitleFontOutline)
		if (db.Fonts.AchievementTitleFontShadowed == true) then
			objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		else
			objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 0.0)
		end
		
		objButton.objFontString1:SetText(strTitle)
		
		objButton:SetWidth(db.AchievementMinion.MinionWidth)

		-- Find out if string is larger then the current largest string
		if (objButton.objFontString1:GetWidth() > intLargestWidth) then
			intLargestWidth = objButton.objFontString1:GetWidth()
		end
		objButton.objFontString1:SetWidth(db.AchievementMinion.MinionWidth)

		intOffset = objButton.objFontString1:GetHeight() + intTitleOutlineOffset + intObjectiveOutlineOffset
		
		-- Set buttons objective text
		objButton.objFontString2:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, -intOffset);
		objButton.objFontString2:SetFont(LSM:Fetch("font", db.Fonts.AchievementObjectiveFont), db.Fonts.AchievementObjectiveFontSize, db.Fonts.AchievementObjectiveFontOutline)
		if (db.Fonts.AchievementObjectiveFontShadowed == true) then
			objButton.objFontString2:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		else
			objButton.objFontString2:SetShadowColor(0.0, 0.0, 0.0, 0.0)
		end
		
		objButton.objFontString2:SetText(strObjectives)

		-- Find out if string is larger then the current largest string
		if (objButton.objFontString2:GetWidth() > intLargestWidth) then
			intLargestWidth = objButton.objFontString2:GetWidth()
		end
		objButton.objFontString2:SetWidth(db.AchievementMinion.MinionWidth)
		
		intOffset = intOffset + objButton.objFontString2:GetHeight() + intTitleOutlineOffset + intObjectiveOutlineOffset

		-- If there is a status bar add one
		if (blnIsQuantity == true and db.AchievementMinion.UseStatusBars == true) then
			intOffset = intOffset + intStatusBarOffset
			if (objButton.StatusBar == nil) then
				objButton.StatusBar = self:GetStatusBar()
			end
		
			objButton.StatusBar:Show()
			objButton.StatusBar:SetParent(objButton)
			objButton.StatusBar:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, -intOffset);
			
			-- Setup colours and texture
			objButton.StatusBar:SetStatusBarTexture(LSM:Fetch("statusbar", db.AchievementMinion.StatusBarTexture))
			objButton.StatusBar:SetStatusBarColor(db.Colours.AchievementStatusBarFillColour.r, db.Colours.AchievementStatusBarFillColour.g, db.Colours.AchievementStatusBarFillColour.b, db.Colours.AchievementStatusBarFillColour.a)
			
			objButton.StatusBar.Background:SetTexture(LSM:Fetch("statusbar", db.AchievementMinion.StatusBarTexture))			
			objButton.StatusBar.Background:SetVertexColor(db.Colours.AchievementStatusBarBackColour.r, db.Colours.AchievementStatusBarBackColour.g, db.Colours.AchievementStatusBarBackColour.b, db.Colours.AchievementStatusBarBackColour.a)
			
			objButton.StatusBar:SetBackdropColor(db.Colours.AchievementStatusBarBackColour.r, db.Colours.AchievementStatusBarBackColour.g, db.Colours.AchievementStatusBarBackColour.b, db.Colours.AchievementStatusBarBackColour.a)

			
			objButton.StatusBar.objFontString:SetFont(LSM:Fetch("font", db.Fonts.AchievementObjectiveFont), db.Fonts.AchievementObjectiveFontSize, db.Fonts.AchievementObjectiveFontOutline)
			if (db.Fonts.AchievementObjectiveFontShadowed == true) then
				objButton.StatusBar.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
			else
				objButton.StatusBar.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
			end
			
			objButton.StatusBar.objFontString:SetText(intHave .. "/" .. intNeed)
			
			-- Find out if string is larger then the current largest string
			if (objButton.StatusBar.objFontString:GetWidth() > intLargestWidth) then
				intLargestWidth = objButton.StatusBar.objFontString:GetWidth()
			end
			objButton.StatusBar.objFontString:SetWidth(db.AchievementMinion.MinionWidth)
			objButton.StatusBar:SetWidth(db.AchievementMinion.MinionWidth)
			objButton.StatusBar:SetHeight(objButton.StatusBar.objFontString:GetHeight() + 1)	

			
			objButton.StatusBar:SetMinMaxValues(0, intMax);
			objButton.StatusBar:SetValue(intValue);

			intOffset = intOffset + objButton.StatusBar:GetHeight() + intTitleOutlineOffset + intObjectiveOutlineOffset + intStatusBarOffset
		else
			if (objButton.StatusBar ~= nil) then
				objButton.StatusBar:Hide()
			end
		end
		
		objButton:SetHeight(intOffset)
		
		-- Set achievements buttons position
		objButton:SetPoint("TOPLEFT", fraAchievementMinionAnchor.fraAchievementAnchor, "TOPLEFT", 0, -intYPosition);
		intYPosition = intYPosition + intOffset		
		
		tinsert(tblAchievementMinionButtons, objButton)
	end

	local intBorderWidth = db.AchievementMinion.MinionWidth
	-- Auto collapse
	if (db.AchievementMinion.MinionCollapseToLeft == true) then
		if (intLargestWidth < db.AchievementMinion.MinionWidth) then
			fraAchievementMinionAnchor:SetWidth(intLargestWidth)
			intBorderWidth = intLargestWidth
			
			for k, objButton in pairs(tblAchievementMinionButtons) do
				objButton.objFontString1:SetWidth(intLargestWidth)
				if (objButton.StatusBar ~= nil) then
					objButton.StatusBar.objFontString:SetWidth(intLargestWidth)
					objButton.StatusBar:SetWidth(intLargestWidth)
				end
				
				objButton:SetWidth(intLargestWidth)
			end
		end
	end
	
	
	-- Show border if at least the title is shown
	if (blnNothingShown == false) then
		fraAchievementMinionAnchor.BorderFrame:Show()
		fraAchievementMinionAnchor.BorderFrame:SetWidth(intBorderWidth + 16)
		fraAchievementMinionAnchor.BorderFrame:SetBackdropColor(db.Colours.AchievementMinionBackGroundColour.r, db.Colours.AchievementMinionBackGroundColour.g, db.Colours.AchievementMinionBackGroundColour.b, db.Colours.AchievementMinionBackGroundColour.a)
		fraAchievementMinionAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.AchievementMinionBorderColour.r, db.Colours.AchievementMinionBorderColour.g, db.Colours.AchievementMinionBorderColour.b, db.Colours.AchievementMinionBorderColour.a)
	else
		fraAchievementMinionAnchor.BorderFrame:Hide()
	end
	
	
	fraAchievementMinionAnchor.BorderFrame:ClearAllPoints()
	
	-- Reposition/Resize the border and the Achievements Anchor based on grow upwards option
	if (db.AchievementMinion.GrowUpwards == false) then
		fraAchievementMinionAnchor.BorderFrame:SetPoint("TOPLEFT", fraAchievementMinionAnchor.fraAchievementAnchor, "TOPLEFT", -8, 6);
		fraAchievementMinionAnchor.BorderFrame:SetHeight(intYPosition + 2 + fraAchievementMinionAnchor:GetHeight()/2)
		fraAchievementMinionAnchor.fraAchievementAnchor:ClearAllPoints()
		fraAchievementMinionAnchor.fraAchievementAnchor:SetPoint("TOPLEFT", fraAchievementMinionAnchor, "TOPLEFT", 0, 0);
	else
		fraAchievementMinionAnchor.BorderFrame:SetPoint("TOPLEFT", fraAchievementMinionAnchor.fraAchievementAnchor, "TOPLEFT", -8,  6 - intInitialYOffset);
		fraAchievementMinionAnchor.BorderFrame:SetHeight(intYPosition + 2 + fraAchievementMinionAnchor:GetHeight()/2)
		fraAchievementMinionAnchor.fraAchievementAnchor:ClearAllPoints()
		fraAchievementMinionAnchor.fraAchievementAnchor:SetPoint("TOPLEFT", fraAchievementMinionAnchor, "TOPLEFT", 0, intYPosition);
	end

	
	
	
	blnAchievementMinionIsUpdating = false
end


-- Button functions
function SorhaQuestLog:GetQuestMinionButton()
	local objButton = self:GetLogButton()
	objButton:SetParent(fraQuestMinionAnchor)
	
	-- Create scripts
	objButton:RegisterForClicks("AnyUp")
	objButton:SetScript("OnClick", function(self, button)
		blnWasAClick = true

		if (button == "LeftButton") then
			if (self.isHeader == 1) then
				if (self.isCollapsed == 1) then
					ExpandQuestHeader(self.LogPosition)
				else
					CollapseQuestHeader(self.LogPosition)
				end
			else
				if (IsShiftKeyDown()) then -- Chat Link Quest/Show or Hide Track
					if ChatEdit_GetActiveWindow() then -- Link in chat
						ChatEdit_InsertLink(GetQuestLink(self.LogPosition))
					else -- Track/untrack quest
						if (db.ZonesAndQuests.AllowHiddenQuests == true) then
							if (IsQuestWatched(self.LogPosition) == nil) then
								if (GetNumQuestWatches() >= 25) then
									UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, 25), 1.0, 0.1, 0.1, 1.0);
								else
									AddQuestWatch(self.LogPosition)
								end
							else
								RemoveQuestWatch(self.LogPosition)
							end
							SorhaQuestLog:UpdateQuestMinion()
						end
					end
									
				elseif (IsControlKeyDown() and IsAltKeyDown()) then -- Abandon Quest
					local intCurrentSelectedIndex = GetQuestLogSelection()
					SelectQuestLogEntry(self.LogPosition)
					SetAbandonQuest()
					if ( QuestLogDetailFrame:IsShown() ) then
						HideUIPanel(QuestLogDetailFrame);
					end
					
					if (db.Minion.ConfirmQuestAbandons == true) then
						local items = GetAbandonQuestItems();
						if ( items ) then
							StaticPopup_Hide("ABANDON_QUEST");
							StaticPopup_Show("ABANDON_QUEST_WITH_ITEMS", GetAbandonQuestName(), items);
						else
							StaticPopup_Hide("ABANDON_QUEST_WITH_ITEMS");
							StaticPopup_Show("ABANDON_QUEST", GetAbandonQuestName());
						end
					else
						DEFAULT_CHAT_FRAME:AddMessage("|cFFDF4444" .. L["Quest abandoned: "] .. GetQuestLogTitle(self.LogPosition) .. "|r")
						PlaySound("igQuestFailed")
						AbandonQuest()
					end

					SelectQuestLogEntry(intCurrentSelectedIndex);
					
				elseif (IsAltKeyDown()) then -- Open quest in full log
					ShowUIPanel(QuestLogFrame);
					QuestLog_OpenToQuest(self.LogPosition, true);
					
				elseif (IsControlKeyDown()) then -- Track quest
					local _, _, _, _, _, _, _, _, questID = GetQuestLogTitle(self.LogPosition);
					SetSuperTrackedQuestID(questID)
				else -- Open quest based on option
				
					if (db.ZonesAndQuests.QuestClicksOpenFullQuestLog == true) then
						ShowUIPanel(QuestLogFrame);
					else
						local selectedIsComplete = select(7, GetQuestLogTitle(self.LogPosition));
						if (selectedIsComplete and GetQuestLogIsAutoComplete()) then
							ShowUIPanel(QuestLogFrame);
						end
					end
					QuestLog_OpenToQuest(self.LogPosition, true);
				end
			end
		else
			local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(self.LogPosition);
			if (self.isHeader == 1) then
				if (IsAltKeyDown()) then
					SorhaQuestLog:DisplayAltRightClickMenu(self)		
				else
					if (db.ZonesAndQuests.AllowHiddenQuests == true) then
						SorhaQuestLog:DisplayRightClickMenu(self)
					end
				end
			else
				if (db.ZonesAndQuests.AllowHiddenQuests == true) then
					if (IsQuestWatched(self.LogPosition) == nil) then
						if (GetNumQuestWatches() >= 25) then
							UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, 25), 1.0, 0.1, 0.1, 1.0);
						else
							AddQuestWatch(self.LogPosition)
						end
					else
						RemoveQuestWatch(self.LogPosition)
					end
					SorhaQuestLog:UpdateQuestMinion()
				end
			end
		end
	end)
	objButton:SetScript("OnEnter", function(self)
		if (not(self.isHeader)) then
		
			local intCurrentSelectedIndex = GetQuestLogSelection()
			local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(self.LogPosition);
			if (questTitle == nil) then 
				return nil
			end
			SelectQuestLogEntry(self.LogPosition);
			

			
			if (db.Minion.MoveItemsAndTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, -50);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, -50);
			end
			
			local strQuestDescription, strQuestObjectives = GetQuestLogQuestText();
			local objColour = GetQuestDifficultyColor(level);
			GameTooltip:SetText(questTitle, 0, 1, 0, 1);
			GameTooltip:AddLine(strQuestObjectives, 0.5, 0.5, 0.5, 1);
			
			local strQuestTag = ""
			if (isDaily) then
				strQuestTag = strQuestTag .. L["Daily"] .. " "
			end				
			if (questTag) then
				strQuestTag = strQuestTag .. questTag
				if (suggestedGroup > 0) then
					strQuestTag = strQuestTag .. " (" .. suggestedGroup .. ")"
				end
			end				
			GameTooltip:AddLine(strQuestTag, 1, 0, 0, 1);
			
			local intPartyMembers = GetNumGroupMembers();
			local blnIsOnQuest = false
			if (intPartyMembers > 0) then
				GameTooltip:AddLine(L["Party members on quest:"], 0, 1, 0, 1);
				for k = 1, intPartyMembers do
					blnIsOnQuest = IsUnitOnQuest(self.LogPosition, "party" .. k);
					if (blnIsOnQuest == 1) then
						GameTooltip:AddLine(UnitName("party" .. k), 1, 1, 1, 1);
					end
				end				
			end
			
			SelectQuestLogEntry(intCurrentSelectedIndex);
			GameTooltip:Show();
		else
			if (db.Main.ShowHelpTooltips == true) then
				if (db.Minion.MoveItemsAndTooltipsRight == true) then
					GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, 0);
				else 
					GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, 0);
				end
				
				GameTooltip:SetText(L["Zone Header"], 0, 1, 0, 1);
				GameTooltip:AddLine(L["Click to collapse/expand zone"], 1, 1, 1, 1);
				if (db.ZonesAndQuests.AllowHiddenQuests == true) then
					GameTooltip:AddLine(L["Right-click to show hidden quests toggle dropdown menu\n"], 1, 1, 1, 1);
				end
				GameTooltip:AddLine(L["Alt Right-click to show zone collapse/expand dropdown menu\n"], 1, 1, 1, 1);
				GameTooltip:AddLine(L["You can disable help tooltips in general settings"], 0.5, 0.5, 0.5, 1);
				
				GameTooltip:Show();
			end
		end
	end)
	objButton:SetScript("OnLeave", function(self) 
		GameTooltip:Hide() 
	end)
	
	return objButton
end

function SorhaQuestLog:GetAchievementMinionButton()
	local objButton = self:GetLogButton()
	objButton:SetParent(fraAchievementMinionAnchor)
	
	-- Create scripts
	objButton:RegisterForClicks("AnyUp")
	objButton:SetScript("OnClick", function(self, button)
		blnWasAClick = true
		if (button == "LeftButton") then
			if (IsShiftKeyDown()) then
				ChatEdit_InsertLink(GetAchievementLink(self.intID))
			else
				if not(AchievementFrame) then
					AchievementFrame_LoadUI();
				end

				if not(AchievementFrame:IsShown()) then
					AchievementFrame_ToggleAchievementFrame();
				end
				AchievementFrame_SelectAchievement(self.intID);
			end
		else
			RemoveTrackedAchievement(self.intID)
		end
	end)
	objButton:SetScript("OnEnter", function(self)
		local strTitle, strObjectives = SorhaQuestLog:GetAchievementText(self.intID, true)
		if (db.AchievementMinion.MoveTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, -50);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, -50);
		end
		GameTooltip:SetText(strTitle, 0, 1, 0, 1);
		GameTooltip:AddLine(strObjectives, 0.5, 0.5, 0.5, 1);
			
		GameTooltip:Show();
	end)
	objButton:SetScript("OnLeave", function(self) 
		GameTooltip:Hide() 
	end)
	
	return objButton
end

function SorhaQuestLog:GetLogButton()
	local objButton = tremove(tblLogButtonCache)
	
	if (objButton == nil) then
		intNumberOfLogButtons = intNumberOfLogButtons + 1
		local strButtonName = strButtonPrefix .. intNumberOfLogButtons
		
		-- Create button
		objButton = self:doCreateLooseFrame("BUTTON", strButtonName, UIParent, 10, 10, 1, "LOW", 1, 1)
		objButton:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
		objButton:SetBackdropColor(0, 0, 0, 0)
		objButton:SetBackdropBorderColor(0, 0, 0, 0)		

		-- Create buttons variables
		objButton.isHeader = nil
		objButton.isCollapsed = nil
		objButton.LogPosition = 0
		objButton.intID = 0

		-- Create Primary fontstring
		objButton.objFontString1 = objButton:CreateFontString(nil, "OVERLAY");
		objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 11, db.Fonts.HeaderFontOutline)
		objButton.objFontString1:SetJustifyH("LEFT")
		objButton.objFontString1:SetJustifyV("TOP")
		objButton.objFontString1:SetText("");
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		objButton.objFontString1:SetShadowOffset(1, -1)
		
		-- Create Secondary fontstring
		objButton.objFontString2 = objButton:CreateFontString(nil, "OVERLAY");
		objButton.objFontString2:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 11, db.Fonts.HeaderFontOutline)
		objButton.objFontString2:SetJustifyH("LEFT")
		objButton.objFontString2:SetJustifyV("TOP")
		objButton.objFontString2:SetText("");
		objButton.objFontString2:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		objButton.objFontString2:SetShadowOffset(1, -1)
	end
	
	objButton:Show()
	objButton:EnableMouse(true);
	
	return objButton
end

function SorhaQuestLog:RecycleLogButton(objButton)
	objButton:Hide()
	objButton:EnableMouse(false);
	
	if (objButton.ItemButton ~= nil) then
		self:RecycleItemButton(objButton.ItemButton)
		objButton.ItemButton = nil
	end
	
	if (objButton.StatusBar ~= nil) then
		self:RecycleStatusBar(objButton.StatusBar)
		objButton.StatusBar = nil
	end
	
	objButton.objFontString1:SetText("")
	objButton.objFontString1:SetWidth(0)
	objButton.objFontString1:SetHeight(0)
	objButton.objFontString1:ClearAllPoints()
	
	objButton.objFontString2:SetText("")
	objButton.objFontString2:SetWidth(0)
	objButton.objFontString2:SetHeight(0)	
	objButton.objFontString2:ClearAllPoints()
	
	objButton:SetScript("OnEnter", nil);
	objButton:SetScript("OnLeave", nil);
	objButton:SetScript("OnClick", nil);
	
	objButton:ClearAllPoints()
	objButton.isHeader = nil
	objButton.isCollapsed = nil
	objButton.LogPosition = 0
	objButton.intID = 0
	objButton.intOffset = 0

	tinsert(tblLogButtonCache, objButton)
end

function SorhaQuestLog:GetStatusBar()
	local objStatusBar = tremove(tblStatusBarsCache)
	if (objStatusBar == nil) then
		intNumberOfStatusBars = intNumberOfStatusBars + 1
		objStatusBar = CreateFrame("STATUSBAR", strStatusBarPrefix .. intNumberOfStatusBars, UIParent)
		
		objStatusBar.objFontString = objStatusBar:CreateFontString(nil, "OVERLAY");
		objStatusBar.objFontString:SetPoint("TOPLEFT", objStatusBar, "TOPLEFT", 0, -0.5);
		objStatusBar.objFontString:SetFont(LSM:Fetch("font", db.Fonts.AchievementObjectiveFont), db.Fonts.AchievementObjectiveFontSize, db.Fonts.AchievementObjectiveFontOutline)
		objStatusBar.objFontString:SetJustifyH("CENTER")
		objStatusBar.objFontString:SetJustifyV("TOP")
		objStatusBar.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		objStatusBar.objFontString:SetShadowOffset(1, -1)
		
		objStatusBar.Background = objStatusBar:CreateTexture(nil, "BORDER")
		objStatusBar.Background:SetAllPoints(objStatusBar)
		
		objStatusBar:SetBackdrop({bgFile = "Interface\\ChatFrame\\ChatFrameBackground", insets = {top = -1, left = -1, bottom = -1, right = -1}})
		objStatusBar:SetBackdropColor(0, 0, 0, 1)

		objStatusBar:SetStatusBarTexture(LSM:Fetch("statusbar", "Blizzard"))
		objStatusBar.Background:SetTexture(LSM:Fetch("statusbar", "Blizzard"))
		
		objStatusBar.Background:SetVertexColor(0, 0, 0, 1)
		objStatusBar:SetStatusBarColor(0, 1, 0, 1)
	end
	return objStatusBar
end

function SorhaQuestLog:GetItemButton(objItem, intLogPosition)
	local objButton = tremove(tblItemButtonCache)
	if (objButton == nil) then
		intNumberOfItemButtons = intNumberOfItemButtons + 1
		objButton = CreateFrame("BUTTON", strItemButtonPrefix .. intNumberOfItemButtons, UIParent, "WatchFrameItemButtonTemplate");		
	end
	
	objButton:Show()
	objButton:SetID(intLogPosition)
	objButton:SetScale(db.Minion.ItemButtonScale)
	objButton.ItemID = objItem["ID"]

	SetItemButtonTexture(objButton, objItem["Item"])
	SetItemButtonTextureVertexColor(objButton, 1.0, 1.0, 1.0)
	SetItemButtonCount(objButton, objItem["Charges"]);
	WatchFrameItem_UpdateCooldown(objButton);
	
	_G[objButton:GetName() .."HotKey"]:Hide()	

	objButton.rangeTimer = -1;
	if (LBF) then
		-- If the button groups not made yet then make it
		if not(LBFGroup) then
			LBFGroup = LBF:Group("SorhaQuestLog", "SQLItemButtons")
			LBFGroup:Skin(db.ButtonSkins.SQLItemButtons.SkinID, db.ButtonSkins.SQLItemButtons.Gloss, db.ButtonSkins.SQLItemButtons.Backdrop, db.ButtonSkins.SQLItemButtons.Colors)
			LBFGroup.SkinID = db.ButtonSkins.SQLItemButtons.SkinID or "DreamLayout"
			LBFGroup.Backdrop = db.ButtonSkins.SQLItemButtons.Backdrop
			LBFGroup.Gloss = db.ButtonSkins.SQLItemButtons.Gloss
			LBFGroup.Colors = db.ButtonSkins.SQLItemButtons.Colors
		end
			
		LBFGroup:AddButton(objButton, tblItemButtonLBFData)
		objButton:RegisterEvent("BAG_UPDATE_COOLDOWN")
	end

	if (ItemHasRange(objItem["Link"]) ~= nil) then
		objButton:RegisterEvent("PLAYER_TARGET_CHANGED")
		objButton:SetScript("OnUpdate", function(self, elapsed)
			self.rangeTimer = self.rangeTimer - elapsed
			if self.rangeTimer <= 0 then
				if IsQuestLogSpecialItemInRange(self:GetID()) == 0 then
					SetItemButtonTextureVertexColor(self, 0.8, 0.1, 0.1)
				else
					SetItemButtonTextureVertexColor(self, 1.0, 1.0, 1.0)
				end
				self.rangeTimer = TOOLTIP_UPDATE_TIME
			end
		end)
	end
	
	objButton:SetScript("OnEvent", function(self, event)
		if (event == "PLAYER_TARGET_CHANGED") then
			self.rangeTimer = -1
		elseif (event == "BAG_UPDATE_COOLDOWN") then
			local objCooldown = _G[self:GetName() .. "Cooldown"]
			local startTime, duration, enable = GetItemCooldown(self.ItemID)
			if startTime > 0 and duration > 0 and enable > 0 then
				objCooldown:SetCooldown(startTime, duration)
				objCooldown:Show()
			else
				objCooldown:Hide()
			end
		end
	end)

	
	return objButton
end

function SorhaQuestLog:GetRemoteQuestButton()
	local objButton = tremove(tblRemoteButtonCache)
	if (objButton == nil) then
		intNumberOfRemoteButtons = intNumberOfRemoteButtons + 1
		objButton = CreateFrame("SCROLLFRAME", strRemoteButtonPrefix .. intNumberOfRemoteButtons, UIParent, "WatchFrameAutoQuestPopUpTemplate");		
	end
	
	objButton:Show()
	return objButton
end

function SorhaQuestLog:RecycleStatusBar(objStatusBar)
	objStatusBar:SetParent(UIParent)
	objStatusBar:ClearAllPoints()
	objStatusBar:Hide()

	objStatusBar:SetHeight(0)
	objStatusBar:SetWidth(0)
	
	objStatusBar.objFontString:SetText("")
	objStatusBar.objFontString:SetHeight(0)
	objStatusBar.objFontString:SetWidth(0)
	
	objStatusBar:SetMinMaxValues(0, 1);
	objStatusBar:SetValue(0);
	tinsert(tblStatusBarsCache, objStatusBar)
end

function SorhaQuestLog:RecycleItemButton(objButton)
	objButton:SetParent(UIParent)
	objButton:ClearAllPoints()
	objButton:Hide()
	if (LBF) then
		LBFGroup:RemoveButton(objButton)
	end	
	objButton:SetScript("OnUpdate", nil)
	objButton:SetScript("OnEvent", nil)
	objButton:UnregisterEvent("BAG_UPDATE_COOLDOWN")
	objButton:UnregisterEvent("PLAYER_TARGET_CHANGED")
	tinsert(tblItemButtonCache, objButton)
end

function SorhaQuestLog:RecycleRemoteQuestButton(objButton)
	objButton:SetParent(UIParent)
	objButton:SetScale(1)
	objButton:ClearAllPoints()

	objButton:SetScript("OnUpdate", nil)	
	objButton.index = 0
	objButton:SetHeight(72)
	objButton.totalTime = 0
	objButton.slideInTime = 0
	objButton.type=""
	objButton.ScrollChild.QuestionMark:Hide();
	objButton.ScrollChild.Exclamation:Hide();
	objButton.ScrollChild.TopText:SetText("");
	objButton.ScrollChild.BottomText:SetText("");
	objButton.ScrollChild.BottomText:Hide();
	objButton.ScrollChild.QuestName:SetText("");
	objButton.ScrollChild.Flash:Hide();
	objButton.questId = nil;

	objButton:Hide()

	tinsert(tblRemoteButtonCache, objButton)
end


-- Quest information functions
function SorhaQuestLog:GetQuestText(QuestInstance)
	local strLevelColor = ""
	local strTitleColor = ""
	
	-- Get quest level colour
	if (db.ZonesAndQuests.QuestLevelColouringSetting == "Level") then
		local objColour = GetQuestDifficultyColor(QuestInstance["Level"]);	
		strLevelColor = format("|c%02X%02X%02X%02X", 255, objColour.r * 255, objColour.g * 255, objColour.b * 255);
	
	elseif (db.ZonesAndQuests.QuestLevelColouringSetting == "Completion") then
		strLevelColor = self:GetCompletionColourString(QuestInstance["CompletionLevel"])
	
	elseif (db.ZonesAndQuests.QuestLevelColouringSetting == "Done/Undone") then
		if (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0) then
			strLevelColor = strObjective100Colour
		else
			strLevelColor = strObjective00to24Colour
		end
	
	else
		strLevelColor = strQuestLevelColour
	end
	
	-- Get quest title colour
	if (db.ZonesAndQuests.QuestTitleColouringSetting == "Level") then
		local objColour = GetQuestDifficultyColor(QuestInstance["Level"]);	
		strTitleColor = format("|c%02X%02X%02X%02X", 255, objColour.r * 255, objColour.g * 255, objColour.b * 255);
	
	elseif (db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") then
		strTitleColor = self:GetCompletionColourString(QuestInstance["CompletionLevel"])
	
	elseif (db.ZonesAndQuests.QuestTitleColouringSetting == "Done/Undone") then
		if (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0) then
			strTitleColor = strObjective100Colour
		else
			strTitleColor = strObjective00to24Colour
		end
	
	else
		strTitleColor = strQuestTitleColour
	end
	
	
	local strQuestReturnText = ""
	local blnShowBrackets = false
	local blnFirstThing = true
	-- Quest level
	if (db.ZonesAndQuests.ShowQuestLevels == true) then
		strQuestReturnText = strQuestReturnText .. QuestInstance["Level"]
		blnShowBrackets = true
		blnFirstThing = false
	end
		
	-- PVP, RAID, Group, etc tags
	if (QuestInstance["Tag"] ~= nil) then
		if (db.ZonesAndQuests.QuestTagsLength == "Full" and dicLongQuestTags[QuestInstance["Tag"]] ~= nil) then
			if (blnFirstThing == true) then
				strQuestReturnText = strQuestReturnText .. dicLongQuestTags[QuestInstance["Tag"]]
			else
				strQuestReturnText = strQuestReturnText .. " " .. dicLongQuestTags[QuestInstance["Tag"]]
			end
			
			blnShowBrackets = true
			blnFirstThing = false
		elseif (db.ZonesAndQuests.QuestTagsLength == "Short" and dicQuestTags[QuestInstance["Tag"]] ~= nil) then
			strQuestReturnText = strQuestReturnText .. dicQuestTags[QuestInstance["Tag"]]
			blnShowBrackets = true
			blnFirstThing = false
		end		
	end
	
	if (QuestInstance["SuggestedGroup"] > 0 and db.ZonesAndQuests.QuestTagsLength ~= "None") then
		strQuestReturnText = strQuestReturnText .. QuestInstance["SuggestedGroup"]
		blnShowBrackets = true
		blnFirstThing = false
	end

	if (QuestInstance["IsDaily"]) then
		if (db.ZonesAndQuests.QuestTagsLength == "Full") then
			if (blnFirstThing == true) then
				strQuestReturnText = strQuestReturnText .. dicLongQuestTags["Daily"]
			else
				strQuestReturnText = strQuestReturnText .. " " .. dicLongQuestTags["Daily"]
			end
			blnShowBrackets = true
		elseif (db.ZonesAndQuests.QuestTagsLength == "Short") then
			strQuestReturnText = strQuestReturnText .. dicQuestTags["Daily"]
			blnShowBrackets = true
		end
	end
	
	if (db.ZonesAndQuests.ShowQuestLevels == true or db.ZonesAndQuests.QuestTagsLength ~= "None" and blnShowBrackets == true) then
		strQuestReturnText = strLevelColor .. "[" .. strQuestReturnText
		
		strQuestReturnText = strQuestReturnText .. "]|r "
	end
	
	strQuestReturnText = strQuestReturnText .. strTitleColor .. QuestInstance["Title"] .. "|r"
	
	-- Completion/failed etc tag
	if (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] < 0) then 
		strQuestReturnText = strQuestReturnText .. strQuestStatusFailed .. L[" (Failed)"] .. "|r"
	elseif (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0) then
		strQuestReturnText = strQuestReturnText .. strQuestStatusDone .. L[" (Done)"] .. "|r"
	elseif (#QuestInstance["ObjectiveList"] == 0) then
		strQuestReturnText = strQuestReturnText .. strQuestStatusGoto .. L[" (goto)"] .. "|r"
	end
	
	-- Hidden tag
	if (QuestInstance["IsHidden"] == true and db.ZonesAndQuests.AllowHiddenQuests == true) then
		strQuestReturnText = strQuestReturnText .. strHeaderColour .. L[" (Hidden)"] .. "|r"
	end 
	
	local strObjectivesReturnText = ""
	
	-- Objectives
	if (#QuestInstance["ObjectiveList"] == 0) then
		-- If no objective and show descriptions on display quest description
		if (db.ZonesAndQuests.ShowDescWhenNoObjectives == true) then
			if (not(QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0)) then
				strObjectivesReturnText = strObjectivesReturnText .. strObjectiveDescriptionColour .. " - " .. QuestInstance["ObjectiveDescription"] .. "|r\n"
			else
				if (db.ZonesAndQuests.HideCompletedObjectives == false) then
					strObjectivesReturnText = strObjectivesReturnText .. strObjectiveDescriptionColour .. " - " .. QuestInstance["ObjectiveDescription"] .. "|r\n"
				end
			end	
		end
	else
		-- For each objective in quest
		for k, ObjectiveInstance in pairs(QuestInstance["ObjectiveList"]) do
			if not(db.ZonesAndQuests.HideCompletedObjectives == true and ObjectiveInstance["Status"] == 1) then
				local strObjectiveGradualColour = "|cffffffff"
				local strObjectiveTitleColourOutput = strObjectiveTitleColour
				local strObjectiveStatusColourOutput = strObjectiveStatusColour
				
				-- If somethings uses gradual colours get colour
				if (db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion") then
					strObjectiveGradualColour = self:GetCompletionColourString(ObjectiveInstance["CompletionLevel"])
				end
				
				-- Decide on quest title colour
				if (db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion") then
					strObjectiveTitleColourOutput = strObjectiveGradualColour
				elseif (db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Done/Undone") then
					if (ObjectiveInstance["Status"] == nil) then
						strObjectiveTitleColourOutput = strObjective00to24Colour
					else
						strObjectiveTitleColourOutput = strObjective100Colour
					end
				end	

				-- Decide on quest status (0/1 etc) colour
				if (db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion") then
					strObjectiveStatusColourOutput = strObjectiveGradualColour
				elseif (db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Done/Undone") then
					if (ObjectiveInstance["Status"] == nil) then
						strObjectiveStatusColourOutput = strObjective00to24Colour
					else
						strObjectiveStatusColourOutput = strObjective100Colour
					end
				end				
				
				-- Depending on quest type display it in a certain way
				if (ObjectiveInstance["Type"] == nil) then
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r"		
					
				elseif (ObjectiveInstance["Type"] == "event") then
					if (ObjectiveInstance["Status"] == nil) then
						strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r"
					else
						strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r" .. strObjective100Colour .. " (Done)|r"
					end
					
				elseif (ObjectiveInstance["Type"] == "log") then
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r"
					
				elseif (ObjectiveInstance["Type"] == "reputation") then
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. ": |r" .. strObjectiveStatusColourOutput .. ObjectiveInstance["Got"] .. " / " .. ObjectiveInstance["Need"] .. "|r"
					
				else
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. ": |r" .. strObjectiveStatusColourOutput .. ObjectiveInstance["Got"] .. "/" .. ObjectiveInstance["Need"] .. "|r"
				end
				
				strObjectivesReturnText = strObjectivesReturnText .. strOutput .. "\n"
			end
		end
	end
	strQuestReturnText = strtrim(strQuestReturnText)
	strObjectivesReturnText = strtrim(strObjectivesReturnText)
	return strQuestReturnText, strObjectivesReturnText
end

function SorhaQuestLog:CheckQuestStateChange()
	local blnQuestNotFound = false
	local blnObjectiveCompleted = false
	local blnQuestCompleted = false
	local strQuestActionedTitle = ""
	local dicOldQuests = {}
	local dicCurQuests = {}	
	local blnObjectiveAlreadyOutput = false
	local dblPercent = nil
	local r, g, b = nil
	local strMessage = nil
	
	
	-- Compile list of old quests
	for k, ZoneInstance in pairs(oldQuestInfo["Zones"]) do -- Get list of old quests
		for k2, QuestInstance in pairs(ZoneInstance["Quests"]) do
			tinsert(dicOldQuests, QuestInstance)
		end
	end	
	
	-- Compile list of new quests
	for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do -- Get list of current quests
		for k2, QuestInstance in pairs(ZoneInstance["Quests"]) do
			tinsert(dicCurQuests, QuestInstance)
		end
	end
	
	-- For each quest in the old compare it to each quest in the new.
		-- When they match:
			-- Check each old objective against each new objective to check for status changes
			-- Check old quests status against new quest status
	for oldQuestKey, oldQuestInstance in pairs(dicOldQuests) do -- For each quest in the old log
		for curQuestKey, curQuestInstance in pairs(dicCurQuests) do -- For each quest in the new log
			if (oldQuestInstance["ID"] == curQuestInstance["ID"]) then -- If they are the same quest
				
				-- Objective State stuff
				for oldObjectiveKey, oldObjectiveInstance in pairs(oldQuestInstance["ObjectiveList"]) do -- For each of the old quests objectives
					for curObjectiveKey, curObjectiveInstance in pairs(curQuestInstance["ObjectiveList"]) do -- For each of the new quests objectives
						
						blnObjectiveAlreadyOutput = false
						if (oldObjectiveInstance["ID"] == curObjectiveInstance["ID"]) then -- If its the same objective
							if (oldObjectiveInstance["Got"] ~= curObjectiveInstance["Got"] and tonumber(oldObjectiveInstance["Got"]) < tonumber(curObjectiveInstance["Got"])) then -- If the objective has changed and has a ?/? format
								-- If LibSink Objectives on: Set a flag so the (Done) message won't output and output current status
								if (db.Notifications.LibSinkObjectiveNotifications == true) then
									blnObjectiveAlreadyOutput = true
									
									if (curObjectiveInstance["Got"] ~= "" and curObjectiveInstance["Need"] ~= "") then
										-- Create message, add quest name if enabled
										if (db.Notifications.DisplayQuestOnObjectiveNotifications == true) then
											strMessage = format("(%s) %s : %s / %s", curQuestInstance["Title"], curObjectiveInstance["Text"], curObjectiveInstance["Got"], curObjectiveInstance["Need"])
										else
											strMessage = format("%s : %s / %s", curObjectiveInstance["Text"], curObjectiveInstance["Got"], curObjectiveInstance["Need"])
										end
										
										-- Account for stupid quests staying in the log a second once complete
										if not(oldObjectiveInstance["Status"] == 1 and curObjectiveInstance["Status"] == nil) then
											if (db.Notifications.LibSinkColourSetting == "Custom") then
												self:Pour(strMessage, db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b)
											else
												dblPercent = tonumber(curObjectiveInstance["Got"])/tonumber(curObjectiveInstance["Need"])
												r, g, b = SorhaQuestLog:GetCompletionColourRGB(dblPercent)
												self:Pour(strMessage, r, g, b)
											end
										end
									end
									
								end
							end

							if (oldObjectiveInstance["Status"] == nil and curObjectiveInstance["Status"] == 1) then -- If objective has completed since last test
								
								-- If there has not already been a ?/? message output for this objective and LibSink objectives on
								if (blnObjectiveAlreadyOutput == false and db.Notifications.LibSinkObjectiveNotifications == true) then 
									-- Create message, add quest name if enabled
									if (db.Notifications.DisplayQuestOnObjectiveNotifications == true) then
										strMessage = format("(%s) %s ", curQuestInstance["Title"], curObjectiveInstance["Text"]) .. L["(Complete)"]
									else
										strMessage = format("%s ", curObjectiveInstance["Text"]) .. L["(Complete)"]
									end
									
									
									if (db.Notifications.LibSinkColourSetting == "Custom") then
										self:Pour(strMessage, db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b)
									else
										self:Pour(strMessage, db.Colours.ObjectiveDoneColour.r, db.Colours.ObjectiveDoneColour.g, db.Colours.ObjectiveDoneColour.b)
									end
								end
								blnObjectiveCompleted = true
							end
						end
					end
				end
				
				-- Quest State stuff
				if (oldQuestInstance["IsComplete"] == nil and curQuestInstance["IsComplete"] == 1) then -- Has quest been completed since last
					
					-- If enabled display complete message
					if (db.Notifications.ShowQuestCompletesAndFails) then
						self:Pour(L["Quest completed: "] .. curQuestInstance["Title"], db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b)
					end
					
					-- Set flag to make sound play
					blnQuestCompleted = true
				elseif (oldQuestInstance["IsComplete"] == nil and curQuestInstance["IsComplete"] == -1) then -- Has quest been failed since last
					
					-- If enabled display failed message
					if (db.Notifications.ShowQuestCompletesAndFails) then
						self:Pour(L["Quest failed: "] .. curQuestInstance["Title"], db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b)
					end
				end
			end
		end	
	end
	
	if (blnQuestCompleted == true) then
		if ((GetTime() - intTimeOfLastSound) > 1 and db.Notifications.QuestDoneSound ~= "None") then
			PlaySoundFile(LSM:Fetch("sound", db.Notifications.QuestDoneSound))
			intTimeOfLastSound = GetTime()
		end
	elseif (blnObjectiveCompleted == true) then
		if ((GetTime() - intTimeOfLastSound) > 1 and db.Notifications.ObjectiveDoneSound ~= "None") then
			PlaySoundFile(LSM:Fetch("sound", db.Notifications.ObjectiveDoneSound))
			intTimeOfLastSound = GetTime()
		end
	end
end

function SorhaQuestLog:GetQuestLogInformation()
	local intNoEntries, intNoQuests = GetNumQuestLogEntries();
	local i = 1
	local intEntrys = 0
	local blnIsZonesWithTrackedQuests = false
	local QuestLogInfo = {["NoEntrys"] = 0, ["NoQuests"] = intNoQuests, ["Zones"] = {}, ["NumCollapsedZones"] = 0, ["IsZonesWithTrackedQuests"] = false,}
	local ZoneEntry = nil
	local QuestEntry = {}
	local ObjectiveEntry = {}
	local ItemEntry = {}
	
	local intNumCollapsedZones = 0
	local intNumHidden = 0
	local intNumQuestsInZone = 0
	local intCurrentSelectedIndex = GetQuestLogSelection()
	
	-- For each entry in the quest log
	for i = 1, intNoEntries, 1 do
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if (questTitle == nil) then
			questTitle = L["Unknown"]
		end

		if (isHeader) then
			-- If no zone data exists create default
			if (dbChar.ZoneIsAllHiddenQuests[questTitle] == nil) then
				dbChar.ZoneIsAllHiddenQuests[questTitle] = false
			end
			
			-- Get zone collapsed state and save to keep log state after login
			if (isCollapsed == nil) then
				dbChar.ZoneIsCollapsed[questTitle] = false
			else
				dbChar.ZoneIsCollapsed[questTitle] = true
			end
			
			
			-- Create new zone entry
			intEntrys = intEntrys + 1
			if not(ZoneEntry == nil) then
				ZoneEntry["NumHidden"] = intNumHidden
				ZoneEntry["NumQuests"] = intNumQuestsInZone
				if (intNumQuestsInZone > 0) then
					if (intNumHidden == intNumQuestsInZone) then
						dbChar.ZoneIsAllHiddenQuests[ZoneEntry["Title"]] = true
					else
						dbChar.ZoneIsAllHiddenQuests[ZoneEntry["Title"]] = false
					end
				end
				if (dbChar.ZoneIsAllHiddenQuests[questTitle] == false) then
					blnIsZonesWithTrackedQuests = true
				end
				
				tinsert(QuestLogInfo["Zones"], ZoneEntry)
				intNumHidden = 0
				intNumQuestsInZone = 0
			end
			if (isCollapsed == 1) then
				intNumCollapsedZones = intNumCollapsedZones + 1
			end
			ZoneEntry = {["Title"] = questTitle, ["Status"] = isCollapsed, ["LogPosition"] = i, ["ID"] = questID, ["HasHidden"] = false, ["NumHidden"] = intNumHidden, ["NumQuests"] = intNumQuestsInZone, ["Quests"] = {}}
		else
			-- Create new zone
			intNumQuestsInZone = intNumQuestsInZone + 1
			intEntrys = intEntrys + 1
			
			QuestEntry = {["Title"] = questTitle, ["Level"] = level, ["Tag"] = questTag, ["IsHidden"] = false, ["SuggestedGroup"] = suggestedGroup, ["IsComplete"] = isComplete, ["IsDaily"] = isDaily, ["LogPosition"] = i, ["ID"] = questID, ["QuestItem"] = nil, ["CompletionLevel"] = 0, ["ObjectiveList"] = {}, ["ObjectiveDescription"] = ""}

			-- Quest item
			local strLink, strItem, intCharges = GetQuestLogSpecialItemInfo(i);
			if (strItem) then
				local strID =  strLink:match("Hitem:(%d+):")
				ItemEntry = {["Link"] = strLink, ["Item"] = strItem, ["Charges"] = intCharges, ["ID"] = strID}
				QuestEntry["QuestItem"] = ItemEntry
			end
			
			-- Get quest text and objective count
			SelectQuestLogEntry(i);
			local strQuestDescription, strQuestObjectiveDescription = GetQuestLogQuestText();
			local intNoOfObjectives = GetNumQuestLeaderBoards(i)
			
			-- Auto hide completed/goto quests
			if (db.ZonesAndQuests.HideCompletedQuests == true) then
				if (isComplete == 1 or intNoOfObjectives == 0) then
					RemoveQuestWatch(i)
				end	
			end

			local isWatched = IsQuestWatched(i)
			
			-- Hidden quest settings
			if (isWatched == nil) then
				intNumHidden = intNumHidden + 1
				QuestEntry["IsHidden"] = true
				ZoneEntry["HasHidden"] = true
			else
				blnIsZonesWithTrackedQuests = true
			end

			local dblQuestCompletionPercent = 0
			local intNumUsedObjectives = 0
			
			QuestEntry["ObjectiveDescription"] = strQuestObjectiveDescription
			
			-- Objectives
			for k = 1, intNoOfObjectives, 1 do
				local leaderboardTxt, itemType, isDone = GetQuestLogLeaderBoard(k,i);

				if not(leaderboardTxt == nil) then
					intNumUsedObjectives = intNumUsedObjectives + 1
					local strObjText = ""
					local strGot = ""
					local strGetTo = ""
					local intCompletion = 0
					local intGot = 0
					local intNeeded = 1
				
					-- Change things based on objective type
					if (itemType == nil) then
						strObjText = leaderboardTxt
						
					elseif (itemType == "event") then
						strObjText = leaderboardTxt
						
					elseif (itemType == "log") then
						strObjText = leaderboardTxt		
						
					elseif (itemType == "spell") then
						strObjText = leaderboardTxt	
						if (isDone == 1) then
							strGot = "1"
						else
							strGot = "0"
						end
						strGetTo = "1"
						
					elseif (itemType == "reputation") then
						local y, z, strObjDesc, strRepHave, strRepNeeded = string.find(leaderboardTxt, "(.*):%s*(.*)%s*/%s*(.*)");
						if (strObjDesc == nil) then strObjText = "" else strObjText = strObjDesc end
						if (strRepHave == nil) then strGot = "" else strGot = strtrim(strRepHave) end
						if (strRepNeeded == nil) then strGetTo = "" else strGetTo = strtrim(strRepNeeded) end
						
						for k, RepInstance in pairs(dicRepLevels) do
							if (strGot == RepInstance["MTitle"] or strGot == RepInstance["FTitle"]) then
								intGot = RepInstance["Value"]
							end
							if (strGetTo == RepInstance["MTitle"] or strGetTo == RepInstance["FTitle"]) then
								intNeeded = RepInstance["Value"]
							end
						end
						intCompletion = intGot / intNeeded
					
					else
						local y, z, strObjDesc, intNumGot, intNumNeeded = string.find(leaderboardTxt, "(.*):%s*([%d]+)%s*/%s*([%d]+)");
						if (strObjDesc == nil) then strObjText = "" else strObjText = strObjDesc end
						if (intNumGot == nil) then strGot = 2 else strGot = intNumGot end
						if (intNumNeeded == nil) then strGetTo = 1 else strGetTo = intNumNeeded end
						intCompletion = strGot / strGetTo
					end
		
					if (isDone) then
						intCompletion = 1
					end
					dblQuestCompletionPercent = dblQuestCompletionPercent + intCompletion
					ObjectiveEntry = {["Text"] = strObjText, ["Got"] = strGot, ["Need"] = strGetTo, ["CompletionLevel"] = intCompletion, ["Type"] = itemType, ["Status"] = isDone, ["ID"] = k}
					tinsert(QuestEntry["ObjectiveList"], ObjectiveEntry)
				end
			end
			if (intNoOfObjectives < 1) then
				dblQuestCompletionPercent = 1
			else
				dblQuestCompletionPercent = dblQuestCompletionPercent / intNumUsedObjectives
			end
			QuestEntry["CompletionLevel"] = dblQuestCompletionPercent
			tinsert(ZoneEntry["Quests"], QuestEntry)
		end
	end
	if (ZoneEntry ~= nil) then
		ZoneEntry["NumHidden"] = intNumHidden
		ZoneEntry["NumQuests"] = intNumQuestsInZone
		if (ZoneEntry["NumQuests"] > 0) then
			if (intNumHidden == intNumQuestsInZone) then
				dbChar.ZoneIsAllHiddenQuests[ZoneEntry["Title"]] = true
			else
				dbChar.ZoneIsAllHiddenQuests[ZoneEntry["Title"]] = false
			end
			
			if (dbChar.ZoneIsAllHiddenQuests[questTitle] == false) then
				blnIsZonesWithTrackedQuests = true
			end
		end
		tinsert(QuestLogInfo["Zones"], ZoneEntry)
	end
	QuestLogInfo["NoEntrys"] = intEntrys
	QuestLogInfo["NumCollapsedZones"] = intNumCollapsedZones
	QuestLogInfo["IsZonesWithTrackedQuests"] = blnIsZonesWithTrackedQuests
	
	-- Reset selected entry to what it was before getting data
	SelectQuestLogEntry(intCurrentSelectedIndex);
	
	return QuestLogInfo
end


-- Remote Quest functions
function RemoteQuestButton_OnUpdate(frame, timestep)
	local height = 72;
	local scrollStart = 65;
	local scrollEnd = -9;
	local offset = 0
	
	-- The first pop-up needs to include the WATCHFRAME_TYPE_OFFSET in the animation
	if (frame.index == 1) then
		height = height + offset;
		scrollEnd = scrollEnd - offset;
	end
	
	frame.totalTime = frame.totalTime+timestep;
	if (frame.totalTime > frame.slideInTime) then
		frame.totalTime = frame.slideInTime;
	end
	
	local scrollPos = scrollEnd;
	if (frame.slideInTime and frame.slideInTime > 0) then
		height = height*(frame.totalTime/frame.slideInTime);
		scrollPos = scrollStart + (scrollEnd-scrollStart)*(frame.totalTime/frame.slideInTime);
	end
	frame:SetHeight(height);
	frame:SetVerticalScroll(floor(scrollPos+0.5));
	if (frame.totalTime >= frame.slideInTime) then
		frame:SetScript("OnUpdate", nil);
		if (blnRemoteQuestsMinionIsUpdating == false) then
			blnRemoteQuestsMinionIsUpdating = true
			SorhaQuestLog:ScheduleTimer("UpdateRemoteQuestsMinion", 0.5)
		end
		frame.ScrollChild.Shine:Show();
		frame.ScrollChild.IconShine:Show();
		frame.ScrollChild.Shine.Flash:Play();
		frame.ScrollChild.IconShine.Flash:Play();
	end
end


-- Achievement information functions
function SorhaQuestLog:GetAchievementText(AchievementID, blnAlwaysReturnAll)
	local strOutputTitle = strAchievementTitleColour
	local strOutputObjectives = strAchievementObjectiveColour
	
	local id, name, points, completed, month, day, year, description, flags, icon, rewardText = GetAchievementInfo(AchievementID)
	strOutputTitle =  strOutputTitle .. name .. "|r"
	
	
	if (tblAchievementTimers[AchievementID] ~= nil) then
		tblAchievementTimers[AchievementID]["TimeLeft"] = tblAchievementTimers[AchievementID]["Duration"] - (GetTime() - tblAchievementTimers[AchievementID]["StartTime"])

		local strTime = SecondsToTime(tblAchievementTimers[AchievementID]["TimeLeft"])
		if (strTime ~= "") then
			strTime = " - " .. strTime
		else
			strTime = " - 0 Sec"
		end
		
		local r,g,b = GetTimerTextColor(tblAchievementTimers[AchievementID]["Duration"], (GetTime() - tblAchievementTimers[AchievementID]["StartTime"]))
		local strTimeColour = format("|c%02X%02X%02X%02X", 255, r * 255, g * 255, b * 255);
		strOutputTitle = strOutputTitle .. strTimeColour .. strTime .. "|r"
	end
		
	local intNumParts = GetAchievementNumCriteria(AchievementID)
	local blnIsQuantity = false
	local intHave = 0
	local intNeed = 0
	local intValue = 0
	local intMax = 0
	local intDisplayedParts = 0

	if (intNumParts > 0) then
		for i = 1, intNumParts, 1 do
			if (blnAlwaysReturnAll == false) then
				if (intDisplayedParts >= db.AchievementMinion.MaxTasksEachAchievement and db.AchievementMinion.MaxTasksEachAchievement > 0) then
					strOutputObjectives = strOutputObjectives .. ".....\n"
					break
				end
			end
			
			local partdescription, atype, completed, quantity, requiredQuantity, characterName, flags, assetID, quantityString, criteriaID = GetAchievementCriteriaInfo(AchievementID, i)
			if (completed == false) then
				intDisplayedParts = intDisplayedParts + 1
				if string.match(quantityString, ".+/.+") then
					intHave, intNeed = string.match(quantityString, "(.+)/(.+)")
					string.trim(intNeed)
					
					intValue = quantity
					intMax = requiredQuantity		

					if ( bit.band(flags, EVALUATION_TREE_FLAG_PROGRESS_BAR) == EVALUATION_TREE_FLAG_PROGRESS_BAR ) then
						blnIsQuantity = true
					end

					if (blnIsQuantity == true and db.AchievementMinion.UseStatusBars == true) then
						strOutputObjectives = strOutputObjectives .. "- " .. description .. "\n"
					else
						strOutputObjectives = strOutputObjectives .. "- " .. intHave .. "/" .. intNeed .. "\n"
					end
				else
					strOutputObjectives = strOutputObjectives .. "- " .. partdescription .. "\n"
				end
			end
		end
	else
		strOutputObjectives = strOutputObjectives .. "- " .. description
	end
	strOutputObjectives = string.trim(strOutputObjectives)
	strOutputObjectives = strOutputObjectives .. "|r"
	
	return strOutputTitle, strOutputObjectives, blnIsQuantity, intHave, intNeed, intValue, intMax
end

function GetTimerTextColor(duration, elapsed) -- Thankyou blizzard >.>
	local START_PERCENTAGE_YELLOW = .66
	local START_PERCENTAGE_RED = .33
	
	local percentageLeft = 1 - ( elapsed / duration )
	if ( percentageLeft > START_PERCENTAGE_YELLOW ) then
		return 1, 1, 1	
	elseif ( percentageLeft > START_PERCENTAGE_RED ) then -- Start fading to yellow by eliminating blue
		local blueOffset = (percentageLeft - START_PERCENTAGE_RED) / (START_PERCENTAGE_YELLOW - START_PERCENTAGE_RED);
		return 1, 1, blueOffset;
	else
		local greenOffset = percentageLeft / START_PERCENTAGE_RED; -- Fade to red by eliminating green
		return 1, greenOffset, 0;
	end
end


-- Colour functions
function SorhaQuestLog:doColourChange() --Changes the addons colours on the fly as changed
	self:SetupColours()
	if (db.Main.HideSorhaQuestLog == false and db.Main.HideAll == false) then
		if (blnIsUpdating == false) then
			blnIsUpdating = true
			self:ScheduleTimer("UpdateQuestMinion", 0.1)
		end
	end
	if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
		if (blnAchievementMinionIsUpdating == false) then
			blnAchievementMinionIsUpdating = true
			self:ScheduleTimer("UpdateAchievementMinion", 0.1)
		end
	end	
	if (db.Main.ShowRemoteQuestsMinionFrame == true and db.Main.HideAll == false) then
		if (blnRemoteQuestsMinionIsUpdating == false) then
			blnRemoteQuestsMinionIsUpdating = true
			self:ScheduleTimer("UpdateRemoteQuestsMinion", 0.1)
		end
	end

end

function SorhaQuestLog:SetupColours() -- Sets the colour strings from profile settings
	strInfoColour = format("|c%02X%02X%02X%02X", 255, db.Colours.InfoColour.r * 255, db.Colours.InfoColour.g * 255, db.Colours.InfoColour.b * 255);
	strQuestMinionTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestMinionTitleColour.r * 255, db.Colours.QuestMinionTitleColour.g * 255, db.Colours.QuestMinionTitleColour.b * 255);
	strAchievementMinionTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.AchievementMinionTitleColour.r * 255, db.Colours.AchievementMinionTitleColour.g * 255, db.Colours.AchievementMinionTitleColour.b * 255);
	strHeaderColour = format("|c%02X%02X%02X%02X", 255, db.Colours.HeaderColour.r * 255, db.Colours.HeaderColour.g * 255, db.Colours.HeaderColour.b * 255);	
	strQuestStatusFailed = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestStatusFailedColour.r * 255, db.Colours.QuestStatusFailedColour.g * 255, db.Colours.QuestStatusFailedColour.b * 255);
	strQuestStatusDone = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestStatusDoneColour.r * 255, db.Colours.QuestStatusDoneColour.g * 255, db.Colours.QuestStatusDoneColour.b * 255);
	strQuestStatusGoto = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestStatusGotoColour.r * 255, db.Colours.QuestStatusGotoColour.g * 255, db.Colours.QuestStatusGotoColour.b * 255);
	strQuestLevelColour = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestLevelColour.r * 255, db.Colours.QuestLevelColour.g * 255, db.Colours.QuestLevelColour.b * 255);
	strQuestTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestTitleColour.r * 255, db.Colours.QuestTitleColour.g * 255, db.Colours.QuestTitleColour.b * 255);
	strObjectiveTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveTitleColour.r * 255, db.Colours.ObjectiveTitleColour.g * 255, db.Colours.ObjectiveTitleColour.b * 255);
	strObjectiveStatusColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveStatusColour.r * 255, db.Colours.ObjectiveStatusColour.g * 255, db.Colours.ObjectiveStatusColour.b * 255);
	strObjective00to24Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective00PlusColour.r * 255, db.Colours.Objective00PlusColour.g * 255, db.Colours.Objective00PlusColour.b * 255);
	strObjective25to49Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective25PlusColour.r * 255, db.Colours.Objective25PlusColour.g * 255, db.Colours.Objective25PlusColour.b * 255);
	strObjective50to74Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective50PlusColour.r * 255, db.Colours.Objective50PlusColour.g * 255, db.Colours.Objective50PlusColour.b * 255);
	strObjective75to99Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective75PlusColour.r * 255, db.Colours.Objective75PlusColour.g * 255, db.Colours.Objective75PlusColour.b * 255);
	strObjective100Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveDoneColour.r * 255, db.Colours.ObjectiveDoneColour.g * 255, db.Colours.ObjectiveDoneColour.b * 255);
	strObjectiveDescriptionColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveDescColour.r * 255, db.Colours.ObjectiveDescColour.g * 255, db.Colours.ObjectiveDescColour.b * 255);
	strAchievementTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.AchievementTitleColour.r * 255, db.Colours.AchievementTitleColour.g * 255, db.Colours.AchievementTitleColour.b * 255);
	strAchievementObjectiveColour = format("|c%02X%02X%02X%02X", 255, db.Colours.AchievementObjectiveColour.r * 255, db.Colours.AchievementObjectiveColour.g * 255, db.Colours.AchievementObjectiveColour.b * 255);
end

function SorhaQuestLog:GetCompletionColourString(dblPercent)
	if (dblPercent < 0.25) then
		return strObjective00to24Colour
	elseif (dblPercent >= 0.25 and dblPercent < 0.50) then
		return strObjective25to49Colour
	elseif (dblPercent >= 0.50 and dblPercent < 0.75) then
		return strObjective50to74Colour
	elseif (dblPercent >= 0.75 and dblPercent < 1) then
		return strObjective75to99Colour
	else
		return strObjective100Colour
	end
end

function SorhaQuestLog:GetCompletionColourRGB(dblPercent)
	if (dblPercent < 0.25) then
		return db.Colours.Objective00PlusColour.r, db.Colours.Objective00PlusColour.g, db.Colours.Objective00PlusColour.b
	elseif (dblPercent >= 0.25 and dblPercent < 0.50) then
		return db.Colours.Objective25PlusColour.r, db.Colours.Objective25PlusColour.g, db.Colours.Objective25PlusColour.b
	elseif (dblPercent >= 0.50 and dblPercent < 0.75) then
		return db.Colours.Objective50PlusColour.r, db.Colours.Objective50PlusColour.g, db.Colours.Objective50PlusColour.b
	elseif (dblPercent >= 0.75 and dblPercent < 1) then
		return db.Colours.Objective75PlusColour.r, db.Colours.Objective75PlusColour.g, db.Colours.Objective75PlusColour.b
	else
		return db.Colours.ObjectiveDoneColour.r, db.Colours.ObjectiveDoneColour.g, db.Colours.ObjectiveDoneColour.b
	end
end


-- Anchor Functions
function SorhaQuestLog:doQuestMinionAnchorUpdate(blnMoveAnchors) -- Updates backgrounds and visibility of quest minion anchor frame
	-- Quest Minion
	if (blnQuestMinionInitialized == false) then
		if (db.Main.HideSorhaQuestLog == false and db.Main.HideAll == false) then
			self:CreateQuestMinionLayout()
		end
	end	
	
	if (blnQuestMinionInitialized == true) then
		-- Enable/Disable movement
		if (db.Minion.MinionWindowLocked == false) then
			fraQuestMinionAnchor:EnableMouse(true)
		else
			fraQuestMinionAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Quest Minion
		if (db.Main.HideSorhaQuestLog == true or db.Main.HideAll == true) then
			fraQuestMinionAnchor:Hide()
		else
			fraQuestMinionAnchor:Show()
			if (db.Main.ShowAnchors == true and db.Minion.MinionWindowLocked == false) then
				fraQuestMinionAnchor:SetBackdropColor(0, 1, 0, 1)
			else
				fraQuestMinionAnchor:SetBackdropColor(0, 1, 0, 0)
			end
			
			if (blnIsUpdating == false) then
				blnIsUpdating = true
				self:ScheduleTimer("UpdateQuestMinion", 0.25)
			end
		end
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraQuestMinionAnchor:ClearAllPoints()
			fraQuestMinionAnchor:SetPoint(db.Minion.MinionWindowLocation.Point, UIParent, db.Minion.MinionWindowLocation.RelativePoint, db.Minion.MinionWindowLocation.X, db.Minion.MinionWindowLocation.Y);
		end	
	end
end

function SorhaQuestLog:doQuestTimersAnchorUpdate(blnMoveAnchors) -- Updates backgrounds and visibility of quest timers anchor frame
	-- Quest Timers
	if (blnTimerInitialized == false) then
		if (db.Main.ShowQuestTimerFrame == true and db.Main.HideAll == false) then
			self:CreateQuestTimerLayout()
		end
	end	
	
	if (blnTimerInitialized == true) then
		-- Enable/Disable movement	
		if (db.Timer.TimerWindowLocked == false) then
			fraQuestTimerAnchor:EnableMouse(true)
		else
			fraQuestTimerAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Quest Minion
		if (db.Main.ShowQuestTimerFrame == true and db.Main.HideAll == false) then
			fraQuestTimerAnchor:Show()
			if (db.Main.ShowAnchors == true and db.Timer.TimerWindowLocked == false) then
				fraQuestTimerAnchor:SetBackdropColor(0, 1, 0, 1)
			else
				fraQuestTimerAnchor:SetBackdropColor(0, 1, 0, 0)
			end
		else
			fraQuestTimerAnchor:Hide()
		end
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraQuestTimerAnchor:ClearAllPoints()
			fraQuestTimerAnchor:SetPoint(db.Timer.TimerWindowLocation.Point, UIParent, db.Timer.TimerWindowLocation.RelativePoint, db.Timer.TimerWindowLocation.X, db.Timer.TimerWindowLocation.Y);
		end
	end
end

function SorhaQuestLog:doRemoteQuestsMinionAnchorUpdate(blnMoveAnchors) -- Updates backgrounds and visibility of remote quests minion anchor frame
	-- Remote Quests
	if (blnRemoteQuestsMinionInitialized == false) then
		if (db.Main.ShowRemoteQuestsMinionFrame == true and db.Main.HideAll == false) then
			self:CreateRemoteQuestsLayout()
		end
	end	
	
	if (blnRemoteQuestsMinionInitialized == true) then
		-- Enable/Disable movement	
		if (db.RemoteQuests.MinionLocked == false) then
			fraRemoteQuestsAnchor:EnableMouse(true)
		else
			fraRemoteQuestsAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Quest Minion
		if (db.Main.ShowRemoteQuestsMinionFrame == true and db.Main.HideAll == false) then
			fraRemoteQuestsAnchor:Show()
			if (db.Main.ShowAnchors == true and db.RemoteQuests.MinionLocked == false) then
				fraRemoteQuestsAnchor:SetBackdropColor(0, 1, 0, 1)
			else
				fraRemoteQuestsAnchor:SetBackdropColor(0, 1, 0, 0)
			end
			
			if (blnRemoteQuestsMinionIsUpdating == false) then
				self:UpdateRemoteQuestsMinion()
			end
		else
			fraRemoteQuestsAnchor:Hide()
		end
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraRemoteQuestsAnchor:ClearAllPoints()
			fraRemoteQuestsAnchor:SetPoint(db.RemoteQuests.MinionLocation.Point, UIParent, db.RemoteQuests.MinionLocation.RelativePoint, db.RemoteQuests.MinionLocation.X, db.RemoteQuests.MinionLocation.Y);
		end
	end
end

function SorhaQuestLog:doAchievementMinionAnchorUpdate(blnMoveAnchors) -- Updates backgrounds and visibility of quest minion anchor frame
	-- Achievement Minion
	if (blnAchievementMinionInitialized == false) then
		if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
			self:CreateAchievementMinionLayout()
		end
	end
	
	if (blnAchievementMinionInitialized == true) then
		-- Enable/Disable movement	
		if (db.AchievementMinion.MinionLocked == false) then
			fraAchievementMinionAnchor:EnableMouse(true)
		else
			fraAchievementMinionAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Minion
		if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
			fraAchievementMinionAnchor:Show()
			if (db.Main.ShowAnchors == true and db.AchievementMinion.MinionLocked == false) then
				fraAchievementMinionAnchor:SetBackdropColor(0, 1, 0, 1)
			else
				fraAchievementMinionAnchor:SetBackdropColor(0, 1, 0, 0)
			end
			
			if (blnAchievementMinionIsUpdating == false) then
				self:UpdateAchievementMinion()
			end
		else
			fraAchievementMinionAnchor:Hide()
		end
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraAchievementMinionAnchor:ClearAllPoints()
			fraAchievementMinionAnchor:SetPoint(db.AchievementMinion.MinionLocation.Point, UIParent, db.AchievementMinion.MinionLocation.RelativePoint, db.AchievementMinion.MinionLocation.X, db.AchievementMinion.MinionLocation.Y);
		end
	end
end

function SorhaQuestLog:doAnchorUpdates(blnMoveAnchors) -- Updates backgrounds and visibility of all anchor frames
	self:doQuestMinionAnchorUpdate(blnMoveAnchors)
	self:doQuestTimersAnchorUpdate(blnMoveAnchors)
	self:doRemoteQuestsMinionAnchorUpdate(blnMoveAnchors)
	self:doAchievementMinionAnchorUpdate(blnMoveAnchors)
end


-- Misc functions
function SorhaQuestLog:doHiddenQuestsUpdate()
	-- Show/Hide hidden quests button and move quest count text accordingly
	if (blnQuestMinionInitialized == true) then
		if (db.ZonesAndQuests.AllowHiddenQuests == true) then
			fraQuestMinionAnchor.buttonShowHidden:Show()	
			fraQuestMinionAnchor.buttonShowHiddenHover:Show()
			fraQuestMinionAnchor.objFontString:ClearAllPoints()
			fraQuestMinionAnchor.objFontString:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 16, 0);
			if (dbChar.ZonesAndQuests.ShowAllQuests == true) then
				fraQuestMinionAnchor.buttonShowHidden:SetBackdropColor(0, 0.3, 1, 1)
			else
				fraQuestMinionAnchor.buttonShowHidden:SetBackdropColor(0.5, 0.5, 0.5, 1)
			end
		else
			fraQuestMinionAnchor.buttonShowHidden:Hide()	
			fraQuestMinionAnchor.buttonShowHiddenHover:Hide()
			fraQuestMinionAnchor.objFontString:ClearAllPoints()
			fraQuestMinionAnchor.objFontString:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, 0);
		end		
	end
end

function SorhaQuestLog:DisplayRightClickMenu(objButton)
	local objMenu = CreateFrame("Frame", "SorhaQuestLogMenuThing")
	local intLevel = 1
	local info = {}
	
	objMenu.displayMode = "MENU"
	objMenu.initialize = function(self, intLevel)
		if not intLevel then return end
		wipe(info)
		if intLevel == 1 then
			-- Create the title of the menu
			info.isTitle = 1
			info.text = L["Show/Hide Quests"]
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, intLevel)
			
			local intCurrentButton = 0
			local curZone = nil
			
			-- Get zone button belongs to
			for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do
				if (ZoneInstance["LogPosition"] == objButton.LogPosition) then
					curZone = ZoneInstance
				end
			end

			-- Show/Hide buttons for each quest
			for k2, QuestInstance in pairs(curZone["Quests"]) do
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = nil
				info.text = QuestInstance["Title"]
				info.func = function()
					if (IsQuestWatched(QuestInstance["LogPosition"]) == nil) then
						if (GetNumQuestWatches() >= 25) then
							UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, 25), 1.0, 0.1, 0.1, 1.0);
						else
							AddQuestWatch(QuestInstance["LogPosition"])
						end
					else
						RemoveQuestWatch(QuestInstance["LogPosition"])
					end
					QuestLog_Update();
					SorhaQuestLog:UpdateQuestMinion()
				end
				info.checked = IsQuestWatched(QuestInstance["LogPosition"])
				UIDropDownMenu_AddButton(info, intLevel)
			end
	
			-- Hide all button if not all hidden
			if not(curZone["NumHidden"] == curZone["NumQuests"])then
				info.text = L["Hide All"]
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = 1
				info.func = function()
					for k2, QuestInstance in pairs(curZone["Quests"]) do
						RemoveQuestWatch(QuestInstance["LogPosition"])
					end
					QuestLog_Update();
					SorhaQuestLog:UpdateQuestMinion()
				end
				UIDropDownMenu_AddButton(info, intLevel)
			end
			
			-- Show all button if not all hidden
			if (curZone["NumHidden"] > 0)then
				info.text = L["Show All"]
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = 1
				info.func = function()
					for k2, QuestInstance in pairs(curZone["Quests"]) do
						if (GetNumQuestWatches() >= 25) then
							UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, 25), 1.0, 0.1, 0.1, 1.0);
							break
						else
							AddQuestWatch(QuestInstance["LogPosition"])
						end
					end
					QuestLog_Update();
					SorhaQuestLog:UpdateQuestMinion()
				end
				UIDropDownMenu_AddButton(info, intLevel)
			end

			-- Close menu item
			info.text = CLOSE
			info.disabled = nil
			info.isTitle = nil
			info.notCheckable = 1
			info.func = function() CloseDropDownMenus() end
			UIDropDownMenu_AddButton(info, intLevel)
		end
	end

	ToggleDropDownMenu(1, nil, objMenu, objButton, 0, 0)
end

function SorhaQuestLog:DisplayAltRightClickMenu(objButton)
	local objMenu = CreateFrame("Frame", "SorhaQuestLogMenuThing")
	local intLevel = 1
	local info = {}
	
	objMenu.displayMode = "MENU"
	objMenu.initialize = function(self, intLevel)
		if not intLevel then return end
		wipe(info)
		if intLevel == 1 then
			-- Create the title of the menu
			info.isTitle = 1
			info.text = L["Expand/Collapse Zones"]
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, intLevel)
			
			-- Collapse/Expand button for each zone
			for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = nil
				info.text = ZoneInstance["Title"]
				info.func = function()
					if (ZoneInstance["Status"] == nil) then
						CollapseQuestHeader(ZoneInstance["LogPosition"])
					else
						ExpandQuestHeader(ZoneInstance["LogPosition"])
					end
				end
				info.checked = not(ZoneInstance["Status"])
				UIDropDownMenu_AddButton(info, intLevel)
			end

			-- Collapse all button if not all hidden
			if not(#curQuestInfo["Zones"] == curQuestInfo["NumCollapsedZones"])then
				info.text = L["Collapse All"]
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = 1
				info.func = function()
					for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do
						CollapseQuestHeader(ZoneInstance["LogPosition"])
					end
				end
				UIDropDownMenu_AddButton(info, intLevel)
			end
			
			-- Expand all button if not all hidden
			if (curQuestInfo["NumCollapsedZones"] > 0)then
				info.text = L["Expand All"]
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = 1
				info.func = function()
					for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do
						ExpandQuestHeader(ZoneInstance["LogPosition"])
					end
				end
				UIDropDownMenu_AddButton(info, intLevel)
			end

			-- Close menu item
			info.text = CLOSE
			info.disabled = nil
			info.isTitle = nil
			info.notCheckable = 1
			info.func = function() CloseDropDownMenus() end
			UIDropDownMenu_AddButton(info, intLevel)
		end
	end

	ToggleDropDownMenu(1, nil, objMenu, objButton, 0, 0)
end

function SorhaQuestLog:CheckBags()
	for k, intBag in pairs(tblBagsToCheck) do
		for intSlot = 1, GetContainerNumSlots(intBag), 1 do 
			local isQuestItem, questId, isActive = GetContainerItemQuestInfo(intBag, intSlot);
			if (questId ~= nil and isActive == false) then
				local intID = GetContainerItemID(intBag, intSlot)
				if (blnFirstBagCheck == true) then
					tinsert(tblHaveQuestItems, intID)
				else
					if not(tContains(tblHaveQuestItems, intID)) then
						tinsert(tblHaveQuestItems, intID)
						local itemName, itemLink, itemRarity , _, _, _, _, _,_, itemTexture, _ = GetItemInfo(intID)
						local _, _, _, hex = GetItemQualityColor(itemRarity)
						
						hex = "|c" .. hex
						
						local strOutput = nil
						if (db.sink20OutputSink == "Channel") then
							strOutput = UnitName("player") .. " " .. L["picked up a quest starting item: "] .. hex .. itemName .. "|r"
							self:Pour(strOutput, db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b,_,_,_,_,_,itemTexture)
						else
							local strItem = ""
							if (db.sink20OutputSink == "ChatFrame") then
								strItem = "|T" .. itemTexture .. ":15|t"
							else
								strItem = "|T" .. itemTexture .. ":20:20:-5|t"
							end
							
							strOutput = L["You picked up a quest starting item: "] .. " " .. strItem .. hex .. itemLink .. "|r"
							self:Pour(strOutput, db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b)
						end
						
						-- Play sound if enabled
						if ((GetTime() - intTimeOfLastSound) > 1 and db.Notifications.QuestItemFoundSound ~= "None") then
							PlaySoundFile(LSM:Fetch("sound", db.Notifications.QuestItemFoundSound))
							intTimeOfLastSound = GetTime()
						end
					end
				end
			end
		end
	end
	wipe(tblBagsToCheck)

	if (blnFirstBagCheck == true) then
		blnFirstBagCheck = false
	end
	blnBagCheckUpdating = false
end

function SorhaQuestLog:RefreshZoneHeadersState()
	blnIgnoreUpdateEvents = true
	local intNoEntries, intNoQuests = GetNumQuestLogEntries();

	for i = intNoEntries, 1, -1 do
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if (dbChar.ZoneIsCollapsed[questTitle] ~= nil) then
			if (dbChar.ZoneIsCollapsed[questTitle] == true) then
				if (isCollapsed == nil) then
					CollapseQuestHeader(i)
				end
			else
				if (isCollapsed == 1) then
					ExpandQuestHeader(i)
				end		
			end
		end
	end
	
	blnIgnoreUpdateEvents = false
end


-- Auto collapsing/expanding zones
function SorhaQuestLog:doHandleZoneChange()
	blnIgnoreUpdateEvents = true
	
	local blnNewZone = not(strZone == GetRealZoneText())
	strZone = GetRealZoneText()
	strSubZone = GetSubZoneText()
	local blnChanged = false
	
	if (db.ZonesAndQuests.CollapseOnLeave == true or db.ZonesAndQuests.ExpandOnEnter == true) then
		self:GetQuestLogInformation()
		local numEntries, numQuests = GetNumQuestLogEntries();	
		for i = numEntries, 1, -1 do
			local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
			if (isHeader) then
				if (questTitle == strZone or questTitle == strSubZone) then
					if (db.ZonesAndQuests.ExpandOnEnter == true and isCollapsed) then
						ExpandQuestHeader(i)
						blnChanged = true
					end
				else		
					if (db.ZonesAndQuests.CollapseOnLeave == true and not(isCollapsed) and blnNewZone) then				
						CollapseQuestHeader(i)
						blnChanged = true
					end				
				end
			end
		end
	end
	
	blnIgnoreUpdateEvents = false
	if (blnQuestMinionInitialized == true and blnChanged == true) then
		self:UpdateQuestMinion()
	end
end


-- Utility functions
function SorhaQuestLog:RemoveObject(objTable, objFind)
	for k, v in pairs(objTable) do
		if (v == objFind) then
			tremove(objTable,k)
			break
		end
	end
end

function SorhaQuestLog:doCreateLooseFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,intAlpha,finherit) -- Returns a frame thats not anchored
	local f = CreateFrame(strType,strName,fraParent,finherit)
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)	
	f:SetAlpha(intAlpha)
	return f  
end 

function SorhaQuestLog:doCreateFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY,intAlpha,finherit) -- Returns an anchored frame
	local f = CreateFrame(strType,strName,fraParent,finherit)
	
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)
	f:SetPoint(strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY)
	f:SetAlpha(intAlpha)
	return f  
end 


-- Visibility functions
function SorhaQuestLog:ShowSorhaQuestLog()
	db.Main.HideAll = false
	self:doAnchorUpdates(false)
end

function SorhaQuestLog:HideSorhaQuestLog()
	db.Main.HideAll = true
	self:doAnchorUpdates(false)
end

function SorhaQuestLog:ToggleSorhaQuestLog()
	if (db.Main.HideAll == true) then
		self:ShowSorhaQuestLog()
	else
		self:HideSorhaQuestLog()
	end
end